@@ -3,10 +3,10 @@
  *  Module    : keymap.h
  *  Author    : J. Faultless, D. Nimmich
  *  Created   : 1999
- *  Updated   : 2024-07-06
+ *  Updated   : 2025-07-25
  *  Notes     :
  *
- * Copyright (c) 1999-2024 Jason Faultless <jason@altarstone.com>
+ * Copyright (c) 1999-2025 Jason Faultless <jason@altarstone.com>
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -126,7 +126,7 @@
 /*
  * Maximum chars (including null byte) needed to print a key name
  * A multibyte character can use up to MB_CUR_MAX chars. But as MB_CUR_MAX
- * can't be used here, use MB_LEN_MAX instead.
+ * can't be used here (avoid VLAs), use MB_LEN_MAX instead.
  * Some values for MB_LEN_MAX:
  * - glibc 2.3.5: 16
  * - gcc 4.0: 1
@@ -135,7 +135,6 @@
  */
 #define MAXKEYLEN 17
 
-/* TODO: permanently move here from tin.h */
 #define ctrl(c)	((c) & 0x1F)
 #define ESC		27
 
@@ -171,6 +170,7 @@ enum defined_functions {
 	CONFIG_RESET_ATTRIB,
 	CONFIG_SCOPE_MENU,
 	CONFIG_SELECT,
+	CONFIG_SERVERRC_MENU,
 	CONFIG_NO_SAVE,
 	CONFIG_TOGGLE_ATTRIB,
 	FEED_ARTICLE,
@@ -295,11 +295,12 @@ enum defined_functions {
 	PAGE_TAG,
 	PAGE_TOGGLE_HEADERS,
 	PAGE_TOGGLE_HIGHLIGHTING,
+	PAGE_TOGGLE_INLINE_DATA,
 	PAGE_TOGGLE_RAW,
 	PAGE_TOGGLE_ROT13,
 	PAGE_TOGGLE_TABS,
 	PAGE_TOGGLE_TEX2ISO,
-	PAGE_TOGGLE_UUE,
+	PAGE_TOGGLE_VERBATIM,
 	PAGE_TOP_THREAD,
 	PAGE_VIEW_ATTACHMENTS,
 	PAGE_VIEW_URL,
@@ -374,7 +375,8 @@ enum defined_functions {
 	THREAD_TOGGLE_SUBJECT_DISPLAY,
 	THREAD_UNDO_SELECTIONS,
 	THREAD_UNTAG,
-	URL_SELECT
+	URL_SELECT,
+	FUNC_MAX
 };
 typedef enum defined_functions t_function;
 
@@ -444,8 +446,8 @@ extern t_function prompt_slk_response(t_function default_func, const struct keyl
 #define PrintFuncKey(buf, func, keys) printascii(buf, (wint_t) func_to_key(func, keys))
 #else
 	extern char *printascii(char *buf, int ch);
-	extern char func_to_key (t_function func, const struct keylist keys);
-	extern t_function key_to_func (const char key, const struct keylist keys);
+	extern char func_to_key(t_function func, const struct keylist keys);
+	extern t_function key_to_func(const char key, const struct keylist keys);
 #define PrintFuncKey(buf, func, keys) printascii(buf, (int) func_to_key(func, keys))
 #endif /* MULTIBYTE_ABLE && !NO_LOCALE */
 #endif /* !KEYMAP_H */
