@@ -3,10 +3,10 @@
  *  Module    : nntplib.h
  *  Author    : I.Lea
  *  Created   : 1991-04-01
- *  Updated   : 2022-12-23
+ *  Updated   : 2025-07-29
  *  Notes     : nntp.h 1.5.11/1.6 with extensions for tin
  *
- * Copyright (c) 1991-2024 Iain Lea <iain@bricbrac.de>
+ * Copyright (c) 1991-2025 Iain Lea <iain@bricbrac.de>
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -173,17 +173,17 @@ enum f_type { OVER_T_ERROR, OVER_T_INT, OVER_T_STRING, OVER_T_FSTRING };
  */
 enum extension_type { NONE, CAPABILITIES, BROKEN };
 
-enum sasl_types {
-	SASL_NONE 		= 0,
-	SASL_PLAIN		= 1 << 0,
-	SASL_CRAM_MD5	= 1 << 1,
-	SASL_DIGEST_MD5	= 1 << 2,
-	SASL_GSSAPI		= 1 << 3,
-	SASL_EXTERNAL	= 1 << 4,
-	SASL_OTP		= 1 << 5,
-	SASL_NTLM		= 1 << 6,
-	SASL_LOGIN		= 1 << 7
+enum sasl_props {
+	SASL_NEED_NONE 				= 0,
+	SASL_NEED_AUTHZID			= 1 << 0,
+	SASL_NEED_AUTHID			= 1 << 1,
+	SASL_NEED_PASSWORD			= 1 << 2,
+	SASL_NEED_SERVICE			= 1 << 3,
+	SASL_NEED_ANONYMOUS_TOKEN	= 1 << 4,
+	SASL_NEED_HOSTNAME			= 1 << 5
+	/* ... */
 };
+
 enum c_algorithms { COMPRESS_NONE, COMPRESS_DEFLATE };
 
 struct t_capabilities {
@@ -205,21 +205,28 @@ struct t_capabilities {
 	t_bool list_distributions:1;	/* LIST DISTRIBUTIONS, "private" extension, RFC 2980 */
 	t_bool list_moderators:1;		/* LIST MODERATORS, "private" extension */
 	t_bool list_counts:1;			/* LIST COUNTS, "private" extension */
+	t_bool listgroup:1;				/* LISTGROUP - old versions of nntpcache (< 2.3.3b1?), leafnode and SurgeNews had a buggy implementation which didn't select the group and would require an extra GROUP cmd. before */
 	t_bool xpat:1;					/* XPAT, "private" extension, RFC 2980 */
 	t_bool hdr:1;					/* HDR: "HDR", "LIST HEADERS" */
 	const char *hdr_cmd;			/* [X]HDR */
 	t_bool over:1;					/* OVER: "OVER", "LIST OVERVIEW.FMT" */
 	t_bool over_msgid:1;			/* OVER: "OVER mid" */
 	const char *over_cmd;			/* [X]OVER */
+	t_bool newgroups:1;				/* NEWGROUPS */
+#if 0
+	/* we don't do NEWNEWS */
 	t_bool newnews:1;				/* NEWNEWS */
+#endif /* 0 */
 	char *implementation;			/* IMPLEMENTATION */
-	t_bool starttls:1;				/* STARTTLS */
+	t_bool starttls:1;				/* STARTTLS (not supported; RFC 7525 3.2, RFC 8143 2) */
 	t_bool authinfo_user:1;			/* AUTHINFO USER/PASS */
 	t_bool authinfo_sasl:1;			/* AUTHINFO SASL */
-	t_bool authinfo_state:1;		/* AUTHINFO not supported in current state */
-	enum sasl_types sasl;			/* SASL_NONE, SASL_PLAIN, SASL_CRAM_MD5, SASL_DIGEST_MD5, SASL_GSSAPI, SASL_EXTERNAL, SASL_OTP, SASL_NTLM, SASL_LOGIN */
+	t_bool authinfo_state:1;		/* AUTHINFO not supported in current state (FALSE==auth allowed) */
+	char *sasl_mechs;				/* SASL mechs of the server which our code supports */
+	char *sasl_mech_used;			/* SASL mech in use */
 	t_bool compress:1;				/* COMPRESS */
 	enum c_algorithms compress_algorithm;	/* COMPRESS_NONE, COMPRESS_DEFLATE */
+	t_distrib_pat *distrib_pats;		/* DISTRIB.PATS linked list */
 #if defined(MAXARTNUM) && defined(USE_LONG_ARTICLE_NUMBERS)
 	t_artnum maxartnum;				/* MAXARTNUM indicating article numbers >=2^31 */
 #endif /* MAXARTNUM && USE_LONG_ARTICLE_NUMBERS */
@@ -228,7 +235,6 @@ struct t_capabilities {
 	t_bool streaming:1;				/* STREAMING: "MODE STREAM", "CHECK", "TAKETHIS" */
 	t_bool ihave:1;					/* IHAVE: "IHAVE" */
 #endif /* 0 */
-	t_bool broken_listgroup:1;		/* LISTGROUP doesn't select newsgroup */
 };
 
 #endif /* !NNTPLIB_H */
