@@ -3,10 +3,10 @@
  *  Module    : tin.h
  *  Author    : I. Lea & R. Skrenta
  *  Created   : 1991-04-01
- *  Updated   : 2024-08-28
+ *  Updated   : 2025-11-17
  *  Notes     : #include files, #defines & struct's
  *
- * Copyright (c) 1997-2024 Iain Lea <iain@bricbrac.de>, Rich Skrenta <skrenta@pbm.com>
+ * Copyright (c) 1997-2026 Iain Lea <iain@bricbrac.de>, Rich Skrenta <skrenta@pbm.com>
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -79,19 +79,27 @@
 #	endif /* !HAVE_SETLOCALE */
 #endif /* !NO_LOCALE */
 
+/* noops */
 #define N_(Str) Str
-
+#define PN_(Singular, Plural) {Singular, Plural}
 #if defined(ENABLE_NLS) && !defined(__BUILD__)
 #	ifdef HAVE_LIBINTL_H
 #		include <libintl.h>
 #	endif /* HAVE_LIBINTL_H */
 #	define _(Text)	gettext(Text)
+/*
+#	ifndef HAVE_NGETTEXT
+#		define P_(Singular, Plural, n) ((n) == 1 ? ((void) (Plural), gettext(Singular)) : ((void) (Singular), gettext(Plural)))
+#	else
+*/
+#	define P_(Singular, Plural, n) ngettext(Singular, Plural, n)
 #else
 #	undef bindtextdomain
 #	define bindtextdomain(Domain, Directory) /* empty */
 #	undef textdomain
 #	define textdomain(Domain) /* empty */
 #	define _(Text) Text
+#	define P_(Singular, Plural, n) ((n) == 1 ? ((void) (Plural), (Singular)) : ((void) (Singular), (Plural)))
 #endif /* ENABLE_NLS && !__BUILD__ */
 
 #ifndef LOCALEDIR
@@ -107,7 +115,7 @@
 #endif /* HAVE_SIGNAL_H */
 
 enum context { cMain, cArt, cAttachment, cAttrib, cConfig, cFilter, cGroup, cInfopager, cPage, cPOSTED, cPost, cPostCancel, cPostFup, cReconnect, cScope, cSelect, cThread, cURL };
-enum icontext { cNone, cGetline, cPromptCONT, cPromptSLK, cPromptYN };
+enum icontext { cNone, cGetline, cPromptCONT, cPromptList, cPromptSLK, cPromptYN };
 enum resizer { cNo, cYes, cRedraw };
 enum rc_state { RC_IGNORE, RC_UPGRADE, RC_DOWNGRADE, RC_ERROR };
 
@@ -118,8 +126,8 @@ enum rc_state { RC_IGNORE, RC_UPGRADE, RC_DOWNGRADE, RC_ERROR };
 #	ifdef HAVE_SYS_ERRNO_H
 #		include <sys/errno.h>
 /*
-	#	else
-	#		error "No errno.h or sys/errno.h found"
+#	else
+#		error "No errno.h or sys/errno.h found"
 */
 #	endif /* HAVE_SYS_ERRNO_H */
 #endif /* HAVE_ERRNO_H */
@@ -190,12 +198,15 @@ enum rc_state { RC_IGNORE, RC_UPGRADE, RC_DOWNGRADE, RC_ERROR };
 #	include <limits.h>
 #endif /* HAVE_LIMITS_H */
 
+#ifdef HAVE_STDINT_H
+#	include <stdint.h>
+#endif /* HAVE_STDINT_H */
+
 #if defined(ENABLE_LONG_ARTICLE_NUMBERS) && !defined(SMALL_MEMORY_MACHINE)
 /*
  * defines and typedefs for 64 bit article numbers
  *
- * TODO: what if !CPP_DOES_CONCAT
- *       add configure check for INT64_MAX, LLONG_MAX, LONG_MAX
+ * TODO: add configure check for INT64_MAX, LLONG_MAX, LONG_MAX
  */
 #	if defined(HAVE_INT_LEAST64_T) && !defined(HAVE_INTTYPES_H)
 #		undef HAVE_INT_LEAST64_T
@@ -207,9 +218,6 @@ enum rc_state { RC_IGNORE, RC_UPGRADE, RC_DOWNGRADE, RC_ERROR };
 #			endif /* HAVE_STRTOLL */
 #		endif /* HAVE_ATOLL || HAVE_ATOQ */
 #	endif /* HAVE_INT_LEAST64_T || HAVE_LONG_LONG || quad_t */
-#	ifdef HAVE_STDINT_H
-#		include <stdint.h>
-#	endif /* HAVE_STDINT_H */
 #endif /* ENABLE_LONG_ARTICLE_NUMBERS && !SMALL_MEMORY_MACHINE */
 #ifdef USE_LONG_ARTICLE_NUMBERS
 #	if defined(HAVE_INT_LEAST64_T) && defined(HAVE_PRIDLEAST64) && defined(HAVE_SCNDLEAST64) && defined(HAVE_INT64_C)
@@ -223,7 +231,11 @@ enum rc_state { RC_IGNORE, RC_UPGRADE, RC_DOWNGRADE, RC_ERROR };
 		typedef long long t_artnum;
 #		define T_ARTNUM_PFMT "lld"
 #		define T_ARTNUM_SFMT "lld"
-#		define T_ARTNUM_CONST(v) v ## LL
+#		ifdef CPP_DOES_CONCAT
+#			define T_ARTNUM_CONST(v) v ## LL
+#		else
+#			define T_ARTNUM_CONST(v) (long long)v
+#		endif /* CPP_DOES_CONCAT */
 #		define ARTNUM_MAX LLONG_MAX
 #	endif /* HAVE_INT_LEAST64_T && HAVE_PRIDLEAST64 && HAVE_SCNDLEAST64 && HAVE_INT64_C */
 #	ifdef HAVE_ATOLL
@@ -236,7 +248,11 @@ enum rc_state { RC_IGNORE, RC_UPGRADE, RC_DOWNGRADE, RC_ERROR };
 	typedef long t_artnum;
 #	define T_ARTNUM_PFMT "ld"
 #	define T_ARTNUM_SFMT "ld"
-#	define T_ARTNUM_CONST(v) v ## L
+#	ifdef CPP_DOES_CONCAT
+#		define T_ARTNUM_CONST(v) v ## L
+#	else
+#		define T_ARTNUM_CONST(v) (long)v
+#	endif /* CPP_DOES_CONCAT */
 #	define ARTNUM_MAX LONG_MAX
 #	define atoartnum atol
 #	define strtoartnum strtol
@@ -478,13 +494,6 @@ enum rc_state { RC_IGNORE, RC_UPGRADE, RC_DOWNGRADE, RC_ERROR };
 #	ifndef NNTP_INEWS
 #		define NNTP_INEWS	1
 #	endif /* !NNTP_INEWS */
-#else
-#	ifdef XHDR_XREF
-#		undef XHDR_XREF
-#	endif /* XHDR_XREF */
-#	ifdef BROKEN_LISTGROUP
-#		undef BROKEN_LISTGROUP
-#	endif /* BROKEN_LISTGROUP */
 #endif /* NNTP_ABLE || NNTP_ONLY || NNTPS_ABLE */
 
 #define FAKE_NNTP_FP		(FILE *) 9999
@@ -596,16 +605,10 @@ enum rc_state { RC_IGNORE, RC_UPGRADE, RC_DOWNGRADE, RC_ERROR };
 #ifndef PATH_METAMAIL	/* only unset if !HAVE_METAMAIL */
 #	define PATH_METAMAIL	"metamail"
 #endif /* !PATH_METAMAIL */
-#define METAMAIL_CMD		PATH_METAMAIL" -e -p -m \"tin\""
+#define METAMAIL_CMD	PATH_METAMAIL" -e -p -m \"tin\""
 
 #define INTERNAL_CMD	"--internal"
 
-#ifdef HAVE_LONG_FILE_NAMES
-#	define INDEX_LOCK	"tin.%.256s.LCK"
-#else
-#	define INDEX_LOCK	"%.10s.LCK"
-#endif /* HAVE_LONG_FILE_NAMES */
-
 /* inews.c:submit_inews() and save.c:save_and_process_art() */
 #define PATHMASTER 	"not-for-mail"
 
@@ -690,17 +693,59 @@ enum rc_state { RC_IGNORE, RC_UPGRADE, RC_DOWNGRADE, RC_ERROR };
  */
 
 /* case sensitive & ^-anchored */
-#define UUBEGIN_REGEX	"begin\\s\\s?[0-7]{3,4}\\s+"
+#define UUBEGIN_REGEX	"begin(?:-encoded)?\\s\\s?[0-7]{3,4}\\s+"
 /* case sensitive & ^-anchored */
 #define UUBODY_REGEX	"(?:`|.[\\x20-\\x60]{1,61})$"
 
-/* case sensitive & ^-anchored */
-#define SHAR_REGEX	"\\#(?:!\\s?(?:/usr)?/bin/sh|\\s?(?i)this\\sis\\sa\\sshell\\sarchive)"
+/* case insensitive && ^-anchored*/
+#define XXBODY_REGEX		"[\\dA-Z+-]{1,62}$"
 
-/* slrn verbatim marks, case sensitive & ^-anchored */
+#if 0 /* we don't have specific xxencode-/base64-code (yet); just document the formats */
+	/* case sensitive & ^-anchored */
+#	define XXBEGIN_REGEX	"begin\\s\\s?[0-7]{3,4}\\s+"
+
+	/* uuencode -m [-e] from gnu-sharutils */
+	/* case sensitive & ^-anchored */
+#	define B64BEGIN_REGEX	"begin-base64(?:-encoded)?\\s\\s?[0-7]{3,4}\\s+"
+#	define B64BODY_REGEX	"(?:[\\da-z/\\+]{1,60})$"
+#	define B64END_REGEX		"={4}"
+#endif /* 0 */
+
+	/* case sensitive & ^-anchored */
+#if 1
+	/*
+	 * fuzzy version for more fine grained error reporting
+	 * the "useles" (?<pcrebug>(.)?) circumvents a bug in pcre1
+	 * not finding the last named capture group
+	 */
+#	define YENCBEGIN_REGEX	"=ybegin(\\spart=(?<y_part>\\S+))?(\\stotal=(?<y_total>\\S+))?(\\sline=(?<y_line>\\S+))?(\\ssize=(?<y_bsize>\\S+))?(\\sname=(?<y_name>.*))?(?<pcrebug>(.)?)$"
+#	define YENCPART_REGEX	"=ypart(\\sbegin=(?<y_pbegin>\\S+))?(\\send=(?<y_pend>\\S+))?.*(?<pcrebug>(.)?)$"
+#	define YENCEND_REGEX	"=yend(\\ssize=(?<y_esize>\\S+))?(\\spart=(?<y_epart>\\S+))?(\\spcrc32=(?<y_epcrc>\\S+))?(\\scrc32=(?<y_ecrc>\\S+))?.*(?<pcrebug>(.)?)$"
+#else
+	/* strict version; just to document the format */
+#	define YENCBEGIN_REGEX	"=ybegin (part=(?<y_part>\\d{1,3}) (total=(?<y_total>\\d{1,3}) )?)?line=\\d{2,3} size=(?<y_bsize>\\d{1,19}) name=(?<y_name>\\S+)$"
+#	define YENCPART_REGEX	"=ypart begin=(?<y_pbegin>\\d{1,19}) end=(?<y_pend>\\d{1,19})$"
+#	define YENCEND_REGEX	"=yend size=(?<y_esize>\\d{1,19})( part=\\d{1,3} pcrc32=(?<y_epcrc>[a-fA-F\\d]{8}))?( crc32=(?<y_ecrc>[a-fA-F\\d]{8}))?$"
+#endif /* 1 */
+
+/* shar archives; case sensitive & ^-anchored */
+#define SHAR_REGEX	"\\#(?:!\\s?(?:/usr)?/bin/sh\\b|\\s?(?i)this\\sis\\sa\\sshell\\sarchive)"
+#define SHAR_END_REGEX	"exit(?:\\s0;?)?(?:\\s*#.*)?$"
+
+/*
+ * LaTeX like umlauts for tex2iso_conv; case sensitive
+ * requires [\sA-Za-z\"'] before "[aou]
+ * require at least one letter before "s and disallow [qxy] after "s
+ */
+#define LATEX_REGEX	"(?<=[\\sA-Za-z\"'])\\\\?\"[AOUaou]|[A-Za-z]\\\\?\"s(?![qxy])"
+
+/* slrn verbatim marks; case sensitive & ^-anchored */
 #define DEFAULT_VERBATIM_BEGIN_REGEX	"#v\\+\\s$"
 #define DEFAULT_VERBATIM_END_REGEX	"#v-\\s$"
 
+/* trn $HIDELINE like feature; never matching re */
+#define NEVER_MATCH_REGEX "(?!)"
+
 /* quoted text from external sources */
 #define DEFAULT_EXTQUOTE_REGEX "^\\|\\s"
 
@@ -715,8 +760,9 @@ enum rc_state { RC_IGNORE, RC_UPGRADE, RC_DOWNGRADE, RC_ERROR };
  * TODO: - split out ftp (only ftp allows username:passwd@, RFC 1738)?
  *       - test IDNA (RFC 3490) case
  *       - adjust to follow RFC 3986 (section 2.3)
+ *       - somehow shorten regex to < 500 chars
  */
-#define URL_REGEX	"\\b(?:https?|ftp|gopher)://(?:[^:@/\\s]*(?::[^:@/\\s]*)?@)?(?:(?:(?:[^\\W_](?:(?:-|[^\\W_]){0,61}(?<!---)[^\\W_])?|xn--[^\\W_](?:-(?!-)|[^\\W_]){1,57}[^\\W_])\\.)+[a-z]{2,18}\\.?|localhost|(?:(?:2[0-4]\\d|25[0-5]|[01]?\\d\\d?)\\.){3}(?:2[0-4]\\d|25[0-5]|[01]?\\d\\d?)|\\[(?:(?:[0-9A-F]{0,4}:){1,7}[0-9A-F]{1,4}|(?:[0-9A-F]{0,4}:){1,3}(?:(?:2[0-4]\\d|25[0-5]|[01]?\\d\\d?)\\.){3}(?:2[0-4]\\d|25[0-5]|[01]?\\d\\d?))\\])(?::\\d+)?(?:[-a-zA-Z0-9()@:%_\\+.~#?&\\/=\\$,]*)"
+#define URL_REGEX	"\\b(?:https?|ftp|gopher)://(?:[^:@/\\s]*(?::[^:@/\\s]*)?@)?(?:(?:(?:[^\\W_](?:(?:-|[^\\W_]){0,61}(?<!---)[^\\W_])?|xn--[^\\W_](?:-(?!-)|[^\\W_]){1,57}[^\\W_])\\.)+[a-z]{2,18}\\.?|localhost|(?:(?:2[0-4]\\d|25[0-5]|[01]?\\d\\d?)\\.){3}(?:2[0-4]\\d|25[0-5]|[01]?\\d\\d?)|\\[(?:(?:(?:[\\dA-F]{0,4}:){1,7}(?:[\\dA-F]{1,4}|:)|(?:::(?:[\\dA-F]{0,4}:){0,6}[\\dA-F]{1,4})|::)|(?:(?:[\\dA-F]{0,4}:){1,5}(?:[\\dA-F]{1,4}|:)|(?:::(?:[\\dA-F]{0,4}:){0,4}[\\dA-F]{1,4})|::)(?:(?:2[0-4]\\d|25[0-5]|[01]?\\d\\d?)\\.){3}(?:2[0-4]\\d|25[0-5]|[01]?\\d\\d?))\\])(?::\\d+)?(?:[a-z\\d()@:%_+.~#?&\\/=\\$,-]*)"
 /*
  * case insensitive
  * TOFO: check against RFC 6068
@@ -733,6 +779,8 @@ enum rc_state { RC_IGNORE, RC_UPGRADE, RC_DOWNGRADE, RC_ERROR };
  * case insensitive
  */
 #	define TELNET_REGEX	"\\btelnet://(?:[^:@/]*(?::[^:@/]*)?@)?(?:(?:[^\\W_](?:(?:-|[^\\W_]){0,61}(?<!---)[^\\W_])?\\.)+[a-z]{2,14}\\.?||localhost|(?:(?:2[0-4]\\d|25[0-5]|[01]?\\d\\d?)\\.){3}(?:2[0-4]\\d|25[0-5]|[01]?\\d\\d?))(?::\\d+)?/?"
+	/* https://github.com/package-url/purl-spec/blob/main/PURL-SPECIFICATION.rst */
+#	define PURL_REGEX	"\\bpkg:/{0,2}[a-zA-Z][a-zA-Z\\d._~-]+/(?:[a-zA-Z\\d._%~-]+/)?[a-zA-Z\\d._~-]+(?:@[a-zA-Z\\d._:%~-]+)?(?:\\?[a-zA-Z\\d._%~+=/&-]+)?(?:#[a-zA-Z\\d._%/~-]+)?\\b"
 #endif /* 0 */
 
 
@@ -756,6 +804,9 @@ enum rc_state { RC_IGNORE, RC_UPGRADE, RC_DOWNGRADE, RC_ERROR };
 #define NEWSGROUPS_FILE	"newsgroups"
 #define MOTD_FILE	"motd"
 #define KEYMAP_FILE	"keymap"
+#define MSGLOG_FILE "msglog"
+#define HELP_PAGE_CACHE_FILE ".help-%d"
+#define CONINFO_CACHE_FILE ".coninfo"
 
 #define SIGDASHES "-- \n"
 
@@ -765,42 +816,48 @@ enum rc_state { RC_IGNORE, RC_UPGRADE, RC_DOWNGRADE, RC_ERROR };
 
 /* Philip Hazel's Perl regular expressions library */
 #ifdef HAVE_LIB_PCRE2
+#	define PCRE2_CODE_UNIT_WIDTH 0
+#	include <pcre2.h>
 
-#define PCRE2_CODE_UNIT_WIDTH 0
-#include <pcre2.h>
+#	define REGEX_CASELESS PCRE2_CASELESS
+#	define REGEX_ANCHORED PCRE2_ANCHORED
+#	define REGEX_NOTEMPTY PCRE2_NOTEMPTY
 
-#define REGEX_CASELESS PCRE2_CASELESS
-#define REGEX_ANCHORED PCRE2_ANCHORED
-#define REGEX_NOTEMPTY PCRE2_NOTEMPTY
-
-#define REGEX_ERROR_NOMATCH PCRE2_ERROR_NOMATCH
-
-#define REGEX_OPTIONS uint32_t
-#define REGEX_NOFFSET uint32_t
-#define REGEX_SIZE size_t
+#	define REGEX_ERROR_NOMATCH PCRE2_ERROR_NOMATCH
 
+#	define REGEX_OPTIONS uint32_t
+#	define REGEX_NOFFSET uint32_t
+#	define REGEX_SIZE size_t
 #else /* HAVE_LIB_PCRE2 */
+#	include <pcre.h>
 
-#include <pcre.h>
+#	define REGEX_CASELESS PCRE_CASELESS
+#	define REGEX_ANCHORED PCRE_ANCHORED
+#	define REGEX_NOTEMPTY PCRE_NOTEMPTY
 
-#define REGEX_CASELESS PCRE_CASELESS
-#define REGEX_ANCHORED PCRE_ANCHORED
-#define REGEX_NOTEMPTY PCRE_NOTEMPTY
-
-#define REGEX_ERROR_NOMATCH PCRE_ERROR_NOMATCH
-
-#define REGEX_OPTIONS int
-#define REGEX_NOFFSET int
-#define REGEX_SIZE int
+#	define REGEX_ERROR_NOMATCH PCRE_ERROR_NOMATCH
 
+#	define REGEX_OPTIONS int
+#	define REGEX_NOFFSET int
+#	define REGEX_SIZE int
 #endif /* HAVE_LIB_PCRE2 */
+#define MATCH_REGEX(x,y,z)	(match_regex_ex(y, (REGEX_SIZE) z, 0, 0, &(x)) >= 0)
 
-#ifdef HAVE_ICONV
+#if defined(HAVE_ICONV) || defined(HAVE_UCNV_OPEN)
 #	define CHARSET_CONVERSION 1
-#	ifdef HAVE_ICONV_H
-#		include <iconv.h>
-#	endif /* HAVE_ICONV_H */
-#endif /* HAVE_ICONV */
+#	ifdef HAVE_ICONV
+#		ifdef HAVE_ICONV_H
+#			include <iconv.h>
+#		endif /* HAVE_ICONV_H */
+#		define CHARSET_CONVERSION_ICONV 1
+#	endif /* HAVE_ICONV */
+#	ifdef HAVE_UCNV_OPEN
+#		ifdef HAVE_UNICODE_UCNV_H
+#			include <unicode/ucnv.h>
+#		endif /* HAVE_UNICODE_UCNV_H */
+#		define CHARSET_CONVERSION_UCNV 1
+#	endif /* HAVE_UCNV_OPEN */
+#endif /* HAVE_ICONV || HAVE_UCNV_OPEN */
 
 #ifdef HAVE_LANGINFO_H
 #	include <langinfo.h>
@@ -815,6 +872,20 @@ enum rc_state { RC_IGNORE, RC_UPGRADE, RC_DOWNGRADE, RC_ERROR };
 #ifndef CODESET
 #	define CODESET ((nl_item) 1)
 #endif /* CODESET */
+#ifndef HAVE_LANGINFO_CODESET
+#	ifndef RADIXCHAR
+#		define RADIXCHAR ((nl_item) 2)
+#	endif /* RADIXCHAR */
+#	ifndef DECIMAL_POINT
+#		define DECIMAL_POINT ((nl_item) 2)
+#	endif /* DECIMAL_POINT */
+#	ifndef NOEXPR
+#		define NOEXPR ((nl_item) 3)
+#	endif /* NOEXPR */
+#	ifndef YESEXPR
+#		define YESEXPR ((nl_item) 4)
+#	endif /* YESEXPR */
+#endif /* HAVE_LANGINFO_CODESET */
 
 #ifdef CHARSET_CONVERSION
 #	define IS_LOCAL_CHARSET(c)	(!strncasecmp(tinrc.mm_local_charset, c, strlen(c)))
@@ -861,15 +932,15 @@ enum rc_state { RC_IGNORE, RC_UPGRADE, RC_DOWNGRADE, RC_ERROR };
  */
 #define STRCPY(dst, src) ((void) snprintf(dst, sizeof(dst), "%s", src))
 /*
-	#ifdef HAVE_STRLCPY
-	#	define STRCPY(dst, src) (strlcpy(dst, src, sizeof(dst)))
-	#else
-	#	if 0
-	#		define STRCPY(dst, src)	(strncpy(dst, src, sizeof(dst) - 1)[sizeof(dst) - 1] = '\0')
-	#	else
-	#		define STRCPY(dst, src)	(*(dst) = '\0', strncat(dst, src, sizeof(dst) - 1))
-	#	endif
-	#endif
+#ifdef HAVE_STRLCPY
+#	define STRCPY(dst, src) (strlcpy(dst, src, sizeof(dst)))
+#else
+#	if 0
+#		define STRCPY(dst, src)	(strncpy(dst, src, sizeof(dst) - 1)[sizeof(dst) - 1] = '\0')
+#	else
+#		define STRCPY(dst, src)	(*(dst) = '\0', strncat(dst, src, sizeof(dst) - 1))
+#	endif
+#endif
 */
 
 #define STRCMPEQ(s1, s2)	(strcmp((s1), (s2)) == 0)
@@ -882,6 +953,7 @@ enum rc_state { RC_IGNORE, RC_UPGRADE, RC_DOWNGRADE, RC_ERROR };
  * LEN         =
  * HEADER_LEN  = max. size of a news/mail header-line
  * NEWSRC_LINE =
+ * LOGIN_NAME_MAX = max. username length (incl. terminating '\0')
  */
 
 #ifdef PATH_MAX
@@ -910,6 +982,9 @@ enum rc_state { RC_IGNORE, RC_UPGRADE, RC_DOWNGRADE, RC_ERROR };
 #else
 #	define NAME_LEN	14
 #endif /* HAVE_LONG_FILE_NAMES */
+#ifndef LOGIN_NAME_MAX
+#	define LOGIN_NAME_MAX 256
+#endif /* !LOGIN_NAME_MAX */
 #define LEN	1024
 #define BUF_SIZE	1024
 
@@ -923,8 +998,6 @@ enum rc_state { RC_IGNORE, RC_UPGRADE, RC_DOWNGRADE, RC_ERROR };
 
 #define DAY	(60*60*24)		/* Seconds in a day */
 
-#define ctrl(c)	((c) & 0x1F)
-
 #ifndef DEFAULT_ISO2ASC
 #	define DEFAULT_ISO2ASC	"-1 "	/* ISO -> ASCII charset conversion */
 #endif /* !DEFAULT_ISO2ASC */
@@ -934,8 +1007,10 @@ enum rc_state { RC_IGNORE, RC_UPGRADE, RC_DOWNGRADE, RC_ERROR };
 #endif /* !DEFAULT_COMMENT */
 #if defined(MULTIBYTE_ABLE) && !defined(NO_LOCALE)
 #	define T_CHAR_FMT "lc"
+#	define T_CHAR_TYPE wint_t
 #else
 #	define T_CHAR_FMT "c"
+#	define T_CHAR_TYPE int
 #endif /* MULTIBYTE_ABLE && !NO_LOCALE */
 #ifndef ART_MARK_UNREAD
 #	define ART_MARK_UNREAD	'+'	/* used to show that an art is unread */
@@ -964,6 +1039,18 @@ enum rc_state { RC_IGNORE, RC_UPGRADE, RC_DOWNGRADE, RC_ERROR };
 #ifndef MARK_INRANGE
 #	define MARK_INRANGE	'#'	/* group/art within a range (# command) */
 #endif /* !MARK_INRANGE */
+#ifndef SCOPE_MARK_GLOBAL
+#	define SCOPE_MARK_GLOBAL '!' /* scope read from global attributes file */
+#endif /* SCOPE_MARK_GLOBAL */
+#ifndef SCOPE_MARK_GLOBAL_MATCH
+#	define SCOPE_MARK_GLOBAL_MATCH ':' /* scope read from global attributes file, current group matches */
+#endif /* SCOPE_MARK_GLOBAL_MATCH */
+#ifndef SCOPE_MARK_LOCAL
+#	define SCOPE_MARK_LOCAL ' ' /* scope read from local attributes file */
+#endif /* SCOPE_MARK_LOCAL */
+#ifndef SCOPE_MARK_LOCAL_MATCH
+#	define SCOPE_MARK_LOCAL_MATCH '*' /* scope read from local attributes file, current group matches */
+#endif /* SCOPE_MARK_LOCAL_MATCH */
 
 
 /*
@@ -975,8 +1062,8 @@ enum rc_state { RC_IGNORE, RC_UPGRADE, RC_DOWNGRADE, RC_ERROR };
 	 * For CURSOR_ARROW and CURSOR_HORIZ tinrc.utf8_graphics ? :
 	 * is handled inside screen.c:draw_arrow_mark().
 	 */
-#	define CURSOR_ARROW		(wint_t) 0x25B6
-#	define CURSOR_HORIZ		(wint_t) 0x2500
+#	define CURSOR_ARROW		(wint_t) 0x25B6 /* 0x27A4 */
+#	define CURSOR_HORIZ		(wint_t) 0x2500 /* 0x2015 */
 #	define TREE_ARROW		(wchar_t) (tinrc.utf8_graphics ? 0x25B6 : '>')
 #	define TREE_ARROW_WRAP	(wchar_t) (tinrc.utf8_graphics ? 0x25B7 : '>')
 #	define TREE_BLANK		(wchar_t) ' '
@@ -1036,6 +1123,16 @@ enum rc_state { RC_IGNORE, RC_UPGRADE, RC_DOWNGRADE, RC_ERROR };
 #define SKIP_TRAILING			2
 #define COMPACT_MULTIPLE		4
 
+/*
+ * defines for tinrc.verbatim_handling
+ */
+enum {
+	VERBATIM_NONE = 0,
+	VERBATIM_SHOW_ALL,
+	VERBATIM_HIDE_MARKS,
+	VERBATIM_HIDE_ALL
+};
+
 /*
  * defines for tinrc.show_help_mail_sign
  */
@@ -1059,7 +1156,7 @@ enum {
 /*
  * Number of charset-traslation tables (iso2asci)
  */
-#define NUM_ISO_TABLES	7
+#define NUM_ISO_TABLES	6
 
 /*
  * Maximum permissible colour number
@@ -1090,7 +1187,7 @@ enum {
 
 #define REGEX_FMT (tinrc.wildcard ? "%s" : "*%s*")
 
-#define IGNORE_ART(i)	((tinrc.kill_level != KILL_THREAD && arts[i].killed) || (arts[i].thread == ART_EXPIRED))
+#define IGNORE_ART(i)	((tinrc.kill_level == KILL_NOTHREAD && arts[i].killed) || (arts[i].thread == ART_EXPIRED))
 /* only used for threading */
 #define IGNORE_ART_THREAD(i)	(arts[i].thread != ART_UNTHREADED || (tinrc.kill_level == KILL_NOTHREAD && arts[i].killed))
 
@@ -1124,11 +1221,6 @@ enum {
  */
 #define UNREAD_GROUP(i)		(!active[my_group[i]].bogus && active[my_group[i]].newsrc.num_unread > 0)
 
-/*
- * Expands to singular/plural version of string
- */
-#define PLURAL(x,y)			((x == 1) ? _(y##_singular) : _(y##_plural))
-
 /*
  * News/Mail group types
  */
@@ -1139,18 +1231,18 @@ enum {
 /*
  * used by get_arrow_key()
  */
-#	define KEYMAP_UNKNOWN		0
-#	define KEYMAP_UP		1
-#	define KEYMAP_DOWN		2
-#	define KEYMAP_LEFT		3
-#	define KEYMAP_RIGHT		4
-#	define KEYMAP_PAGE_UP		5
-#	define KEYMAP_PAGE_DOWN	6
-#	define KEYMAP_HOME		7
-#	define KEYMAP_END		8
-#	define KEYMAP_DEL		9
-#	define KEYMAP_INS		10
-#	define KEYMAP_MOUSE		11
+#define KEYMAP_UNKNOWN		0
+#define KEYMAP_UP			1
+#define KEYMAP_DOWN			2
+#define KEYMAP_LEFT			3
+#define KEYMAP_RIGHT		4
+#define KEYMAP_PAGE_UP		5
+#define KEYMAP_PAGE_DOWN	6
+#define KEYMAP_HOME			7
+#define KEYMAP_END			8
+#define KEYMAP_DEL			9
+#define KEYMAP_INS			10
+#define KEYMAP_MOUSE		11
 
 
 /*
@@ -1175,17 +1267,20 @@ enum {
 /*
  * used by feed_articles() & show_mini_help() & quick_filter & add_filter_rule
  */
-#define SELECT_LEVEL	1
-#define GROUP_LEVEL	2
-#define THREAD_LEVEL	3
-#define PAGE_LEVEL	4
-#define INFO_PAGER	5
-#define SCOPE_LEVEL	6
-#define CONFIG_LEVEL	7
-#define ATTRIB_LEVEL	8
-#define ATTACHMENT_LEVEL	9
-#define URL_LEVEL	10
-#define POSTED_LEVEL	11
+#define SELECT_LEVEL		(1 << 0)
+#define GROUP_LEVEL			(1 << 1)
+#define THREAD_LEVEL		(1 << 2)
+#define PAGE_LEVEL			(1 << 3)
+#define SCOPE_LEVEL			(1 << 4)
+#define CONFIG_LEVEL		(1 << 5)
+#define ATTRIB_LEVEL		(1 << 6)
+#define ATTACHMENT_LEVEL	(1 << 7)
+#define URL_LEVEL			(1 << 8)
+#define POSTED_LEVEL		(1 << 9)
+#define SERVERRC_LEVEL		(1 << 10)
+#define INFO_PAGER			(1 << 11)	/* has no help text, put at end */
+#define MIN_LEVEL			SELECT_LEVEL
+#define MAX_LEVEL			INFO_PAGER
 
 #define MINI_HELP_LINES		5
 
@@ -1314,9 +1409,44 @@ enum {
  */
 #define UUE_NO			0		/* Don't hide uue data */
 #define UUE_YES			1		/* Hide uue data */
-#define UUE_ALL			2		/* Hide uue data harder */
-#define UUE_COMPLETE	3		/* uue part is complete */
-#define UUE_INCOMPLETE	4		/* uue part is not complete */
+#define UUE_INCOMPL		2
+#define UUE_ALL			(UUE_YES|UUE_INCOMPL)	/* Hide uue data harder */
+#define PGP_ALL			4		/* Hide inline PGP signature/key */
+#define SHAR_ALL		8		/* hide shell-scripts */
+#define HIDE_ALL		(UUE_YES|UUE_ALL|PGP_ALL|SHAR_ALL)
+
+
+/* RFC 9580 6.2.1 / 6.2.3 */
+#define PGP_PUBLIC_KEY_TAG "-----BEGIN PGP PUBLIC KEY BLOCK-----\n"
+#define PGP_PUBLIC_KEY_END_TAG "-----END PGP PUBLIC KEY BLOCK-----\n"
+#if 0 /* we do not look for these */
+#	define PGP_PRIVATE_KEY_TAG "-----BEGIN PGP PRIVATE KEY BLOCK-----\n"
+#	define PGP_PRIVATE_KEY_END_TAG "-----END PGP PRIVATE KEY BLOCK-----\n"
+#endif /* 0*/
+#define PGP_SIG_TAG "-----BEGIN PGP SIGNATURE-----\n"
+#define PGP_SIG_END_TAG "-----END PGP SIGNATURE-----\n"
+
+
+/*
+ * Different sections of an article
+ */
+enum section_type {
+	SECTION_DEFAULT = 0,
+	SECTION_ATTACHMENT,
+	SECTION_UUE_COMPLETE,
+	SECTION_UUE_INCOMPLETE,
+	SECTION_YENC_COMPLETE,
+	SECTION_YENC_INCOMPLETE,	/* something missing */
+	SECTION_YENC_PARTIAL,		/* one full part of a multipart */
+	SECTION_YENC_CORRUPT		/* e.g. crc or size mismatch */
+};
+
+
+enum menu_type {
+	options_menu = 0,
+	serverrc_menu
+};
+
 
 /*
  * used in misc.c/rfc1524.c
@@ -1389,8 +1519,6 @@ enum {
 #	endif /* CPP_DOES_EXPAND */
 #endif /* NDEBUG */
 
-#define ESC	27
-
 /*
  * filter entries expire after DEFAULT_FILTER_DAYS
  */
@@ -1443,15 +1571,20 @@ enum {
  */
 #define SCORE_MAX		10000
 
-#define FILTER_SUBJ_CASE_SENSITIVE		0
-#define FILTER_SUBJ_CASE_IGNORE		1
-#define FILTER_FROM_CASE_SENSITIVE		2
-#define FILTER_FROM_CASE_IGNORE		3
-#define FILTER_MSGID		4
-#define FILTER_MSGID_LAST	5
-#define FILTER_MSGID_ONLY	6
-#define FILTER_REFS_ONLY	7
-#define FILTER_LINES		8
+enum {
+	FILTER_SUBJ_CASE_SENSITIVE = 0,
+	FILTER_SUBJ_CASE_IGNORE,
+	FILTER_FROM_CASE_SENSITIVE,
+	FILTER_FROM_CASE_IGNORE,
+	FILTER_MSGID,
+	FILTER_MSGID_LAST,
+	FILTER_MSGID_ONLY,
+	FILTER_REFS_ONLY,
+	FILTER_XREF_CASE_IGNORE,
+	FILTER_PATH_CASE_IGNORE,
+	FILTER_LIST_MAX,
+	FILTER_LINES = FILTER_LIST_MAX
+};
 
 #define FILTER_LINES_NO		0
 #define FILTER_LINES_EQ		1
@@ -1469,6 +1602,7 @@ enum {
 #define DEFAULT_ATTACHMENT_FORMAT	"%T%S%E%C%d"
 #define DEFAULT_PAGE_MIME_FORMAT	"[-- %T%S%*n%z%*l%!c%!d%*e --]"
 #define DEFAULT_PAGE_UUE_FORMAT		"[-- %T%S%*n%I%!d%*e --]"
+#define DEFAULT_PAGE_YENC_FORMAT	"[-- %*N%!d%I [%F%G]%V%W%X --]"
 #define DEFAULT_DATE_FORMAT			"%a, %d %b %Y %H:%M:%S"
 
 /*
@@ -1536,6 +1670,7 @@ enum {
 #	endif /* HAVE_UNICODE_UBIDI_H */
 #endif /* HAVE_LIBICUUC */
 
+#define ACE_PREFIX "xn--"
 
 /*
  * used in checking article header before posting
@@ -1613,6 +1748,8 @@ struct t_mailbox {
 struct t_article {
 	t_artnum artnum;		/* Article number in spool directory for group */
 	char *subject;			/* Subject: line from mail header */
+	char *subject_raw;		/* plain undecoded subject */
+	char *from_raw;			/* plain undecoded from */
 	char *xref;			/* Xref: cross posted article reference line */
 	char *path;			/* Path: line */
 	/* NB: The msgid and refs are only retained until the reference tree is built */
@@ -1665,34 +1802,61 @@ struct t_newsheader {
 #define CAST_MASK(value,bits)	(((1U << (bits)) - 1) & (unsigned) (value))
 #define CAST_BOOL(value)	CAST_MASK(value, 1)
 #define CAST_BITS(value,bits)	CAST_MASK(value, BITS_OF(bits))
-#define BITS_OF(bits)		BITS_OF_ ## bits
 
 #define BoolField(value)	unsigned value:1
 #define IntField(value)	unsigned value:BITS_OF(value)
 
-#define BITS_OF_auto_cc_bcc		2
-#define BITS_OF_mail_mime_encoding	2
-#define BITS_OF_mm_network_charset	6
-#define BITS_OF_post_mime_encoding	2
-#define BITS_OF_post_process_type	2
-#define BITS_OF_quick_kill_header	3
-#define BITS_OF_quick_select_header	3
-#define BITS_OF_show_author		2
-#define BITS_OF_sort_article_type	4
-#define BITS_OF_sort_threads_type	3
-#define BITS_OF_thread_articles	3
-#define BITS_OF_thread_perc		7
-#define BITS_OF_trim_article_body	3
+#ifdef CPP_DOES_CONCAT
+#	define BITS_OF(bits)				BITS_OF_ ## bits
+#	define BITS_OF_auto_cc_bcc			2
+#	define BITS_OF_mail_mime_encoding	2
+#	define BITS_OF_mm_network_charset	6
+#	define BITS_OF_post_mime_encoding	2
+#	define BITS_OF_post_process_type	2
+#	define BITS_OF_quick_kill_header	3
+#	define BITS_OF_quick_select_header	3
+#	define BITS_OF_show_author			2
+#	define BITS_OF_sort_article_type	4
+#	define BITS_OF_sort_threads_type	3
+#	define BITS_OF_thread_articles		3
+#	define BITS_OF_thread_perc			7
+#	define BITS_OF_hide_inline_data		4
+#	define BITS_OF_trim_article_body	3
+#	define BITS_OF_verbatim_handling	2
+#else
+#	define BITS_OF(bits)	bits
+	enum bits_of {
+		auto_cc_bcc	        = 2,
+		mail_mime_encoding  = 2,
+		mm_network_charset  = 6,
+		post_mime_encoding  = 2,
+		post_process_type   = 2,
+		quick_kill_header   = 3,
+		quick_select_header = 3,
+		show_author         = 2,
+		sort_article_type   = 4,
+		sort_threads_type   = 3,
+		thread_articles     = 3,
+		thread_perc         = 7,
+		hide_inline_data	= 4,
+		trim_article_body   = 3,
+		verbatim_handling   = 2
+	};
+#endif /* CPP_DOES_CONCAT */
 
 /*
  * used as flags for t_attach_item
  */
-#define ATTACH_SHOW_CONTENT		(1 << 0) /* content visible */
-#define ATTACH_SHOW_BOTH		(1 << 1) /* description and content visible */
-#define ATTACH_OMIT_DESC		(1 << 2) /* description can be omitted */
-#define ATTACH_OMIT_BOTH		(1 << 3) /* description and content can be omitted */
-#define ATTACH_ITEM_IS_TYPE		(1 << 4) /* content type */
-#define ATTACH_ITEM_IS_SUBTYPE	(1 << 5) /* content subtype */
+#define ATTACH_SHOW_CONTENT				(1 << 0) /* content visible */
+#define ATTACH_SHOW_BOTH				(1 << 1) /* description and content visible */
+#define ATTACH_OMIT_DESC				(1 << 2) /* description can be omitted */
+#define ATTACH_OMIT_BOTH				(1 << 3) /* description and content can be omitted */
+#define ATTACH_ITEM_IS_TYPE				(1 << 4) /* content type */
+#define ATTACH_ITEM_IS_SUBTYPE			(1 << 5) /* content subtype */
+#define ATTACH_ITEM_IS_YENC_PART		(1 << 6) /* yenc part */
+#define ATTACH_ITEM_IS_YENC_TOTAL		(1 << 7) /* yenc total # of parts */
+#define ATTACH_ITEM_IS_YENC_PART_SIZE	(1 << 8) /* size if yenc part */
+#define ATTACH_ITEM_IS_YENC_TOTAL_SIZE	(1 << 9) /* size if yenc */
 
 /*
  * struct t_attach_item - information about a specific header part of a mime attachment
@@ -1744,6 +1908,7 @@ struct t_attribute {
 #endif /* CHARSET_CONVERSION */
 	struct t_newsheader *headers_to_display;	/* array of which headers to display */
 	struct t_newsheader *headers_to_not_display;	/* array of which headers to not display */
+	int getart_limit;
 	BoolField(global);			/* global/group specific */
 	IntField(quick_kill_header);	/* quick filter kill header */
 	BoolField(quick_kill_expire);	/* quick filter kill limited/unlimited time */
@@ -1786,6 +1951,13 @@ struct t_attribute {
 #if defined(MULTIBYTE_ABLE) && !defined(NO_LOCALE)
 	BoolField(suppress_soft_hyphens);	/* set TRUE to remove soft hyphens (U+00AD) from articles */
 #endif /* MULTIBYTE_ABLE && !NO_LOCALE */
+	IntField(hide_inline_data);	/* 0=No, display raw data
+						1=uuencoded and yenc data will be condensed to a single tag line showing
+						  size and filename, similar to how MIME attachments are displayed
+						2=Also hide partial, as for 1, but any line that looks like uuencoded or
+						  yenc data will be folded into a tag line.
+						4=Hide Inline PGP
+						8=Hide Shar */
 	IntField(trim_article_body);	/* 0=Don't trim article body, 1=Skip leading blank lines,
 						2=Skip trailing blank lines, 3=Skip leading and trailing blank lines,
 						4=Compact multiple blank lines between textblocks,
@@ -1793,7 +1965,9 @@ struct t_attribute {
 						6=Compact multiple blank lines between textblocks and skip trailing blank lines,
 						7=Compact multiple blank lines between textblocks and skip leading and trailing
 						  blank lines */
-	BoolField(verbatim_handling);	/* 0=none, 1=detect verbatim blocks */
+	IntField(verbatim_handling);	/* 0=none, 1=detect verbatim blocks and show all,
+						2=detect verbatim blocks, don't show begin and end marker,
+						3=detect verbatim blocks and hide them completely */
 #ifdef HAVE_COLOR
 	BoolField(extquote_handling);		/* 0=none, 1=detect quoted text from external sources */
 #endif /* HAVE_COLOR */
@@ -1829,8 +2003,10 @@ struct t_attribute_state {
 	BoolField(fcc);
 	BoolField(followup_to);
 	BoolField(from);
+	BoolField(getart_limit);
 	BoolField(group_catchup_on_exit);
 	BoolField(group_format);
+	BoolField(hide_inline_data);
 #ifdef HAVE_ISPELL
 	BoolField(ispell);
 #endif /* HAVE_ISPELL */
@@ -1912,6 +2088,14 @@ struct t_scope {
 	BoolField(global);			/* TRUE for scopes from global_attributes_file */
 };
 
+/*
+ * Scopes that matched the group
+ */
+struct t_scope_list{
+	struct t_scope *scope;
+	struct t_scope_list *next;
+};
+
 /*
  * struct t_newsrc - newsrc related info.
  */
@@ -1947,6 +2131,7 @@ struct t_group {
 	struct t_newsrc newsrc;		/* newsrc bitmap specific info. */
 	struct t_attribute *attribute;	/* group specific attributes */
 	struct t_filters *glob_filter;	/* points to filter array */
+	struct t_scope_list *scopes_list;	/* list of scopes that matched the group */
 };
 
 /*
@@ -1965,8 +2150,8 @@ struct t_hashnode {
  * precalculated length
  */
 struct t_fmt {
-	char str[LEN];
-	char date_str[LEN];
+	char *str;
+	char *date_str;
 	size_t len_date;		/* %D Date */
 	size_t len_date_max;
 	size_t len_grpdesc;		/* %d newsgroup description */
@@ -2107,12 +2292,16 @@ struct t_screen {
 };
 #endif /* !USE_CURSES */
 
+/*
+ * NOTE: the members are used during display only
+ * TODO: add user-defined display-format ("%D %80G %K %120s %256M")
+ */
 typedef struct posted {
-	char date[10];
-	char group[80];
-	char action;
-	char subj[120];
-	char mid[256];
+	char date[10];	/* dd-mm-yy */
+	char action;	/* [ dfrwx] */
+	char *group;
+	char *subj;
+	char *mid;
 	struct posted *next;
 } t_posted;
 
@@ -2247,6 +2436,17 @@ typedef struct urllist {
 } t_url;
 
 
+#ifdef NNTP_ABLE
+typedef struct distribpat {
+	int weight;
+	char *pattern;
+	char *distribution;
+	char *description;
+	struct distribpat *next;
+} t_distrib_pat;
+#endif /* NNTP_ABLE */
+
+
 /*
  * Determine signal return type
  */
@@ -2271,7 +2471,7 @@ typedef struct urllist {
 
 /* Define a matching function pointer type */
 typedef int (*t_compfunc)(t_comptype, t_comptype);
-typedef void (*t_sortfunc)(void *, size_t, size_t, t_compfunc);
+/* typedef void (*t_sortfunc)(void *, size_t, size_t, t_compfunc); */
 
 #define _CDECL
 
@@ -2450,7 +2650,6 @@ typedef void (*t_sortfunc)(void *, size_t, size_t, t_compfunc);
 #endif /* TIN_STDC */
 
 typedef OUTC_RETTYPE (*OutcPtr) (OUTC_ARGS);
-typedef FILE TCP;
 
 #ifndef EXTERN_H
 #	include "extern.h"
@@ -2646,6 +2845,8 @@ extern int fclose(FILE *);
 #	define MM_CHARSET "US-ASCII"
 #endif /* !MM_CHARSET */
 
+/* RFC 2978 2.3 */
+#define CHARSET_MAX_NAME_LEN	40
 
 #if !defined(SEEK_SET)
 #	define SEEK_SET 0L
@@ -2661,12 +2862,10 @@ extern int fclose(FILE *);
 #define TIN_LETTER_NAME	".letter"
 #define TIN_BUGREPORT_NAME	".bugreport"
 
-/* read_news_active_file() / open_newsgroups_fp() */
-#ifndef DISABLE_PIPELINING
-#	define PIPELINE_LIMIT 45
-#else
-#	define PIPELINE_LIMIT 1
-#endif /* DISABLE_PIPELINING */
+/* serverrc.nntp_pipeline_limit (was DISABLE_PIPELINING) */
+#define PIPELINE_LIMIT_DEFAULT	50
+#define PIPELINE_LIMIT_MAX		256
+#define PIPELINE_LIMIT_MIN		1
 
 #ifndef DEBUG_H
 #	include "debug.h"
@@ -2697,4 +2896,7 @@ struct t_version {
 /* C90: isxdigit(3) */
 #define IS_XDIGIT(c) (((c) >= '0' && (c) <= '9') || ((c) >= 'a' && (c) <= 'f') || ((c) >= 'A' && (c) <= 'F'))
 
+#define RFC5322_SPECIALS	"()<>[]:;@\\,.\""
+#define CHECK_RFC5322_SPECIALS(name)	((strpbrk(name, RFC5322_SPECIALS) != NULL) && name[0] != '"' && name[strlen(name)] != '"')
+
 #endif /* !TIN_H */
