@@ -3,10 +3,10 @@
  *  Module    : active.c
  *  Author    : I. Lea
  *  Created   : 1992-02-16
- *  Updated   : 2024-06-21
+ *  Updated   : 2025-07-05
  *  Notes     :
  *
- * Copyright (c) 1992-2024 Iain Lea <iain@bricbrac.de>
+ * Copyright (c) 1992-2025 Iain Lea <iain@bricbrac.de>
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -73,11 +73,11 @@ static FILE *open_newgroups_fp(int idx);
 static FILE *open_news_active_fp(void);
 static int check_for_any_new_groups(void);
 static void active_add(struct t_group *ptr, t_artnum count, t_artnum max, t_artnum min, const char *moderated);
-static void append_group_line(char *active_file, char *group_path, t_artnum art_max, t_artnum art_min, char *base_dir);
+static void append_group_line(const char *active_file, const char *group_path, t_artnum art_max, t_artnum art_min, char *base_dir);
 static void make_group_list(char *active_file, char *base_dir, char *fixed_base, char *group_path);
 static void read_active_file(void);
 static void read_newsrc_active_file(void);
-static void subscribe_new_group(char *group, char *autosubscribe, char *autounsubscribe);
+static void subscribe_new_group(char *group, const char *autosubscribe, const char *autounsubscribe);
 #ifdef NNTP_ABLE
 	static t_bool do_read_newsrc_active_file(FILE *fp);
 	static t_bool parse_count_line(char *line, t_artnum *max, t_artnum *min, t_artnum *count, char *moderated);
@@ -117,7 +117,7 @@ resync_active_file(
 		old_group = my_strdup(CURR_GROUP.name);
 
 	write_newsrc();
-	read_news_active_file();
+	read_news_active_file(FALSE);
 
 #ifdef HAVE_MH_MAIL_HANDLING
 	read_mail_active_file();
@@ -193,7 +193,7 @@ active_add(
  */
 t_bool
 process_bogus(
-	char *name) /* return value is always ignored */
+	const char *name) /* return value is always ignored */
 {
 	struct t_group *ptr;
 
@@ -235,7 +235,7 @@ parse_active_line(
 #endif /* DEBUG */
 	t_bool lineok = FALSE;
 
-	if (line[0] == '#' || line[0] == '\0')
+	if (!*line || *line == '#')
 		return lineok;
 
 #ifdef DEBUG
@@ -270,6 +270,7 @@ parse_active_line(
 	*min = atoartnum(q);
 
 	if (!lineok) { /* missing moderation flag - seen on usenet.farm */
+		/* TODO: don't do the y-guessing with mail-active-files */
 		strcpy(moderated, "y");	/* guess posting is fine */
 		lineok = TRUE;
 	} else
@@ -299,7 +300,7 @@ parse_count_line(
 	char *p = NULL, *q = NULL, *r = NULL, *s = NULL;
 	t_bool lineok = FALSE;
 
-	if (line[0] == '#' || line[0] == '\0')
+	if (!*line || *line == '#')
 		return FALSE;
 
 	if (strtok(line, ACTIVE_SEP)) {		/* skip group name */
@@ -361,11 +362,11 @@ do_read_newsrc_active_file(
 #ifdef NNTP_ABLE
 	t_bool need_auth = FALSE;
 	char *ngnames[NUM_SIMULTANEOUS_GROUP_COMMAND] = { NULL };
-	int index_i = 0, index_o = 0;
-	int respcode, i, j;
 	char fmt[25];
 	char buf[NNTP_STRLEN];
 	char line[NNTP_STRLEN];
+	int index_i = 0, index_o = 0;
+	int respcode, i, j;
 
 	snprintf(fmt, sizeof(fmt), "%%"T_ARTNUM_SFMT" %%"T_ARTNUM_SFMT" %%"T_ARTNUM_SFMT" %%%ds", NNTP_GRPLEN);
 #endif /* NNTP_ABLE */
@@ -394,7 +395,7 @@ do_read_newsrc_active_file(
 
 		if (read_news_via_nntp && !read_saved_news) { /* this should be limited to GROUP_TYPE_NEWS, but ... */
 #ifdef NNTP_ABLE
-			if (window < NUM_SIMULTANEOUS_GROUP_COMMAND && ptr && (!list_active || (newsrc_active && list_active && group_find(ptr, FALSE)))) {
+			if (window < NUM_SIMULTANEOUS_GROUP_COMMAND && ptr && (!list_active || (newsrc_active && group_find(ptr, FALSE)))) {
 				ngnames[index_i] = my_strdup(ptr);
 				snprintf(buf, sizeof(buf), "GROUP %s", ngnames[index_i]);
 #	ifdef DEBUG
@@ -403,7 +404,7 @@ do_read_newsrc_active_file(
 #	endif /* DEBUG */
 				put_server(buf, FALSE);
 				index_i = (index_i + 1) % NUM_SIMULTANEOUS_GROUP_COMMAND;
-				window++;
+				++window;
 			}
 			if (window == NUM_SIMULTANEOUS_GROUP_COMMAND || ptr == NULL) {
 				respcode = get_only_respcode(line, sizeof(line));
@@ -447,9 +448,9 @@ do_read_newsrc_active_file(
 #	endif /* DEBUG */
 							}
 							ptr = ngname;
-							free(ngnames[index_o]);
+							FreeAndNull(ngnames[index_o]);
 							index_o = (index_o + 1) % NUM_SIMULTANEOUS_GROUP_COMMAND;
-							window--;
+							--window;
 							break;
 						}
 
@@ -460,16 +461,21 @@ do_read_newsrc_active_file(
 						/* FALLTHROUGH */
 
 					case ERR_NOGROUP:
-						free(ngnames[index_o]);
+						FreeAndNull(ngnames[index_o]);
 						index_o = (index_o + 1) % NUM_SIMULTANEOUS_GROUP_COMMAND;
-						window--;
+						--window;
 						continue;
 
 					case ERR_ACCESS:
-						for (index_i = 0; index_i < NUM_SIMULTANEOUS_GROUP_COMMAND - 1; index_i++) {
-							FreeIfNeeded(ngnames[index_i]);
+						{
+							char bbuf[4 + NNTP_GRPLEN + NNTP_STRLEN + 3 + 1];
+
+							snprintf(bbuf, sizeof(bbuf), "%d %s (%s)", respcode, line, BlankIfNull(ngnames[index_o]));
+							for (index_i = 0; index_i < NUM_SIMULTANEOUS_GROUP_COMMAND - 1; ++index_i) {
+								FreeIfNeeded(ngnames[index_i]);
+							}
+							tin_done(NNTP_ERROR_EXIT, "%s", bbuf);
 						}
-						tin_done(NNTP_ERROR_EXIT, "%s", line);
 						/* keep lint quiet: */
 						/* FALLTHROUGH */
 
@@ -478,9 +484,9 @@ do_read_newsrc_active_file(
 						if ((debug & DEBUG_NNTP) && verbose > 1)
 							debug_print_file("NNTP", "NOT_OK %s", line);
 #	endif /* DEBUG */
-						free(ngnames[index_o]);
+						FreeAndNull(ngnames[index_o]);
 						index_o = (index_o + 1) % NUM_SIMULTANEOUS_GROUP_COMMAND;
-						window--;
+						--window;
 						continue;
 				}
 			} else
@@ -542,9 +548,6 @@ read_newsrc_active_file(
 	void)
 {
 	FILE *fp;
-#ifdef NNTP_ABLE
-	t_bool need_auth;
-#endif /* NNTP_ABLE */
 
 	/*
 	 * return immediately if no .newsrc can be found or .newsrc is empty
@@ -556,9 +559,7 @@ read_newsrc_active_file(
 #ifndef NNTP_ABLE
 	do_read_newsrc_active_file(fp);
 #else
-	need_auth = do_read_newsrc_active_file(fp);
-
-	if (need_auth) { /* delayed auth */
+	if (do_read_newsrc_active_file(fp)) {	/* delayed auth */
 		if (!authenticate(nntp_server, userid, FALSE)) {
 			fclose(fp);
 			tin_done(EXIT_FAILURE, _(txt_auth_failed), ERR_ACCESS);
@@ -635,11 +636,10 @@ read_active_file(
 			tin_done(EXIT_FAILURE, _(txt_cannot_open_active_file), news_active_file, tin_progname);
 		}
 #	endif /* !NNTP_ONLY */
+#else
+		perror_message("%s", news_active_file);
+		tin_done(EXIT_FAILURE, _(txt_cannot_open), news_active_file);
 #endif /* NNTP_ABLE */
-		{
-			perror_message("%s", news_active_file);
-			tin_done(EXIT_FAILURE, _(txt_cannot_open), news_active_file);
-		}
 	}
 
 	while ((ptr = tin_fgets(fp, FALSE)) != NULL) {
@@ -787,7 +787,7 @@ read_active_counts(
  */
 int
 read_news_active_file(
-	void)
+	t_bool check_any_unread)
 {
 	FILE *fp;
 	int newgrps = 0;
@@ -830,20 +830,20 @@ read_news_active_file(
 		 * PIPELINE_LIMIT groups in newsrc
 		 */
 		if (read_news_via_nntp && (list_active || nntp_caps.list_counts) && !did_list_cmd) {
-			char buff[NNTP_STRLEN];
-			char *ptr, *q;
-			char moderated[PATH_LEN];
-			int r = 0, j = 0;
-			int i;
-			struct t_group *grpptr;
-			t_artnum count = T_ARTNUM_CONST(-1), min = T_ARTNUM_CONST(1), max = T_ARTNUM_CONST(0);
-			t_bool need_auth = FALSE;
-
-			*buff = '\0';
 			/* we can't use for_each_group(i) yet, so we have to parse the newsrc */
 			if ((fp = tin_fopen(newsrc, "r")) != NULL) {
+				char buff[NNTP_STRLEN];
+				char moderated[PATH_LEN];
+				char *ptr, *q;
+				int r = 0, j = 0;
+				int i;
+				t_artnum count = T_ARTNUM_CONST(-1), min = T_ARTNUM_CONST(1), max = T_ARTNUM_CONST(0);
+				struct t_group *grpptr;
+				t_bool need_auth = FALSE;
+
+				*buff = '\0';
 				while (tin_fgets(fp, FALSE) != NULL)
-					j++;
+					++j;
 				rewind(fp);
 				if (j < PIPELINE_LIMIT) {
 					while ((ptr = tin_fgets(fp, FALSE)) != NULL) {
@@ -852,25 +852,26 @@ read_news_active_file(
 						*q = '\0';
 						if (nntp_caps.type == CAPABILITIES && (nntp_caps.list_active || nntp_caps.list_counts)) {
 							/* LIST ACTIVE or LIST COUNTS takes wildmats */
-							if (*buff && ((strlen(buff) + strlen(ptr)) < (NNTP_GRPLEN - 1))) { /* append group name */
+							if (*buff && ((strlen(buff) + strlen(ptr)) < (NNTP_GRPLEN - 1))) /* append group name */
 								snprintf(buff + strlen(buff), sizeof(buff) - strlen(buff), ",%s", ptr);
-							} else {
+							else {
 								if (*buff) {
 									put_server(buff, FALSE);
-									r++;
+									++r;
 								}
 								snprintf(buff, sizeof(buff), "LIST %s %s", nntp_caps.list_counts ? "COUNTS" : "ACTIVE", ptr);
 							}
 							continue;
 						} else
 							snprintf(buff, sizeof(buff), "LIST ACTIVE %s", ptr);
+
 						put_server(buff, FALSE);
-						r++;
+						++r;
 						*buff = '\0';
 					}
 					if (*buff) {
 						put_server(buff, FALSE);
-						r++;
+						++r;
 					}
 				} else
 					do_group_cmds = TRUE;
@@ -894,7 +895,7 @@ read_news_active_file(
 								if ((debug & DEBUG_NNTP) && verbose) /* long multiline response */
 									debug_print_file("NNTP", "<<<%s%s", logtime(), ptr);
 #		endif /* DEBUG */
-								if (nntp_caps.type == CAPABILITIES && nntp_caps.list_counts) {
+								if (nntp_caps.list_counts) {
 									if (!parse_count_line(ptr, &max, &min, &count, moderated))
 										continue;
 								} else {
@@ -918,11 +919,11 @@ read_news_active_file(
 								}
 								active_add(grpptr, count, max, min, moderated);
 							}
-#	ifdef DEBUG
+#		ifdef DEBUG
 							/* log end of multiline response to get timing data */
 							if ((debug & DEBUG_NNTP) && !verbose)
 								debug_print_file("NNTP", "<<<%s%s", logtime(), txt_log_data_hidden);
-#	endif /* DEBUG */
+#		endif /* DEBUG */
 						}
 					}
 					if (need_auth) { /* retry after auth is overkill here, so just auth */
@@ -951,10 +952,13 @@ read_news_active_file(
 
 	/*
 	 * finally we have a list of all groups and can set the attributes
+	 * if required
 	 */
-	BegStopWatch();
-	assign_attributes_to_groups();
-	EndStopWatch("assign_attributes_to_groups()");
+	if (!check_any_unread) {
+		BegStopWatch();
+		assign_attributes_to_groups();
+		EndStopWatch("assign_attributes_to_groups()");
+	}
 
 	return newgrps;
 }
@@ -962,40 +966,62 @@ read_news_active_file(
 
 /*
  * Open the active.times file locally or send the NEWGROUPS command
- * "NEWGROUPS yymmdd hhmmss"
+ * "NEWGROUPS yymmdd hhmmss" (or "NEWGROUPS yyymmdd hhmmss GMT" if
+ * server knows CAPABILITIES (=RFC 3977)).
  */
 static FILE *
 open_newgroups_fp(
 	int idx)
 {
 #ifdef NNTP_ABLE
-	char line[NNTP_STRLEN];
-	struct tm *ngtm;
+	if (read_news_via_nntp && !read_saved_news && nntp_caps.newgroups) {
+		char line[NNTP_STRLEN];
+		const struct tm *ngtm;
 
-	if (read_news_via_nntp && !read_saved_news) {
-		/*
-		 * not checking for caps_type == CAPABILITIES && reader as some
-		 * servers do not support it even if advertising READER so we must
-		 * handle errors anyway and just issue the cmd.
-		 */
-		if (idx == -1 || ((ngtm = localtime(&newnews[idx].time)) == NULL))
+		if (idx >= 0) {
+			if (nntp_caps.type == CAPABILITIES)
+				ngtm = gmtime(&newnews[idx].time);
+			else
+				ngtm = localtime(&newnews[idx].time);
+
+			if (ngtm == NULL)
+				return (FILE *) 0;
+
+			if (nntp_caps.type == CAPABILITIES) {
+			/*
+			 * not checking for caps_type == CAPABILITIES && reader as some
+			 * servers do not support it even if advertising READER so we must
+			 * handle errors anyway and just issue the cmd. but we use
+			 * GMT and 4 didigit year on RFC 3977 servers. 4 digit year
+			 * was introduced before Y2K ...
+			 */
+				snprintf(line, sizeof(line), "NEWGROUPS %04d%02d%02d %02d%02d%02d GMT",
+				ngtm->tm_year + 1900, ngtm->tm_mon + 1, ngtm->tm_mday,
+				ngtm->tm_hour, ngtm->tm_min, ngtm->tm_sec);
+			} else {
+				/*
+				 * RFC 3977 states that we SHOULD use 4 digit year but some servers
+				 * still do not support it.
+				 */
+				snprintf(line, sizeof(line), "NEWGROUPS %02d%02d%02d %02d%02d%02d",
+				ngtm->tm_year % 100, ngtm->tm_mon + 1, ngtm->tm_mday,
+				ngtm->tm_hour, ngtm->tm_min, ngtm->tm_sec);
+			}
+		} else
 			return (FILE *) 0;
 
-		/*
-		 * RFC 3977 states that we SHOULD use 4 digit year but some servers
-		 * still do not support it.
-		 */
-		snprintf(line, sizeof(line), "NEWGROUPS %02d%02d%02d %02d%02d%02d",
-			ngtm->tm_year % 100, ngtm->tm_mon + 1, ngtm->tm_mday,
-			ngtm->tm_hour, ngtm->tm_min, ngtm->tm_sec);
-
 		return (nntp_command(line, OK_NEWGROUPS, NULL, 0));
 	}
 #else
 	/* silence compiler warning (unused parameter) */
 	(void) idx;
 #endif /* NNTP_ABLE */
+
+#ifndef NNTP_ONLY
 	return (fopen(active_times_file, "r"));
+#else
+	return (FILE *) 0;
+#endif /* !NNTP_ONLY */
 }
 
 
@@ -1013,7 +1039,7 @@ check_for_any_new_groups(
 	void)
 {
 	FILE *fp;
-	char *autosubscribe, *autounsubscribe;
+	const char *autosubscribe, *autounsubscribe;
 	char *ptr, *line, buf[NNTP_STRLEN];
 	char old_newnews_host[PATH_LEN];
 	int newnews_index;
@@ -1056,13 +1082,13 @@ check_for_any_new_groups(
 			 * we must check the creation date manually
 			 */
 			if ((ptr = strchr(line, ' ')) != NULL) {
-				if (!read_news_via_nntp && ((time_t) atol(ptr) < old_newnews_time || old_newnews_time == (time_t) 0))
+				if (!read_news_via_nntp && ((time_t) strtol(ptr, NULL, 10) < old_newnews_time || old_newnews_time == (time_t) 0))
 					continue;
 
 				*ptr = '\0';
 			}
 			subscribe_new_group(line, autosubscribe, autounsubscribe);
-			newgrps++;
+			++newgrps;
 		}
 		TIN_FCLOSE(fp);
 
@@ -1101,8 +1127,8 @@ check_for_any_new_groups(
 static void
 subscribe_new_group(
 	char *group,
-	char *autosubscribe,
-	char *autounsubscribe)
+	const char *autosubscribe,
+	const char *autounsubscribe)
 {
 	int idx;
 	struct t_group *ptr;
@@ -1166,10 +1192,9 @@ match_group_list(
 	char *separator;
 	char pattern[HEADER_LEN];
 	char ngname[NNTP_GRPLEN + 1]; /* RFC 3977 3.1 limits group names to 497 octets */
-	size_t group_len, list_len;
+	size_t group_len, list_len = strlen(group_list);
 	t_bool negate, accept = FALSE;
 
-	list_len = strlen(group_list);
 	/*
 	 * walk through comma-separated entries in list
 	 */
@@ -1184,9 +1209,9 @@ match_group_list(
 			/*
 			 * a '!' before the pattern inverts sense of match
 			 */
-			group_list++;
-			group_len--;
-			list_len--;
+			++group_list;
+			--group_len;
+			--list_len;
 		}
 		/*
 		 * copy out the entry and terminate it properly
@@ -1206,7 +1231,7 @@ match_group_list(
 		 * now examine next entry if any
 		 */
 		if (group_list[group_len] != '\0')
-			group_len++;	/* skip the separator */
+			++group_len;	/* skip the separator */
 
 		group_list += group_len;
 		list_len -= group_len;
@@ -1219,9 +1244,6 @@ match_group_list(
  * Add or update an entry to the in-memory newnews[] array (The times newgroups
  * were last checked for a particular news server)
  * If this is first time we've been called, zero out the array.
- *
- * Side effects:
- *   'info' is modified. Caller should not depend on it.
  */
 void
 load_newnews_info(
@@ -1248,7 +1270,10 @@ load_newnews_info(
 		return;
 
 	*ptr++ = '\0';
-	new_time = (time_t) atol(ptr);
+	errno = 0;
+	new_time = (time_t) strtol(ptr, NULL, 10);
+	if (errno == ERANGE)
+		new_time = (time_t) 0;
 
 	/*
 	 * If this is a new host entry, set it up
@@ -1327,7 +1352,7 @@ create_save_active_file(
 	if (no_write && file_size(local_save_active_file) != -1L)
 		return;
 
-	if (strfpath((cmdline.args & CMDLINE_SAVEDIR) ? cmdline.savedir : tinrc.savedir, group_path, sizeof(group_path), NULL, FALSE)) {
+	if (strfpath(cmdline.savedir ? cmdline.savedir : tinrc.savedir, group_path, sizeof(group_path), NULL, FALSE)) {
 		wait_message(0, _(txt_creating_active));
 		print_active_head(local_save_active_file);
 
@@ -1335,7 +1360,7 @@ create_save_active_file(
 			group_path[strlen(group_path) - 1] = '\0';
 
 		fb = my_strdup(group_path);
-		make_group_list(local_save_active_file, (cmdline.args & CMDLINE_SAVEDIR) ? cmdline.savedir : tinrc.savedir, fb, group_path);
+		make_group_list(local_save_active_file, cmdline.savedir ? cmdline.savedir : tinrc.savedir, fb, group_path);
 		free(fb);
 	}
 }
@@ -1356,10 +1381,9 @@ make_group_list(
 	t_artnum art_max;
 	t_artnum art_min;
 	struct stat stat_info;
-	t_bool is_dir;
+	t_bool is_dir = FALSE;
 
 	if ((dir = opendir(group_path)) != NULL) {
-		is_dir = FALSE;
 		while ((direntry = readdir(dir)) != NULL) {
 			STRCPY(filename, direntry->d_name);
 			joinpath(path, sizeof(path), group_path, filename);
@@ -1388,8 +1412,8 @@ make_group_list(
 
 static void
 append_group_line(
-	char *active_file,
-	char *group_path,
+	const char *active_file,
+	const char *group_path,
 	t_artnum art_max,
 	t_artnum art_min,
 	char *base_dir)
@@ -1400,7 +1424,8 @@ append_group_line(
 	if (art_max == 0 && art_min == 1)
 		return;
 
-	file_tmp = get_tmpfilename(active_file);
+	if ((file_tmp = get_tmpfilename(active_file)) == NULL)
+		return;
 
 	if (!backup_file(active_file, file_tmp)) {
 		free(file_tmp);
@@ -1413,7 +1438,7 @@ append_group_line(
 		int err;
 
 		ptr = group_name = my_strdup(group_path);
-		ptr++;
+		++ptr;
 		while ((ptr = strchr(ptr, '/')) != NULL)
 			*ptr = '.';
 
