@@ -3,7 +3,7 @@
  *  Module    : curses.c
  *  Author    : D. Taylor & I. Lea
  *  Created   : 1986-01-01
- *  Updated   : 2024-07-28
+ *  Updated   : 2025-05-01
  *  Notes     : This is a screen management library borrowed with permission
  *              from the Elm mail system. This library was hacked to provide
  *              what tin needs.
@@ -82,8 +82,8 @@ t_bool have_linescroll = FALSE;
 #				define USE_SGTTY 1
 #				define TTY struct sgttyb
 /*
-	#			else
-	#				error "No termios.h, termio.h or sgtty.h found"
+#			else
+#				error "No termios.h, termio.h or sgtty.h found"
 */
 #			endif /* HAVE_SGTTY_H */
 #		endif /* HAVE_TERMIO_H */
@@ -126,8 +126,8 @@ static int _columns, _line, _lines, _colors;
 #				define SET_TTY(arg) stty(TTYIN, arg)
 #				define GET_TTY(arg) gtty(TTYIN, arg)
 /*
-	#			else
-	#				error "No termios.h, termio.h or sgtty.h found"
+#			else
+#				error "No termios.h, termio.h or sgtty.h found"
 */
 #			endif /* USE_SGTTY */
 #		endif /* USE_TERMIO */
@@ -544,7 +544,7 @@ StartInverse(
 /*	in_inverse = 1; */
 	if (_setinverse && tinrc.inverse_okay) {
 #	ifdef HAVE_COLOR
-		if (use_color) {
+		if (use_color && (tinrc.col_invers_bg != -1 || tinrc.col_invers_fg != -1)) {
 			bcol(tinrc.col_invers_bg);
 			fcol(tinrc.col_invers_fg);
 		} else {
@@ -568,7 +568,7 @@ EndInverse(
 /*	in_inverse = 0; */
 	if (_clearinverse && tinrc.inverse_okay) {
 #	ifdef HAVE_COLOR
-		if (use_color) {
+		if (use_color && (tinrc.col_invers_bg != -1 || tinrc.col_invers_fg != -1)) {
 			fcol(tinrc.col_normal);
 			bcol(tinrc.col_back);
 		} else {
@@ -1181,7 +1181,7 @@ ReadWch(
 	void)
 {
 	char *mbs = my_malloc(MB_CUR_MAX + 1);
-	int result, to_read;
+	int result;
 	wchar_t wch;
 
 	fflush(stdout);
@@ -1235,7 +1235,7 @@ ReadWch(
 	 * UTF-8
 	 */
 	if (IS_LOCAL_CHARSET("UTF-8")) {
-		int ch = mbs[0] & 0xFF;
+		int to_read, ch = mbs[0] & 0xFF;
 
 		/* determine the count of bytes we have still have to read */
 		if (ch <= 0x7F) {
