@@ -3,10 +3,10 @@
  *  Module    : debug.c
  *  Author    : I. Lea
  *  Created   : 1991-04-01
- *  Updated   : 2024-07-28
+ *  Updated   : 2025-12-08
  *  Notes     : debug routines
  *
- * Copyright (c) 1991-2024 Iain Lea <iain@bricbrac.de>
+ * Copyright (c) 1991-2026 Iain Lea <iain@bricbrac.de>
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -48,15 +48,15 @@
 #	endif /* !NEWSRC_H */
 #endif /* DEBUG */
 
-unsigned short debug;
+unsigned int debug = 0;
 
 #ifdef DEBUG
 /*
  * Local prototypes
  */
-static void debug_print_attributes(struct t_attribute *attr, FILE *fp);
+static void debug_print_attributes(const struct t_attribute *attr, FILE *fp);
 static void debug_print_filter(FILE *fp, int num, struct t_filter *the_filter);
-static void debug_print_newsrc(struct t_newsrc *lnewsrc, FILE *fp);
+static void debug_print_newsrc(const struct t_newsrc *lnewsrc, FILE *fp);
 
 
 /*
@@ -219,7 +219,7 @@ debug_print_active(
 
 static void
 debug_print_attributes(
-	struct t_attribute *attr,
+	const struct t_attribute *attr,
 	FILE *fp)
 {
 	if (attr == NULL)
@@ -236,19 +236,19 @@ debug_print_attributes(
 		(unsigned) attr->post_process_type);
 	fprintf(fp, "select_header=[%u] select_global=[%s] select_expire=[%s]\n",
 		(unsigned) attr->quick_select_header,
-		BlankIfNull(attr->quick_select_scope),
+		attr->quick_select_scope ? BlankIfNull(*attr->quick_select_scope) : "",
 		bool_unparse(attr->quick_select_expire));
 	fprintf(fp, "kill_header  =[%u] kill_global  =[%s] kill_expire  =[%s]\n",
 		(unsigned) attr->quick_kill_header,
-		BlankIfNull(attr->quick_kill_scope),
+		attr->quick_kill_scope ? BlankIfNull(*attr->quick_kill_scope) : "",
 		bool_unparse(attr->quick_kill_expire));
 	fprintf(fp, "maildir=[%s] savedir=[%s] savefile=[%s]\n",
-		BlankIfNull(attr->maildir),
-		BlankIfNull(attr->savedir),
-		BlankIfNull(attr->savefile));
+		attr->maildir ? BlankIfNull(*attr->maildir) : "",
+		attr->savedir ? BlankIfNull(*attr->savedir) : "",
+		attr->savefile ? BlankIfNull(*attr->savefile) : "");
 	fprintf(fp, "sigfile=[%s] followup_to=[%s]\n\n",
-		BlankIfNull(attr->sigfile),
-		BlankIfNull(attr->followup_to));
+		attr->sigfile ? BlankIfNull(*attr->sigfile) : "",
+		attr->followup_to ? BlankIfNull(*attr->followup_to) : "");
 	fflush(fp);
 }
 
@@ -329,13 +329,13 @@ debug_print_filters(
 	if (!*file)
 		joinpath(file, sizeof(file), tmpdir, "FILTER");
 
-	if ((fp = fopen(file, "w")) != NULL) {
+	if ((fp = fopen(file, "a")) != NULL) {
 		/*
 		 * print global filter
 		 */
 		num = glob_filter.num;
 		the_filter = glob_filter.filter;
-		fprintf(fp, "*** BEG GLOBAL FILTER=[%3d] ***\n", num);
+		fprintf(fp, "\n*** BEG GLOBAL FILTER=[%3d] ***\n", num);
 		for (i = 0; i < num; i++) {
 			debug_print_filter(fp, i, &the_filter[i]);
 			fprintf(fp, "\n");
@@ -445,10 +445,10 @@ debug_print_bitmap(
 
 static void
 debug_print_newsrc(
-	struct t_newsrc *lnewsrc,
+	const struct t_newsrc *lnewsrc,
 	FILE *fp)
 {
-	int j;
+	int j = 0;
 	t_artnum i;
 
 	fprintf(fp, "Newsrc: min=[%"T_ARTNUM_PFMT"] max=[%"T_ARTNUM_PFMT"] bitlen=[%"T_ARTNUM_PFMT"] num_unread=[%"T_ARTNUM_PFMT"] present=[%d]\n",
@@ -457,7 +457,7 @@ debug_print_newsrc(
 
 	fprintf(fp, "bitmap=[");
 	if (lnewsrc->xbitlen && lnewsrc->xbitmap) {
-		for (j = 0, i = lnewsrc->xmin; i <= lnewsrc->xmax; i++) {
+		for (i = lnewsrc->xmin; i <= lnewsrc->xmax; i++) {
 			fprintf(fp, "%d",
 				(NTEST(lnewsrc->xbitmap, i - lnewsrc->xmin) == ART_READ ?
 				ART_READ : ART_UNREAD));
@@ -468,27 +468,4 @@ debug_print_newsrc(
 	fprintf(fp, "]\n");
 	fflush(fp);
 }
-
-
-#	ifdef NNTP_ABLE
-const char *
-logtime(
-	void)
-{
-#		if defined(HAVE_CLOCK_GETTIME) || defined(HAVE_GETTIMEOFDAY)
-	static struct t_tintime log_time;
-	static char out[40];
-
-	if (tin_gettime(&log_time) == 0) {
-		if (my_strftime(out, 39, " [%H:%M:%S.", gmtime(&(log_time.tv_sec)))) {
-			snprintf(out + 11, sizeof(out) - 11, "%09ld", log_time.tv_nsec); /* strlen(" [hh:mm:ss.") */
-			out[17] = '\0'; /* strlen(" [hh:mm:ss.uuuuuu") */
-			strcat(out, "] ");
-			return out;
-		}
-	}
-#		endif /* HAVE_CLOCK_GETTIME || HAVE_GETTIMEOFDAY */
-	return " ";
-}
-#	endif /* NNTP_ABLE */
 #endif /* DEBUG */
