@@ -3,10 +3,10 @@
  *  Module    : filter.c
  *  Author    : I. Lea
  *  Created   : 1992-12-28
- *  Updated   : 2024-05-05
+ *  Updated   : 2025-02-14
  *  Notes     : Filter articles. Kill & auto selection are supported.
  *
- * Copyright (c) 1991-2024 Iain Lea <iain@bricbrac.de>
+ * Copyright (c) 1991-2025 Iain Lea <iain@bricbrac.de>
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -96,12 +96,12 @@ int filter_file_offset;
 /*
  * Local prototypes
  */
-static int get_choice(int x, const char *help, const char *prompt, char *list[], int list_size);
+static int get_choice(int x, const char *help, const char *prompt, const char *list[], int list_size);
 static int set_filter_scope(struct t_group *group);
 static struct t_filter_comment *add_filter_comment(struct t_filter_comment *ptr, char *text);
 static struct t_filter_comment *free_filter_comment(struct t_filter_comment *ptr);
 static struct t_filter_comment *copy_filter_comment(struct t_filter_comment *from, struct t_filter_comment *to);
-static t_bool add_filter_rule(struct t_group *group, struct t_article *art, struct t_filter_rule *rule, t_bool quick_filter_rule);
+static t_bool add_filter_rule(const struct t_group *group, struct t_article *art, struct t_filter_rule *rule, t_bool quick_filter_rule);
 static int test_match(const char *string, char *regex, t_bool nocase, t_bool use_regex, struct regex_cache *cache);
 static void expand_filter_array(struct t_filters *ptr);
 static void fmt_filter_menu_prompt(char *dest, size_t dest_len, const char *fmt_str, int len, const char *text);
@@ -322,7 +322,7 @@ read_filter_file(
 	char scbuf[PATH_LEN];
 	int i = 0;
 	int icase = 0;
-	int score = 0;
+	int score;
 	long secs = 0L;
 	struct t_filter_comment *comment = NULL;
 	struct t_filter *ptr = NULL;
@@ -333,28 +333,37 @@ read_filter_file(
 	static t_bool first_read = TRUE;
 	struct t_version *upgrade = NULL;
 
+	/* Reset all filter arrays */
+	free_filter_array(&glob_filter);
+
 	if ((fp = tin_fopen(file, "r")) == NULL)
 		return FALSE;
 
 	if (!batch_mode || verbose)
 		wait_message(0, _(txt_reading_filter_file), file);
 
-	(void) time(&current_secs);
-
-	/*
-	 * Reset all filter arrays if doing a reread of the active file
-	 */
-	if (!first_read)
-		free_filter_array(&glob_filter);
-
 	filter_file_offset = 1;
 	scope[0] = '\0';
+	(void) time(&current_secs);
+
 	while (fgets(buf, (int) sizeof(buf), fp) != NULL) {
 		if (*buf == '\n')
 			continue;
+		/* sanity check for mixed up -F/-f */
+		if (!i && first_read && no_version_line) {
+			/*
+			 * we know that a filter-file line is either a comment (#) or
+			 * has at least an equal signe ('=') whereas a newsrc-file line
+			 * has either a ':' or a '!'
+			 */
+			if (*buf != '#' && !strchr(buf, '=') && (strchr(buf, ':') || strchr(buf, '!'))) {
+				fclose(fp);
+				tin_done(EXIT_FAILURE, _(txt_error_mixed_up_opt), "-F", file);
+			}
+		}
 		if (*buf == '#') {
-			if (scope[0] == '\0')
-				filter_file_offset++;
+			if (!*scope)
+				++filter_file_offset;
 			if (upgrade == NULL && first_read && match_string(buf, "# Filter file V", NULL, 0)) {
 				first_read = FALSE;
 				no_version_line = FALSE;
@@ -682,7 +691,8 @@ write_filter_file(
 		return;
 
 	/* generate tmp-filename */
-	file_tmp = get_tmpfilename(filename);
+	if ((file_tmp = get_tmpfilename(filename)) == NULL)
+		return;
 
 	if (!backup_file(filename, file_tmp)) {
 		error_message(2, _(txt_filesystem_full_backup), filename);
@@ -715,7 +725,7 @@ write_filter_file(
 		filter_file_offset = 1;
 		while ((i = fgetc(fp)) != EOF) {
 			if (i == '\n')
-				filter_file_offset++;
+				++filter_file_offset;
 		}
 		if (fseek(fp, fpos, SEEK_SET)) {
 			clearerr(fp);
@@ -890,7 +900,7 @@ get_choice(
 	int x,
 	const char *help,
 	const char *prompt,
-	char *list[],
+	const char *list[],
 	int list_size)
 {
 	int ch, y, i = 0;
@@ -911,7 +921,7 @@ get_choice(
 		ch = ReadCh();
 		switch (ch) {
 			case ' ':
-				i++;
+				++i;
 				i %= list_size;
 				break;
 
@@ -921,14 +931,14 @@ get_choice(
 #endif /* HAVE_KEY_PREFIX */
 				switch (get_arrow_key(ch)) {
 					case KEYMAP_UP:
-						i--;
+						--i;
 						if (i < 0)
 							i = list_size - 1;
 						ch = ' ';	/* don't exit the while loop yet */
 						break;
 
 					case KEYMAP_DOWN:
-						i++;
+						++i;
 						i %= list_size;
 						ch = ' ';	/* don't exit the while loop yet */
 						break;
@@ -1061,9 +1071,9 @@ filter_menu(
 	const char *ptr_filter_subj;
 	const char *ptr_filter_help_scope;
 	const char *ptr_filter_quit_edit_save;
+	const char **list;
 	char *ptr;
-	char **list;
-	char comment_line[LEN];
+	char *prompt_buf = NULL;
 	char buf[LEN];
 	char keyedit[MAXKEYLEN], keyquit[MAXKEYLEN], keysave[MAXKEYLEN];
 	char text_time[PATH_LEN];
@@ -1073,13 +1083,14 @@ filter_menu(
 	struct t_filter_rule rule;
 	t_bool proceed;
 	t_bool ret;
+	t_bool has_path_filter = filter_on_path(group);
 	t_function func, default_func = FILTER_SAVE;
 
 	signal_context = cFilter;
 
 	rule.comment = (struct t_filter_comment *) 0;
-	rule.text[0] = '\0';
-	rule.scope[0] = '\0';
+	rule.text = NULL;
+	rule.scope = NULL;
 	rule.counter = 0;
 	rule.lines_cmp = FILTER_LINES_NO;
 	rule.lines_num = 0;
@@ -1093,8 +1104,6 @@ filter_menu(
 	rule.expire_time = FALSE;
 	rule.check_string = FALSE;
 
-	comment_line[0] = '\0';
-
 	/*
 	 * setup correct text for user selected menu
 	 */
@@ -1141,10 +1150,10 @@ filter_menu(
 
 	len = cCOLS - flen - clen - 1 + 4;
 
-	snprintf(text_time, sizeof(text_time), _(txt_time_default_days), tinrc.filter_days);
+	snprintf(text_time, sizeof(text_time), P_(txt_time_default_day_sp[0], txt_time_default_day_sp[1], tinrc.filter_days), tinrc.filter_days);
 	fmt_filter_menu_prompt(text_subj, sizeof(text_subj), ptr_filter_subj, len, art->subject);
 	snprintf(text_score, sizeof(text_score), _(txt_filter_score), (type == GLOBAL_MENU_FILTER_KILL ? tinrc.score_kill : tinrc.score_select));
-	fmt_filter_menu_prompt(text_from, sizeof(text_from), ptr_filter_from, len, art->from);
+	fmt_filter_menu_prompt(text_from, sizeof(text_from), ptr_filter_from, len, art->mailbox.from);
 	fmt_filter_menu_prompt(text_msgid, sizeof(text_msgid), ptr_filter_msgid, len - 4, MSGID(art));
 
 	print_filter_menu();
@@ -1155,10 +1164,13 @@ filter_menu(
 	 * The empty line is ignored.
 	 */
 	show_menu_help(_(txt_help_filter_comment));
-	while ((proceed = prompt_menu_string(INDEX_TOP, ptr_filter_comment, comment_line)) && comment_line[0] != '\0') {
-		rule.comment = add_filter_comment(rule.comment, comment_line);
-		comment_line[0] = '\0';
+	while ((proceed = prompt_menu_string(INDEX_TOP, ptr_filter_comment, &prompt_buf)) && prompt_buf && *prompt_buf) {
+		rule.comment = add_filter_comment(rule.comment, prompt_buf);
+		FreeAndNull(prompt_buf);
 	}
+
+	FreeAndNull(prompt_buf);
+
 	if (!proceed) {
 		free_filter_comment(rule.comment);
 		return FALSE;
@@ -1168,26 +1180,29 @@ filter_menu(
 	 * Text which might be used to filter on subj, from or msgid
 	 */
 	show_menu_help(_(txt_help_filter_text));
-	if (!prompt_menu_string(INDEX_TOP + 2, ptr_filter_text, rule.text)) {
+	if (!prompt_menu_string(INDEX_TOP + 2, ptr_filter_text, &rule.text)) {
 		free_filter_comment(rule.comment);
 		return FALSE;
 	}
 
-	if (*rule.text) {
-		list = my_malloc(sizeof(char *) * 8);
-		list[0] = (char *) _(txt_subj_line_only_case);
-		list[1] = (char *) _(txt_subj_line_only);
-		list[2] = (char *) _(txt_from_line_only_case);
-		list[3] = (char *) _(txt_from_line_only);
-		list[4] = (char *) _(txt_msgid_refs_line);
-		list[5] = (char *) _(txt_msgid_line_last);
-		list[6] = (char *) _(txt_msgid_line_only);
-		list[7] = (char *) _(txt_refs_line_only);
+	if (rule.text && *rule.text) {
+		list = my_malloc(sizeof(char *) * FILTER_LIST_MAX);
+		list[FILTER_SUBJ_CASE_SENSITIVE] = _(txt_subj_line_only_case);
+		list[FILTER_SUBJ_CASE_IGNORE] = _(txt_subj_line_only);
+		list[FILTER_FROM_CASE_SENSITIVE] = _(txt_from_line_only_case);
+		list[FILTER_FROM_CASE_IGNORE] = _(txt_from_line_only);
+		list[FILTER_MSGID] = _(txt_msgid_refs_line);
+		list[FILTER_MSGID_LAST] = _(txt_msgid_line_last);
+		list[FILTER_MSGID_ONLY] = _(txt_msgid_line_only);
+		list[FILTER_REFS_ONLY] = _(txt_refs_line_only);
+		list[FILTER_XREF_CASE_IGNORE] = _(txt_xref_line_nocasse); /* RFC 5536 3.1.4 "... SHOULD NOT contain uppercase letters." */
+		list[FILTER_PATH_CASE_IGNORE] = _(txt_path_line_nocasse); /* RFC 5537 3.2 "... SHOULD compare identities case-insensitively." */
 
-		i = get_choice(INDEX_TOP + 3, _(txt_help_filter_text_type), _(txt_filter_text_type), list, 8);
+		i = get_choice(INDEX_TOP + 3, _(txt_help_filter_text_type), _(txt_filter_text_type), list, FILTER_LIST_MAX);
 		free(list);
 
 		if (i == -1) {
+			free(rule.text);
 			free_filter_comment(rule.comment);
 			return FALSE;
 		}
@@ -1196,6 +1211,8 @@ filter_menu(
 		switch (i) {
 			case FILTER_SUBJ_CASE_IGNORE:
 			case FILTER_FROM_CASE_IGNORE:
+			case FILTER_PATH_CASE_IGNORE:
+			case FILTER_XREF_CASE_IGNORE:
 				rule.icase = TRUE;
 				break;
 
@@ -1214,18 +1231,19 @@ filter_menu(
 		}
 	}
 
-	if (!*rule.text) {
+	if (!rule.text || !*rule.text) {
 		rule.check_string = TRUE;
 		/*
 		 * Subject:
 		 */
 		list = my_malloc(sizeof(char *) * 2);
-		list[0] = (char *) _(txt_yes);
-		list[1] = (char *) _(txt_no);
+		list[0] = _(txt_yes);
+		list[1] = _(txt_no);
 		i = get_choice(INDEX_TOP + 5, _(txt_help_filter_subj), text_subj, list, 2);
 		free(list);
 
 		if (i == -1) {
+			FreeIfNeeded(rule.text);
 			free_filter_comment(rule.comment);
 			return FALSE;
 		} else
@@ -1236,16 +1254,17 @@ filter_menu(
 		 */
 		list = my_malloc(sizeof(char *) * 2);
 		if (rule.subj_ok) {
-			list[0] = (char *) _(txt_no);
-			list[1] = (char *) _(txt_yes);
+			list[0] = _(txt_no);
+			list[1] = _(txt_yes);
 		} else {
-			list[0] = (char *) _(txt_yes);
-			list[1] = (char *) _(txt_no);
+			list[0] = _(txt_yes);
+			list[1] = _(txt_no);
 		}
 		i = get_choice(INDEX_TOP + 6, _(txt_help_filter_from), text_from, list, 2);
 		free(list);
 
 		if (i == -1) {
+			FreeIfNeeded(rule.text);
 			free_filter_comment(rule.comment);
 			return FALSE;
 		} else
@@ -1256,20 +1275,21 @@ filter_menu(
 		 */
 		list = my_malloc(sizeof(char *) * 4);
 		if (rule.subj_ok || rule.from_ok) {
-			list[0] = (char *) _(txt_no);
-			list[1] = (char *) _(txt_full);
-			list[2] = (char *) _(txt_last);
-			list[3] = (char *) _(txt_only);
+			list[0] = _(txt_no);
+			list[1] = _(txt_full);
+			list[2] = _(txt_last);
+			list[3] = _(txt_only);
 		} else {
-			list[0] = (char *) _(txt_full);
-			list[1] = (char *) _(txt_last);
-			list[2] = (char *) _(txt_only);
-			list[3] = (char *) _(txt_no);
+			list[0] = _(txt_full);
+			list[1] = _(txt_last);
+			list[2] = _(txt_only);
+			list[3] = _(txt_no);
 		}
 		i = get_choice(INDEX_TOP + 7, _(txt_help_filter_msgid), text_msgid, list, 4);
 		free(list);
 
 		if (i == -1) {
+			FreeIfNeeded(rule.text);
 			free_filter_comment(rule.comment);
 			return FALSE;
 		} else {
@@ -1301,7 +1321,6 @@ filter_menu(
 					break;
 			}
 		}
-
 	}
 
 	/*
@@ -1309,9 +1328,8 @@ filter_menu(
 	 */
 	show_menu_help(_(txt_help_filter_lines));
 
-	buf[0] = '\0';
-
-	if (!prompt_menu_string(INDEX_TOP + 9, ptr_filter_lines, buf)) {
+	if (!prompt_menu_string(INDEX_TOP + 9, ptr_filter_lines, &prompt_buf)) {
+		FreeIfNeeded(rule.text);
 		free_filter_comment(rule.comment);
 		return FALSE;
 	}
@@ -1319,24 +1337,26 @@ filter_menu(
 	/*
 	 * Get the < > sign if any for the lines rule
 	 */
-	ptr = buf;
+	ptr = prompt_buf;
 	while (*ptr == ' ')
-		ptr++;
+		++ptr;
 
 	if (*ptr == '>') {
 		rule.lines_cmp = FILTER_LINES_GT;
-		ptr++;
+		++ptr;
 	} else if (*ptr == '<') {
 		rule.lines_cmp = FILTER_LINES_LT;
-		ptr++;
+		++ptr;
 	} else if (*ptr == '=') {
 		rule.lines_cmp = FILTER_LINES_EQ;
-		ptr++;
+		++ptr;
 	}
 
 	if (*ptr)
 		rule.lines_num = s2i(ptr, 0, INT_MAX);
 
+	FreeAndNull(prompt_buf);
+
 	if (rule.lines_num && rule.lines_cmp == FILTER_LINES_NO)
 		rule.lines_cmp = FILTER_LINES_EQ;
 
@@ -1349,16 +1369,16 @@ filter_menu(
 	snprintf(buf, sizeof(buf), _(txt_filter_score_help), SCORE_MAX);
 	show_menu_help(buf);
 
-	buf[0] = '\0';
-	if (!prompt_menu_string(INDEX_TOP + 10, text_score, buf)) {
+	if (!prompt_menu_string(INDEX_TOP + 10, text_score, &prompt_buf)) {
+		FreeIfNeeded(rule.text);
 		free_filter_comment(rule.comment);
 		return FALSE;
 	}
 
 	/* check if a score has been entered */
-	if (buf[0] != '\0')
+	if (*prompt_buf)
 		/* use entered score */
-		rule.score = s2i(buf, -SCORE_MAX, SCORE_MAX);
+		rule.score = s2i(prompt_buf, -SCORE_MAX, SCORE_MAX);
 	else {
 		/* use default score */
 		if (type == GLOBAL_MENU_FILTER_KILL)
@@ -1367,7 +1387,10 @@ filter_menu(
 			rule.score = tinrc.score_select;
 	}
 
+	FreeAndNull(prompt_buf);
+
 	if (!rule.score) { /* ignore 0 scores */
+		FreeIfNeeded(rule.text);
 		free_filter_comment(rule.comment);
 		return FALSE;
 	}
@@ -1392,7 +1415,7 @@ filter_menu(
 	sprintf(double_time, "2x %s", text_time);
 	sprintf(quat_time, "4x %s", text_time);
 	list = my_malloc(sizeof(char *) * 4);
-	list[0] = (char *) _(txt_unlimited_time);
+	list[0] = _(txt_unlimited_time);
 	list[1] = text_time;
 	list[2] = double_time;
 	list[3] = quat_time;
@@ -1402,6 +1425,7 @@ filter_menu(
 	free(quat_time);
 
 	if (i == -1) {
+		FreeIfNeeded(rule.text);
 		free_filter_comment(rule.comment);
 		return FALSE;
 	}
@@ -1411,35 +1435,43 @@ filter_menu(
 	/*
 	 * Scope
 	 */
-	if (*rule.text || rule.subj_ok || rule.from_ok || rule.msgid_ok || rule.lines_ok) {
+	if ((rule.text && *rule.text) || rule.subj_ok || rule.from_ok || rule.msgid_ok || rule.lines_ok) {
+		char *grp_name = my_strdup(group->name);
+		char **tmp_list;
 		int j = 0;
 
 		list = my_malloc(sizeof(char *) * 2); /* at least 2 scopes */
-		list[j++] = my_strdup(group->name);
-		list[j] = my_strdup(list[j - 1]);
-		while ((ptr = strrchr(list[j], '.')) != NULL) {
+		tmp_list = my_malloc(sizeof(char *) * 2);
+		list[j] = tmp_list[j] = my_strdup(grp_name);
+		++j;
+		while ((ptr = strrchr(grp_name, '.')) != NULL) {
 			*(++ptr) = '*';
 			*(++ptr) = '\0';
-			j++;
-			list = my_realloc(list, sizeof(char *) * (size_t) (j + 1)); /* one element more */
-			list[j] = my_strdup(list[j - 1]);
-			list[j][strlen(list[j]) - 2] = '\0';
+			list[j] = tmp_list[j] = my_strdup(grp_name);
+			list = my_realloc(list, sizeof(char *) * (size_t) (j + 2)); /* one element more */
+			tmp_list = my_realloc(tmp_list, sizeof(char *) * (size_t) (j + 2));
+			++j;
+			grp_name[strlen(grp_name) - 2] = '\0';
 		}
-		free(list[j]); /* this copy isn't needed anymore */
-		list[j] = (char *) _(txt_all_groups);
+		free(grp_name);
+		list[j] = tmp_list[j] = my_strdup(_(txt_all_groups));
 
 		if ((i = get_choice(INDEX_TOP + 13, ptr_filter_help_scope, ptr_filter_scope, list, j + 1)) > 0)
-			my_strncpy(rule.scope, i == j ? "*" : list[i], sizeof(rule.scope) - 1);
+			rule.scope = my_strdup(i == j ? "*" : list[i]);
 
-		for (j--; j >= 0; j--)
-			free(list[j]);
+		while (j >= 0)
+			free(tmp_list[j--]);
+		free(tmp_list);
 		free(list);
 
 		if (i == -1) {
+			FreeIfNeeded(rule.text);
+			FreeIfNeeded(rule.scope);
 			free_filter_comment(rule.comment);
 			return FALSE;
 		}
 	} else {
+		FreeIfNeeded(rule.text);
 		free_filter_comment(rule.comment);
 		return FALSE;
 	}
@@ -1451,31 +1483,42 @@ filter_menu(
 
 		case FILTER_EDIT:
 			add_filter_rule(group, art, &rule, FALSE); /* save the rule */
+			FreeIfNeeded(rule.text);
+			FreeIfNeeded(rule.scope);
 			rule.comment = free_filter_comment(rule.comment);
 			if (!invoke_editor(filter_file, filter_file_offset, NULL))
 				return FALSE;
 			unfilter_articles(group);
 			(void) read_filter_file(filter_file);
 			return TRUE;
-			/* keep lint quiet: */
-			/* FALLTHROUGH */
+			/* NOTREACHED */
+			break;
 
 		case GLOBAL_QUIT:
 		case GLOBAL_ABORT:
+			FreeIfNeeded(rule.text);
+			FreeIfNeeded(rule.scope);
 			free_filter_comment(rule.comment);
 			return FALSE;
-			/* keep lint quiet: */
-			/* FALLTHROUGH */
+			/* NOTREACHED */
+			break;
 
 		case FILTER_SAVE:
 			/*
 			 * Add the filter rule and save it to the filter file
 			 */
 			ret = add_filter_rule(group, art, &rule, FALSE);
+			if (!art->path && !has_path_filter && filter_on_path(group)) { /* we don't have Path-data yet */
+				index_group(group); /* fetch Path:-data */
+				if (tinrc.cache_overview_files) /* update cache */
+					write_overview(group);
+			}
+			FreeIfNeeded(rule.text);
+			FreeIfNeeded(rule.scope);
 			free_filter_comment(rule.comment);
 			return ret;
-			/* keep lint quiet: */
-			/* FALLTHROUGH */
+			/* NOTREACHED */
+			break;
 
 		default:
 			break;
@@ -1495,7 +1538,7 @@ quick_filter(
 	struct t_group *group,
 	struct t_article *art)
 {
-	char *scope;
+	const char *scope;
 	char txt[LEN];
 	int header, expire;
 	struct t_filter_rule rule;
@@ -1506,26 +1549,24 @@ quick_filter(
 		expire = group->attribute->quick_kill_expire;
 		/* ON=case sensitive, OFF=ignore case -> invert */
 		icase = bool_not(group->attribute->quick_kill_case);
-		scope = group->attribute->quick_kill_scope;
+		scope = group->attribute->quick_kill_scope ? *group->attribute->quick_kill_scope : "";
 	} else {	/* type == GLOBAL_QUICK_FILTER_SELECT */
 		header = group->attribute->quick_select_header;
 		expire = group->attribute->quick_select_expire;
 		/* ON=case sensitive, OFF=ignore case -> invert */
 		icase = bool_not(group->attribute->quick_select_case);
-		scope = group->attribute->quick_select_scope;
+		scope = group->attribute->quick_select_scope ? *group->attribute->quick_select_scope : "";
 	}
 
 #ifdef DEBUG
 	if (debug & DEBUG_FILTER)
-		error_message(2, "%s header=[%d] scope=[%s] expire=[%s] case=[%c]", (type == GLOBAL_QUICK_FILTER_KILL) ? "KILL" : "SELECT", header, BlankIfNull(scope), txt_onoff[expire != FALSE ? 1 : 0], icase ? "I" : "C");
+		error_message(2, "%s header=[%d] scope=[%s] expire=[%s] case=[%c]", (type == GLOBAL_QUICK_FILTER_KILL) ? "KILL" : "SELECT", header, scope, txt_onoff[expire != FALSE ? 1 : 0], icase ? "I" : "C");
 #endif /* DEBUG */
 
 	/*
 	 * Setup rules
 	 */
-	if (strlen(BlankIfNull(scope)) > (sizeof(rule.scope) - 1))
-		return FALSE;
-	my_strncpy(rule.scope, BlankIfNull(scope), sizeof(rule.scope) - 1);
+	rule.scope = my_strdup(scope);
 	rule.counter = 0;
 	rule.lines_cmp = FILTER_LINES_NO;
 	rule.lines_num = 0;
@@ -1542,13 +1583,14 @@ quick_filter(
 		snprintf(txt, sizeof(txt), "%s%s%c%s%s%s", _(txt_filter_rule_created), "'", '[', "' (", _(txt_help_article_quick_select), ").");
 	rule.comment = add_filter_comment(NULL, txt);
 
-	rule.text[0] = '\0';
+	rule.text = NULL;
 	rule.icase = icase ? 1 : 0;
 	rule.expire_time = expire;
 	rule.check_string = TRUE;
 	rule.score = (type == GLOBAL_QUICK_FILTER_KILL) ? tinrc.score_kill : tinrc.score_select;
 
 	ret = add_filter_rule(group, art, &rule, TRUE);
+	free(rule.scope);
 	free_filter_comment(rule.comment);
 	return ret;
 }
@@ -1561,7 +1603,7 @@ quick_filter(
  */
 t_bool
 quick_filter_select_posted_art(
-	struct t_group *group,
+	const struct t_group *group,
 	const char *subj,
 	const char *a_message_id)	/* return value is always ignored */
 {
@@ -1572,13 +1614,6 @@ quick_filter_select_posted_art(
 		struct t_article art;
 		struct t_filter_rule rule;
 
-#ifdef __cplusplus /* keep C++ quiet */
-		rule.scope[0] = '\0';
-#endif /* __cplusplus */
-
-		if (strlen(group->name) > (sizeof(rule.scope) - 1)) /* groupname to long? */
-			return FALSE;
-
 		/*
 		 * Setup rules
 		 */
@@ -1590,13 +1625,14 @@ quick_filter_select_posted_art(
 		rule.msgid_ok = FALSE;
 		rule.fullref = FILTER_MSGID;
 		rule.subj_ok = TRUE;
-		rule.text[0] = '\0';
+		rule.text = NULL;
+		rule.scope = NULL;
 		rule.icase = FALSE;
 		rule.expire_time = TRUE;
 		rule.check_string = TRUE;
 		rule.score = tinrc.score_select;
 
-		strcpy(rule.scope, group->name);
+		rule.scope = my_strdup(group->name);
 
 		/* create an auto-comment. */
 		snprintf(txt, sizeof(txt), "%s%s", _(txt_filter_rule_created), "add_posted_to_filter=ON.");
@@ -1635,6 +1671,7 @@ quick_filter_select_posted_art(
 			filtered = add_filter_rule(group, &art, &rule, FALSE);
 			FreeIfNeeded(art.subject);
 		}
+		free(rule.scope);
 		rule.comment = free_filter_comment(rule.comment);
 	}
 	return filtered;
@@ -1646,7 +1683,7 @@ quick_filter_select_posted_art(
  */
 static t_bool
 add_filter_rule(
-	struct t_group *group,
+	const struct t_group *group,
 	struct t_article *art,
 	struct t_filter_rule *rule,
 	t_bool quick_filter_rule)
@@ -1682,7 +1719,7 @@ add_filter_rule(
 	if (rule->comment != NULL)
 		ptr[i].comment = copy_filter_comment(rule->comment, ptr[i].comment);
 
-	if (rule->scope[0] == '\0') /* replace empty scope with current group name */
+	if (!rule->scope || !*rule->scope) /* replace empty scope with current group name */
 		ptr[i].scope = my_strdup(group->name);
 	else {
 		if ((rule->scope[0] != '*') && (rule->scope[1] != '\0')) /* copy non-global scope */
@@ -1709,7 +1746,7 @@ add_filter_rule(
 	}
 
 	ptr[i].icase = rule->icase;
-	if (*rule->text) {
+	if (rule->text && *rule->text) {
 		snprintf(acbuf, sizeof(acbuf), REGEX_FMT, quote_wild_whitespace(rule->text));
 
 		switch (rule->counter) {
@@ -1731,6 +1768,16 @@ add_filter_rule(
 				ptr[i].fullref = rule->counter;
 				break;
 
+			case FILTER_XREF_CASE_IGNORE:
+				ptr[i].xref = my_strdup(acbuf);
+				str_lwr(ptr[i].xref);
+				break;
+
+			case FILTER_PATH_CASE_IGNORE:
+				ptr[i].path = my_strdup(acbuf);
+				str_lwr(ptr[i].path);
+				break;
+
 			default: /* should not happen */
 				/* CONSTANTCONDITION */
 				assert(0 != 0);
@@ -1749,7 +1796,7 @@ add_filter_rule(
 			ptr[i].subj = my_strdup(acbuf);
 		}
 		if (rule->from_ok) {
-			STRCPY(sbuf, art->from);
+			STRCPY(sbuf, art->mailbox.from);
 			snprintf(acbuf, sizeof(acbuf), REGEX_FMT, quote_wild(sbuf));
 			ptr[i].from = my_strdup(acbuf);
 		}
@@ -1867,7 +1914,7 @@ filter_articles(
 	 * ie. group=comp.os.linux.help  scope=comp.os.linux.*
 	 */
 	inscope = set_filter_scope(group);
-	if (!cmd_line && !batch_mode)
+	if (!cmd_line && !batch_mode) /* FIXME: txt_filter_global_rules always uses plural-form "rules" */
 		wait_message(0, _(txt_filter_global_rules), inscope, group->glob_filter->num);
 	num = group->glob_filter->num;
 	ptr = group->glob_filter->filter;
@@ -1877,9 +1924,8 @@ filter_articles(
 	 * (only for regexp matching)
 	 */
 	if (use_regex) {
-		size_t msiz;
+		size_t msiz = sizeof(struct regex_cache) * (size_t) num;
 
-		msiz = sizeof(struct regex_cache) * (size_t) num;
 		regex_cache_subj = my_malloc(msiz);
 		regex_cache_from = my_malloc(msiz);
 		regex_cache_msgid = my_malloc(msiz);
@@ -1935,10 +1981,10 @@ filter_articles(
 				 *       as that is what we use elsewhere?
 				 */
 				if (ptr[j].from != NULL) {
-					if (arts[i].name != NULL)
-						snprintf(buf, sizeof(buf), "%s (%s)", arts[i].from, arts[i].name);
+					if (arts[i].mailbox.name != NULL)
+						snprintf(buf, sizeof(buf), "%s (%s)", arts[i].mailbox.from, arts[i].mailbox.name);
 					else
-						STRCPY(buf, arts[i].from);
+						STRCPY(buf, arts[i].mailbox.from);
 
 					switch (test_match(buf, ptr[j].from, ptr[j].icase, use_regex, &regex_cache_from[j])) {
 						case 1:
@@ -1962,8 +2008,8 @@ filter_articles(
 				 */
 				if (ptr[j].msgid != NULL) {
 					char *refs = NULL;
-					const char *myrefs = NULL;
-					const char *mymsgid = NULL;
+					const char *myrefs;
+					const char *mymsgid;
 					int x;
 					struct t_article *art = &arts[i];
 					/*
@@ -2057,22 +2103,22 @@ filter_articles(
 				/*
 				 * Filter on GNKSA code
 				 */
-				if ((ptr[j].gnksa_cmp != FILTER_LINES_NO) && (arts[i].gnksa_code >= 0)) {
+				if ((ptr[j].gnksa_cmp != FILTER_LINES_NO) && (arts[i].mailbox.gnksa_code >= 0)) {
 					switch (ptr[j].gnksa_cmp) {
 						case FILTER_LINES_EQ:
-							if (arts[i].gnksa_code == ptr[j].gnksa_num) {
+							if (arts[i].mailbox.gnksa_code == ptr[j].gnksa_num) {
 								SET_FILTER(group, i, j);
 							}
 							break;
 
 						case FILTER_LINES_LT:
-							if (arts[i].gnksa_code < ptr[j].gnksa_num) {
+							if (arts[i].mailbox.gnksa_code < ptr[j].gnksa_num) {
 								SET_FILTER(group, i, j);
 							}
 							break;
 
 						case FILTER_LINES_GT:
-							if (arts[i].gnksa_code > ptr[j].gnksa_num) {
+							if (arts[i].mailbox.gnksa_code > ptr[j].gnksa_num) {
 								SET_FILTER(group, i, j);
 							}
 							break;
@@ -2091,15 +2137,15 @@ filter_articles(
 				 */
 				if (arts[i].xref && *arts[i].xref) {
 					if (ptr[j].score && ptr[j].xref != NULL) {
-						char *s, *e, *k;
+						char *e, *k;
+						const char *s = arts[i].xref;
 						t_bool skip = FALSE;
 
-						s = arts[i].xref;
 						if (strchr(s, ' ') || strchr(s, '\t')) {
 							while (*s && !isspace((unsigned char) *s))	/* skip server name */
-								s++;
+								++s;
 							while (*s && isspace((unsigned char) *s))
-								s++;
+								++s;
 						}
 #ifdef DEBUG
 						else { /* server name missing in overview, i.e. colobus 2.1 */
@@ -2121,7 +2167,7 @@ filter_articles(
 									*e++ = *s;
 								if (isspace((unsigned char) *s))
 									skip = FALSE;
-								s++;
+								++s;
 							}
 							if (e == k) {
 #ifdef DEBUG
@@ -2161,7 +2207,7 @@ filter_articles(
 				/*
 				 * Filter on Path: lines
 				 */
-				if (arts[i].path && *arts[i].path) {
+				if (arts[i].path) {
 					if (ptr[j].path != NULL) {
 						switch (test_match(arts[i].path, ptr[j].path, ptr[j].icase, use_regex, &regex_cache_path[j])) {
 							case 1:
@@ -2217,7 +2263,7 @@ filter_articles(
 					arts[i].killed = ART_KILLED;
 				filtered = TRUE;
 				art_mark(group, &arts[i], ART_READ);
-				if (group->attribute->show_only_unread_arts)
+				if (group->attribute && group->attribute->show_only_unread_arts)
 					arts[i].keep_in_base = FALSE;
 			} else if (arts[i].score >= tinrc.score_limit_select) {
 				arts[i].selected = TRUE;
@@ -2274,7 +2320,7 @@ set_filter_scope(
 		if (ptr[i].scope != NULL) {
 			if (!match_group_list(group->name, ptr[i].scope)) {
 				ptr[i].inscope = FALSE;
-				inscope--;
+				--inscope;
 			}
 		}
 	}
