@@ -3,10 +3,10 @@
  *  Module    : group.c
  *  Author    : I. Lea & R. Skrenta
  *  Created   : 1991-04-01
- *  Updated   : 2024-02-22
+ *  Updated   : 2025-02-25
  *  Notes     :
  *
- * Copyright (c) 1991-2024 Iain Lea <iain@bricbrac.de>, Rich Skrenta <skrenta@pbm.com>
+ * Copyright (c) 1991-2025 Iain Lea <iain@bricbrac.de>, Rich Skrenta <skrenta@pbm.com>
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -61,7 +61,7 @@ static struct t_fmt grp_fmt;
 static int do_search(t_function func, t_bool repeat);
 static int enter_pager(int art, t_bool ignore_unavail);
 static int enter_thread(int depth, t_pagerinfo *page);
-static int find_new_pos(long old_artnum, int cur_pos);
+static int find_new_pos(t_artnum old_artnum, int cur_pos);
 static int group_catchup(t_function func);
 static int tab_pressed(void);
 static t_bool prompt_getart_limit(void);
@@ -337,7 +337,7 @@ group_page(
 
 			case GROUP_CANCEL:	/* cancel current basenote */
 				if (grpmenu.curr >= 0) {
-					if (can_post || group->attribute->mailing_list != NULL) {
+					if (can_post || (group->attribute->mailing_list && *group->attribute->mailing_list)) {
 						n = (int) base[grpmenu.curr];
 						ii = art_open(TRUE, &arts[n], group, &pgart, TRUE, _(txt_reading_article));
 						if (ii != ART_UNAVAILABLE && ii != ART_ABORT && cancel_article(group, &arts[n], n))
@@ -378,8 +378,8 @@ group_page(
 				if (invoke_editor(filter_file, filter_file_offset, NULL)) {
 					old_artnum = grpmenu.max > 0 ? arts[(int) base[grpmenu.curr]].artnum : T_ARTNUM_CONST(-1);
 					unfilter_articles(group);
-					(void) read_filter_file(filter_file);
-					filter_articles(group);
+					if (read_filter_file(filter_file))
+						filter_articles(group);
 					make_threads(group, FALSE);
 					grpmenu.curr = old_artnum >= T_ARTNUM_CONST(0) ? find_new_pos(old_artnum, grpmenu.curr) : grpmenu.max - 1;
 				}
@@ -469,6 +469,7 @@ group_page(
 
 			case GLOBAL_TOGGLE_INVERSE_VIDEO:		/* toggle inverse video */
 				toggle_inverse_video();
+				need_parse_fmt |= GROUP_LEVEL;
 				show_group_page();
 				show_inverse_video_status();
 				break;
@@ -500,8 +501,43 @@ group_page(
 				break;
 
 			case GLOBAL_LOOKUP_MESSAGEID:
-				if ((i = prompt_msgid()) != ART_UNAVAILABLE)
-					ret_code = enter_pager(i, FALSE);
+				{
+					t_bool only_unread = group->attribute->show_only_unread_arts;
+
+					if (only_unread)
+						toggle_read_unread(FALSE);
+					if ((i = prompt_msgid()) != ART_UNAVAILABLE)
+						ret_code = enter_pager(i, FALSE);
+#ifdef NNTP_ABLE
+					else {
+						const char *eyde = input_history[HIST_MESSAGE_ID][(hist_pos[HIST_MESSAGE_ID] - 1 + HIST_SIZE) % HIST_SIZE];
+						int sc = selmenu.curr;
+
+						if (*eyde) {
+							switch (show_article_by_msgid(eyde)) {
+								case LOOKUP_OK:
+									selmenu.curr = sc;
+									if (!index_group(curr_group))
+										ret_code = GRP_RETSELECT; /* TODO: return GRP_RETSELECT; ? */
+									break;
+
+								case LOOKUP_UNAVAIL: /* check for !reading_saved_news? */
+									wait_message(2, _(txt_lookup_func_not_available));
+									break;
+
+								default:
+									wait_message(2, _(txt_art_unavailable));
+									break;
+							}
+						}
+					}
+#endif /* NNTP_ABLE */
+					if (ret_code == 0) {
+						if (only_unread)
+							toggle_read_unread(FALSE);
+						show_group_page();
+					}
+				}
 				break;
 
 			case GLOBAL_OPTION_MENU:			/* option menu */
@@ -599,7 +635,7 @@ group_page(
 					 * tinrc.getart_limit
 					 */
 					if (cmdline.args & CMDLINE_GETART_LIMIT)
-						cmdline.args &= ~CMDLINE_GETART_LIMIT;
+						cmdline.args ^= CMDLINE_GETART_LIMIT;
 					ret_code = GRP_NEXTUNREAD;
 				}
 				break;
@@ -610,9 +646,9 @@ group_page(
 
 			case GROUP_TAG_PARTS: /* tag all in order */
 				if (0 <= grpmenu.curr) {
-					int old_num = num_of_tagged_arts;
-
 					if (tag_multipart((int) base[grpmenu.curr]) != 0) {
+						int old_num = num_of_tagged_arts;
+
 						/*
 						 * on success, move the pointer to the next
 						 * untagged article just for ease of use's sake
@@ -620,7 +656,7 @@ group_page(
 						n = grpmenu.curr;
 						update_group_page();
 						do {
-							n++;
+							++n;
 							n %= grpmenu.max;
 							if (arts[base[n]].tagged == 0) {
 								move_to_item(n);
@@ -693,7 +729,7 @@ group_page(
 						stat_thread(grpmenu.curr, &sbuf);
 						mark[0] = sbuf.art_mark;
 #if defined(MULTIBYTE_ABLE) && !defined(NO_LOCALE)
-						mark_screen(grpmenu.curr, mark_offset - (3 - art_mark_width), L"   "); /* clear space used by tag numbering */
+						mark_screen(grpmenu.curr, mark_offset - (3 - art_mark_width), (const wchar_t *) L"   "); /* clear space used by tag numbering */
 						mark_screen(grpmenu.curr, mark_offset + (art_mark_width - wcwidth(mark[0])), mark);
 #else
 						mark_screen(grpmenu.curr, mark_offset - 2, "  "); /* clear space used by tag numbering */
@@ -708,8 +744,7 @@ group_page(
 					else
 						draw_subject_arrow();
 
-					info_message(tagged ? _(txt_prefix_untagged) : _(txt_prefix_tagged), txt_thread_singular);
-
+					info_message(tagged ? _(txt_prefix_untagged) : _(txt_prefix_tagged), txt_thread_sp[0]);
 				}
 				break;
 
@@ -864,13 +899,16 @@ group_page(
 				if (grpmenu.curr >= 0) {
 					char pat[LEN];
 					char *prompt;
+					char *spat;
 					struct regex_cache cache = REGEX_CACHE_INITIALIZER;
 
-					prompt = fmt_string(_(txt_select_pattern), tinrc.default_select_pattern);
-					if (!(prompt_string_default(prompt, tinrc.default_select_pattern, _(txt_info_no_previous_expression), HIST_SELECT_PATTERN))) {
+					prompt = fmt_string(_(txt_select_pattern), BlankIfNull(tinrc.default_select_pattern));
+					if (!(spat = prompt_string_default(prompt, tinrc.default_select_pattern, _(txt_info_no_previous_expression), HIST_SELECT_PATTERN))) {
+						FreeIfNeeded(spat);
 						free(prompt);
 						break;
 					}
+					tinrc.default_select_pattern = spat;
 					free(prompt);
 
 					if (STRCMPEQ(tinrc.default_select_pattern, "*")) {	/* all */
@@ -936,7 +974,8 @@ group_page(
 
 			case GLOBAL_TOGGLE_INFO_LAST_LINE:
 				tinrc.info_in_last_line = bool_not(tinrc.info_in_last_line);
-				show_group_page();
+				clear_message();
+				draw_subject_arrow();
 				break;
 
 			default:
@@ -952,6 +991,9 @@ group_page(
 
 	art_close(&pgart);				/* Close any open art */
 
+	FreeAndNull(grp_fmt.str);
+	FreeAndNull(grp_fmt.date_str);
+
 	curr_group = NULL;
 
 	return ret_code;
@@ -967,10 +1009,14 @@ show_group_page(
 	signal_context = cGroup;
 	currmenu = &grpmenu;
 
+	if (!grp_fmt.str || (need_parse_fmt & GROUP_LEVEL)) {
+		parse_format_string(curr_group->attribute->group_format ? BlankIfNull(*curr_group->attribute->group_format) : "", &grp_fmt);
+		need_parse_fmt &= ~GROUP_LEVEL;
+		mark_offset = 0;
+	}
+
 	ClearScreen();
 	set_first_screen_item();
-	parse_format_string(curr_group->attribute->group_format, &grp_fmt);
-	mark_offset = 0;
 	show_group_title(FALSE);
 
 	for (i = grpmenu.first; i < grpmenu.first + NOTESLINES && i < grpmenu.max; ++i)
@@ -1014,7 +1060,7 @@ update_group_page(
 			stat_thread(i, &sbuf);
 			mark[0] = sbuf.art_mark;
 #if defined(MULTIBYTE_ABLE) && !defined(NO_LOCALE)
-			mark_screen(i, mark_offset - (3 - art_mark_width), L"   ");	/* clear space used by tag numbering */
+			mark_screen(i, mark_offset - (3 - art_mark_width), (const wchar_t *) L"   ");	/* clear space used by tag numbering */
 			mark_screen(i, mark_offset + (art_mark_width - wcwidth(mark[0])), mark);
 #else
 			mark_screen(i, mark_offset - 2, "  ");	/* clear space used by tag numbering */
@@ -1044,7 +1090,7 @@ draw_subject_arrow(
 
 		stat_thread(grpmenu.curr, &statbuf);
 		if (!statbuf.unread || (n = next_unread((int) base[grpmenu.curr])) == -1)
-			n = base[grpmenu.curr];
+			n = (int) base[grpmenu.curr];
 		info_message("%s", arts[n].subject);
 	} else if (grpmenu.curr == grpmenu.max - 1)
 		info_message(_(txt_end_of_arts));
@@ -1083,7 +1129,7 @@ toggle_read_unread(
 	if (force)
 		curr_group->attribute->show_only_unread_arts = TRUE;	/* Yes - really, we change it in a bit */
 
-	wait_message(0, _(txt_reading_arts),
+	wait_message(0, _(txt_reading_arts), /* TODO: plural-forms? ("unread" fr:("non lu", "non lus")) */
 		(curr_group->attribute->show_only_unread_arts) ? _(txt_all) : _(txt_unread));
 
 	if (grpmenu.curr >= 0) {
@@ -1114,7 +1160,7 @@ toggle_read_unread(
  */
 static int
 find_new_pos(
-	long old_artnum,
+	t_artnum old_artnum,
 	int cur_pos)
 {
 	int i, pos;
@@ -1157,7 +1203,6 @@ mark_screen(
 	const char *value)
 #endif /* MULTIBYTE_ABLE && !NO_LOCALE */
 {
-
 	if (tinrc.draw_arrow) {
 		MoveCursor(INDEX2LNUM(screen_row), screen_col);
 #if defined(MULTIBYTE_ABLE) && !defined(NO_LOCALE)
@@ -1268,9 +1313,9 @@ build_sline(
 	 * make it the same size like in !USE_CURSES case to simplify the code
 	 */
 #	if defined(MULTIBYTE_ABLE) && !defined(NO_LOCALE)
-		buffer = my_malloc(cCOLS * MB_CUR_MAX + 2);
+		buffer = my_malloc((size_t) cCOLS * MB_CUR_MAX + 2);
 #	else
-		buffer = my_malloc(cCOLS + 2);
+		buffer = my_malloc((size_t) cCOLS + 2);
 #	endif /* MULTIBYTE_ABLE && !NO_LOCALE */
 #else
 	buffer = screen[INDEX2SNUM(i)].col;
@@ -1310,11 +1355,11 @@ build_sline(
 				if (my_strftime(buf, LEN - 1, grp_fmt.date_str, localtime((const time_t *) &arts[j].date))) {
 #if defined(MULTIBYTE_ABLE) && !defined(NO_LOCALE)
 					if ((wtmp = char2wchar_t(buf)) != NULL) {
-						wtmp2 = wcspart(wtmp, (int) grp_fmt.len_date_max, TRUE);
+						wtmp2 = wcspart(wtmp, grp_fmt.len_date_max, TRUE);
+						free(wtmp);
 						if (wcstombs(tmp, wtmp2, sizeof(tmp) - 1) != (size_t) -1)
 							strcat(buffer, tmp);
 
-						free(wtmp);
 						free(wtmp2);
 					}
 #else
@@ -1330,13 +1375,14 @@ build_sline(
 					get_author(FALSE, &arts[j], tmp, sizeof(tmp) - 1);
 
 					if ((wtmp = char2wchar_t(tmp)) != NULL) {
-						wtmp2 = wcspart(wtmp, (int) grp_fmt.len_from, TRUE);
-						if (wcstombs(tmp, wtmp2, sizeof(tmp) - 1) != (size_t) -1)
-							strcat(buffer, tmp);
-
+						wtmp2 = wcspart(wtmp, grp_fmt.len_from, TRUE);
 						free(wtmp);
-						free(wtmp2);
-					}
+					} else
+						wtmp2 = wcspart(L"", grp_fmt.len_from, TRUE);
+					if (wcstombs(tmp, wtmp2, sizeof(tmp) - 1) != (size_t) -1)
+						strcat(buffer, tmp);
+
+					free(wtmp2);
 #else
 					len_start = strwidth(buffer);
 					get_author(FALSE, &arts[j], buffer + strlen(buffer), grp_fmt.len_from);
@@ -1363,7 +1409,7 @@ build_sline(
 
 			case 'L':	/* lines */
 				if (arts[j].line_count != -1)
-					strcat(buffer, tin_ltoa(arts[j].line_count, (int) grp_fmt.len_linecnt));
+					strcat(buffer, tin_ltoa(arts[j].line_count, grp_fmt.len_linecnt));
 				else {
 					buf = buffer + strlen(buffer);
 					for (k = (int) grp_fmt.len_linecnt; k > 1; --k)
@@ -1406,13 +1452,13 @@ build_sline(
 				break;
 
 			case 'n':
-				strcat(buffer, tin_ltoa(i + 1, (int) grp_fmt.len_linenumber));
+				strcat(buffer, tin_ltoa(i + 1, grp_fmt.len_linenumber));
 				break;
 
 			case 'R':
 				n = ((curr_group->attribute->show_only_unread_arts) ? (sbuf.unread + sbuf.seen) : sbuf.total);
 				if (n > 1)
-					strcat(buffer, tin_ltoa(n, (int) grp_fmt.len_respcnt));
+					strcat(buffer, tin_ltoa(n, grp_fmt.len_respcnt));
 				else {
 					buf = buffer + strlen(buffer);
 					for (k = (int) grp_fmt.len_respcnt; k > 0; --k)
@@ -1422,7 +1468,7 @@ build_sline(
 				break;
 
 			case 'S':	/* score */
-				strcat(buffer, tin_ltoa(sbuf.score, (int) grp_fmt.len_score));
+				strcat(buffer, tin_ltoa(sbuf.score, grp_fmt.len_score));
 				break;
 
 			case 's':	/* thread/subject */
@@ -1435,13 +1481,15 @@ build_sline(
 
 #if defined(MULTIBYTE_ABLE) && !defined(NO_LOCALE)
 				if ((wtmp = char2wchar_t(arts_sub)) != NULL) {
-					wtmp2 = wcspart(wtmp, (int) len, TRUE);
-					if (wcstombs(tmp, wtmp2, sizeof(tmp) - 1) != (size_t) -1)
-						strcat(buffer, tmp);
-
+					wtmp2 = wcspart(wtmp, len, TRUE);
 					free(wtmp);
-					free(wtmp2);
-				}
+				} else
+					wtmp2 = wcspart(L"", len, TRUE);
+
+				if (wcstombs(tmp, wtmp2, sizeof(tmp) - 1) != (size_t) -1)
+					strcat(buffer, tmp);
+
+				free(wtmp2);
 #else
 				len_start = strwidth(buffer);
 				strncat(buffer, arts_sub, len);
@@ -1492,25 +1540,25 @@ show_group_title(
 
 		if (curr_group->attribute->show_only_unread_arts) {
 			if (arts[i].status != ART_READ) {
-				art_cnt++;
+				++art_cnt;
 				if (tinrc.recent_time && ((time((time_t *) 0) - arts[i].date) < (tinrc.recent_time * DAY)))
-					recent_art_cnt++;
+					++recent_art_cnt;
 			}
 			if (arts[i].killed == ART_KILLED_UNREAD)
-				killed_art_cnt++;
+				++killed_art_cnt;
 		} else {
-			art_cnt++;
+			++art_cnt;
 			if (tinrc.recent_time && ((time((time_t *) 0) - arts[i].date) < (tinrc.recent_time * DAY)))
-				recent_art_cnt++;
+				++recent_art_cnt;
 
 			if (arts[i].killed)
-				killed_art_cnt++;
+				++killed_art_cnt;
 		}
 		if (arts[i].selected) {
 			if (arts[i].status != ART_READ)
-				selected_art_cnt++;
+				++selected_art_cnt;
 			else
-				read_selected_art_cnt++;
+				++read_selected_art_cnt;
 		}
 	}
 
@@ -1585,6 +1633,8 @@ show_group_title(
 	} else
 		len = cCOLS - strwidth(buf) - 2;
 
+	if (len < 0)
+		len = 0;
 	/* group name */
 #if defined(MULTIBYTE_ABLE) && !defined(NO_LOCALE)
 	if ((wtmp = char2wchar_t(curr_group->name)) != NULL) {
@@ -1592,8 +1642,8 @@ show_group_title(
 			wtmp2 = abbr_wcsgroupname(wtmp, len);
 		else
 			wtmp2 = wstrunc(wtmp, len);
-		grpname = wchar_t2char(wtmp2);
 		free(wtmp);
+		grpname = wchar_t2char(wtmp2);
 		free(wtmp2);
 		if (grpname) {
 			STRCPY(tmp, grpname);
@@ -1802,13 +1852,9 @@ group_catchup(
 
 				case 1:						/* We caught up - advance group */
 					return GRP_NEXT;
-					/* NOTREACHED */
-					break;
 
 				default:					/* Just leave the group */
 					return GRP_EXIT;
-					/* NOTREACHED */
-					break;
 			}
 			/* FALLTHROUGH */
 		default:							/* Should not be here */
@@ -1822,13 +1868,14 @@ static t_bool
 prompt_getart_limit(
 	void)
 {
-	char *p;
+	const char *p;
 	t_bool ret = FALSE;
 
 	clear_message();
 	if ((p = tin_getline(_(txt_enter_getart_limit), 2, NULL, 0, FALSE, HIST_OTHER)) != NULL) {
-		tinrc.getart_limit = atoi(p);
-		ret = TRUE;
+		tinrc.getart_limit = s2i(p, INT_MIN, INT_MAX);
+		if (!errno)
+			ret = TRUE;
 	}
 	clear_message();
 	return ret;
