@@ -3,9 +3,9 @@
  *  Module    : header.c
  *  Author    : Urs Janssen <urs@tin.org>
  *  Created   : 1997-03-10
- *  Updated   : 2024-08-28
+ *  Updated   : 2025-04-07
  *
- * Copyright (c) 1997-2024 Urs Janssen <urs@tin.org>
+ * Copyright (c) 1997-2025 Urs Janssen <urs@tin.org>
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -44,7 +44,6 @@
 #endif /* !TNNTP_H */
 
 static const char *get_full_name(void);
-static const char *get_user_name(void);
 
 
 /* find hostname */
@@ -162,9 +161,14 @@ get_fqdn(
 			in.s_addr = (*hp->h_addr);
 #		endif /* HAVE_HOSTENT_H_ADDR_LIST */
 		}
+#		ifdef HAVE_INET_NTOA
 		return (hp && strchr(hp->h_name, '.') ? hp->h_name : inet_ntoa(in));
+#		else
+		return (hp && strchr(hp->h_name, '.') ? hp->h_name : "");
+#		endif /* HAVE_INET_NTOA */
 	}
 #	endif /* HAVE_INET_ADDR */
+
 	if ((hp = gethostbyname(name)) && !strchr(hp->h_name, '.')) {
 #	ifdef HAVE_HOSTENT_H_ADDR_LIST
 		if ((hp = gethostbyaddr(hp->h_addr_list[0], hp->h_length, hp->h_addrtype)))
@@ -174,10 +178,11 @@ get_fqdn(
 			in.s_addr = (*hp->h_addr);
 #	endif /* HAVE_HOSTENT_H_ADDR_LIST */
 	}
-	snprintf(fqdn, sizeof(fqdn), "%s", hp
-		? strchr(hp->h_name, '.')
-			? hp->h_name : inet_ntoa(in)
-		: "");
+#	ifdef HAVE_INET_NTOA
+	snprintf(fqdn, sizeof(fqdn), "%s", hp ? strchr(hp->h_name, '.') ? hp->h_name : inet_ntoa(in) : "");
+#	else
+	snprintf(fqdn, sizeof(fqdn), "%s", hp ? strchr(hp->h_name, '.') ? hp->h_name : "" : "");
+#	endif /* HAVE_INET_NTOA */
 
 	if (!*fqdn || (fqdn[strlen(fqdn) - 1] <= '9')) { /* see FIXME above about IPv6 */
 		FILE *inf;
@@ -214,46 +219,6 @@ get_fqdn(
 #endif /* HAVE_GETHOSTBYNAME */
 
 
-/*
- * Find username & fullname
- */
-void
-get_user_info(
-	char *user_name,
-	char *full_name)
-{
-	const char *ptr;
-
-	user_name[0] = '\0';
-	full_name[0] = '\0';
-
-	if ((ptr = get_full_name()))
-		strcpy(full_name, ptr);
-	if ((ptr = get_user_name()))
-		strcpy(user_name, ptr);
-}
-
-
-static const char *
-get_user_name(
-	void)
-{
-	static char username[128];
-	struct passwd *pw;
-
-	username[0] = '\0';
-
-	if ((pw = getpwuid(getuid())) != NULL)
-		STRCPY(username, pw->pw_name);
-	else {
-		if (!*username)
-			tin_done(EXIT_FAILURE, _(txt_error_passwd_missing));
-	}
-
-	return username;
-}
-
-
 static const char *
 get_full_name(
 	void)
@@ -263,8 +228,7 @@ get_full_name(
 	static char fullname[128];
 #ifndef DONT_HAVE_PW_GECOS
 	char buf[128];
-	char tmp[128];
-	struct passwd *pw;
+	const struct passwd *pw;
 #endif /* !DONT_HAVE_PW_GECOS */
 
 	fullname[0] = '\0';
@@ -281,19 +245,20 @@ get_full_name(
 
 #ifndef DONT_HAVE_PW_GECOS
 	if ((pw = getpwuid(getuid())) != NULL) {
-		int ret;
-
 		STRCPY(buf, pw->pw_gecos);
-		if ((p = strchr(buf, ',')))
+		if ((p = strchr(buf, ','))) /* cut off office, wphone, hphone */
 			*p = '\0';
 		if ((p = strchr(buf, '&'))) {
+			char tmp[128];
+			int ret;
+
 			*p++ = '\0';
 			STRCPY(tmp, pw->pw_name);
 			if (*tmp && isalpha((unsigned char) *tmp) && islower((unsigned char) *tmp))
 				*tmp = (char) my_toupper((unsigned char) *tmp);
 			ret = snprintf(fullname, sizeof(fullname), "%s%s%s", buf, tmp, p);
 			if (ret == -1 || ret > (int) sizeof(fullname))
-				error_message(2, "Fullname truncated");
+				error_message(2, "Fullname truncated"); /* -> lang.c */
 		} else
 			STRCPY(fullname, buf);
 	}
@@ -312,20 +277,18 @@ get_full_name(
 void
 get_from_name(
 	char *from_name,
-	struct t_group *thisgrp)
+	const struct t_group *thisgrp)
 {
-	char *fromhost = domain_name;
+	const char *fromhost = domain_name;
 
-	if (thisgrp && thisgrp->attribute->from && *thisgrp->attribute->from) {
+	if (thisgrp && thisgrp->attribute->from && *thisgrp->attribute->from && strchr(*thisgrp->attribute->from, '@'))
 		strcpy(from_name, *thisgrp->attribute->from);
-		return;
-	}
-
-	sprintf(from_name, ((strpbrk(get_full_name(), "!()<>@,;:\\\".[]")) ? "\"%s\" <%s@%s>" : "%s <%s@%s>"), BlankIfNull(get_full_name()), BlankIfNull(get_user_name()), BlankIfNull(fromhost));
+	else
+		sprintf(from_name, ((strpbrk(get_full_name(), "!()<>@,;:\\\".[]")) ? "\"%s\" <%s@%s>" : "%s <%s@%s>"), BlankIfNull(get_full_name()), userid, BlankIfNull(fromhost));
 
 #ifdef DEBUG
 	if (debug & DEBUG_MISC)
-		error_message(2, "FROM=[%s] USER=[%s] HOST=[%s] NAME=[%s]", from_name, BlankIfNull(get_user_name()), domain_name, BlankIfNull(get_full_name()));
+		error_message(2, "FROM=[%s] USER=[%s] HOST=[%s] NAME=[%s]", from_name, userid, BlankIfNull(fromhost), BlankIfNull(get_full_name()));
 #endif /* DEBUG */
 }
 
@@ -339,29 +302,28 @@ char *
 build_sender(
 	void)
 {
-	const char *ptr = NULL;
-	static char sender[8192];
+	const char *ptr;
+	static char sender[HEADER_LEN];
 
 	sender[0] = '\0';
 
-	if ((ptr = get_full_name())) /* TODO: rfc2047 encode */
+	ptr = get_full_name();
+	if (*ptr) /* TODO: rfc2047 encode */
 		snprintf(sender, sizeof(sender), ((strpbrk(ptr, "\".:;<>@[]()\\")) ? "\"%s\"" : "%s "), ptr);
-	if ((ptr = get_user_name())) {
-		snprintf(sender + strlen(sender), sizeof(sender) - strlen(sender), "<%s@", ptr);
+
+	snprintf(sender + strlen(sender), sizeof(sender) - strlen(sender), "<%.*s@", LOGIN_NAME_MAX, userid);
 
 #	ifdef HAVE_GETHOSTBYNAME
-		ptr = get_fqdn(get_host_name());
+	ptr = get_fqdn(get_host_name());
 #	else
-		ptr = get_host_name();
+	ptr = get_host_name();
 #	endif /* HAVE_GETHOSTBYNAME */
 
 		/* intentionally do not fall back to *domain_name */
 
-		if (*ptr)
-			snprintf(sender + strlen(sender), sizeof(sender) - strlen(sender), "%s>", ptr);
-		else
-			return NULL;
-	} else
+	if (*ptr)
+		snprintf(sender + strlen(sender), sizeof(sender) - strlen(sender), "%s>", ptr);
+	else
 		return NULL;
 
 	return sender;
