@@ -3,10 +3,10 @@
  *  Module    : memory.c
  *  Author    : I. Lea & R. Skrenta
  *  Created   : 1991-04-01
- *  Updated   : 2023-12-06
+ *  Updated   : 2025-11-16
  *  Notes     :
  *
- * Copyright (c) 1991-2024 Iain Lea <iain@bricbrac.de>, Rich Skrenta <skrenta@pbm.com>
+ * Copyright (c) 1991-2026 Iain Lea <iain@bricbrac.de>, Rich Skrenta <skrenta@pbm.com>
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -74,10 +74,16 @@ struct t_save *save;			/* sorts articles before saving them */
  */
 static void free_active_arrays(void);
 static void free_attributes(struct t_attribute *attributes);
+static void free_cmdline_args(void);
 static void free_scopes_arrays(void);
+static void free_tinrc_strings(void);
+static void free_serverrc_strings(void);
 static void free_newnews_array(void);
-static void free_if_not_default(char **attrib, const char *deflt);
+static void free_if_not_default(char ***attrib, char **deflt);
 static void free_input_history(void);
+#ifdef NNTP_ABLE
+	static void free_distrib_pats(void);
+#endif /* NNTP_ABLE */
 
 
 /*
@@ -99,7 +105,7 @@ init_alloc(
 
 	active = my_malloc(sizeof(*active) * (size_t) max_active);
 	newnews = my_malloc(sizeof(*newnews) * (size_t) max_newnews);
-	my_group = my_calloc(1, sizeof(int) * (size_t) max_active);
+	my_group = my_calloc((size_t) max_active, sizeof(int));
 
 	/*
 	 * article headers array
@@ -107,10 +113,10 @@ init_alloc(
 	max_art = DEFAULT_ARTICLE_NUM;
 	max_base = DEFAULT_ARTICLE_NUM;
 
-	arts = my_calloc(1, sizeof(*arts) * (size_t) max_art);
+	arts = my_calloc((size_t) max_art, sizeof(*arts));
 	base = my_malloc(sizeof(t_artnum) * (size_t) max_base);
 
-	ofmt = my_calloc(1, sizeof(*ofmt) * 9);	/* initial number of overview fields */
+	ofmt = my_calloc(9, sizeof(*ofmt));	/* initial number of overview fields */
 
 	/*
 	 * save file array
@@ -143,8 +149,10 @@ expand_art(
 		 * memset(&arts[i].artnum, 0, (max_art - i - 1) * sizeof(*arts));
 		 * seems to be not faster at all
 		 */
-	for (; i < max_art; i++)
-		arts[i].subject = arts[i].from = arts[i].xref = arts[i].path = arts[i].refs = arts[i].msgid = NULL;
+	for (; i < max_art; i++) {
+		arts[i].subject = arts[i].xref = arts[i].path = arts[i].refs = arts[i].msgid = arts[i].mailbox.from = arts[i].mailbox.name = NULL;
+		arts[i].mailbox.next = NULL;
+	}
 }
 
 
@@ -155,7 +163,7 @@ expand_active(
 	max_active += max_active >> 1;		/* increase by 50% */
 	if (active == NULL) {
 		active = my_malloc(sizeof(*active) * (size_t) max_active);
-		my_group = my_calloc(1, sizeof(int) * (size_t) max_active);
+		my_group = my_calloc((size_t) max_active, sizeof(int));
 	} else {
 		active = my_realloc(active, sizeof(*active) * (size_t) max_active);
 		my_group = my_realloc(my_group, sizeof(int) * (size_t) max_active);
@@ -217,9 +225,9 @@ init_screen_array(
 
 		for (i = 0; i < cLINES; i++) {
 #	if defined(MULTIBYTE_ABLE) && !defined(NO_LOCALE)
-			screen[i].col = my_malloc((size_t) ((size_t) cCOLS * MB_CUR_MAX + 2));
+			screen[i].col = my_malloc((size_t) cCOLS * MB_CUR_MAX + 2);
 #	else
-			screen[i].col = my_malloc((size_t) ((size_t) cCOLS + 2));
+			screen[i].col = my_malloc((size_t) cCOLS + 2);
 #	endif /* MULTIBYTE_ABLE && !NO_LOCALE */
 		}
 	} else {
@@ -245,12 +253,17 @@ free_all_arrays(
 		init_screen_array(FALSE);
 #endif /* !USE_CURSES */
 
+#ifdef NNTP_ABLE
+	free_distrib_pats();
+#endif /* NNTP_ABLE */
 	free_art_array();
 	free_msgids();
 	FreeAndNull(arts);
 	free_filter_array(&glob_filter);
 	free_active_arrays();
 	free_scopes_arrays();
+	free_tinrc_strings();
+	free_serverrc_strings();
 
 #ifdef HAVE_COLOR
 	regex_cache_destroy(&quote_regex);
@@ -266,12 +279,19 @@ free_all_arrays(
 	regex_cache_destroy(&strip_was_regex);
 	regex_cache_destroy(&uubegin_regex);
 	regex_cache_destroy(&uubody_regex);
+	regex_cache_destroy(&xxbody_regex);
 	regex_cache_destroy(&verbatim_begin_regex);
 	regex_cache_destroy(&verbatim_end_regex);
+	regex_cache_destroy(&yencbegin_regex);
+	regex_cache_destroy(&yencpart_regex);
+	regex_cache_destroy(&yencend_regex);
 	regex_cache_destroy(&url_regex);
 	regex_cache_destroy(&mail_regex);
 	regex_cache_destroy(&news_regex);
 	regex_cache_destroy(&shar_regex);
+	regex_cache_destroy(&shar_end_regex);
+	regex_cache_destroy(&hideline_regex);
+	regex_cache_destroy(&latex_regex);
 
 	if (!batch_mode) {
 		free_keymaps();
@@ -293,6 +313,8 @@ free_all_arrays(
 	FreeAndNull(nntp_caps.headers_range);
 	FreeAndNull(nntp_caps.headers_id);
 	FreeAndNull(nntp_caps.implementation);
+	FreeAndNull(nntp_caps.sasl_mechs);
+	FreeAndNull(nntp_caps.sasl_mech_used);
 
 	if (ofmt) { /* ofmt might not be allocated yet on early abort */
 		int i;
@@ -306,6 +328,7 @@ free_all_arrays(
 	rfc1522_decode(NULL);
 
 	free(backup_article_name);
+	free_cmdline_args();
 	free(tin_progname);
 }
 
@@ -320,7 +343,7 @@ free_art_array(
 		arts[i].artnum = T_ARTNUM_CONST(0);
 		arts[i].date = (time_t) 0;
 		FreeAndNull(arts[i].xref);
-		FreeAndNull(arts[i].path);
+		free_mailbox_list(arts[i].mailbox.next);
 
 		/*
 		 * .refs & .msgid are usually free()d in build_references()
@@ -341,16 +364,18 @@ free_art_array(
 
 
 /*
- * Use this only for attributes that have a fixed default of a static string
- * in tinrc
+ * Use this for attributes that have a default in tinrc
+ * otherwise use (char **) 0 as deflt
  */
 static void
 free_if_not_default(
-	char **attrib,
-	const char *deflt)
+	char ***attrib,
+	char **deflt)
 {
-	if (*attrib != deflt)
+	if (*attrib && *attrib != deflt) {
+		FreeAndNull(**attrib);
 		FreeAndNull(*attrib);
+	}
 }
 
 
@@ -361,48 +386,150 @@ static void
 free_attributes(
 	struct t_attribute *attributes)
 {
-	free_if_not_default(&attributes->group_format, tinrc.group_format);
-	free_if_not_default(&attributes->thread_format, tinrc.thread_format);
-	free_if_not_default(&attributes->date_format, tinrc.date_format);
-	free_if_not_default(&attributes->editor_format, tinrc.editor_format);
-	FreeAndNull(attributes->fcc);
-	free_if_not_default(&attributes->from, tinrc.mail_address);
-	FreeAndNull(attributes->followup_to);
+	free_if_not_default(&attributes->group_format, &tinrc.group_format);
+	free_if_not_default(&attributes->thread_format, &tinrc.thread_format);
+	free_if_not_default(&attributes->date_format, &tinrc.date_format);
+	free_if_not_default(&attributes->editor_format, &tinrc.editor_format);
+	free_if_not_default(&attributes->fcc, (char **) 0);
+	free_if_not_default(&attributes->from, &tinrc.mail_address);
+	free_if_not_default(&attributes->followup_to, (char **) 0);
 #ifdef HAVE_ISPELL
-	FreeAndNull(attributes->ispell);
+	free_if_not_default(&attributes->ispell, (char **) 0);
 #endif /* HAVE_ISPELL */
-	free_if_not_default(&attributes->maildir, tinrc.maildir);
-	FreeAndNull(attributes->mailing_list);
-	FreeAndNull(attributes->mime_types_to_save);
-	free_if_not_default(&attributes->news_headers_to_display, tinrc.news_headers_to_display);
-	free_if_not_default(&attributes->news_headers_to_not_display, tinrc.news_headers_to_not_display);
+	free_if_not_default(&attributes->maildir, &tinrc.maildir);
+	free_if_not_default(&attributes->mailing_list, (char **) 0);
+	free_if_not_default(&attributes->mime_types_to_save, &default_mime_types_to_save);
+	free_if_not_default(&attributes->news_headers_to_display, &tinrc.news_headers_to_display);
+	free_if_not_default(&attributes->news_headers_to_not_display, &tinrc.news_headers_to_not_display);
 	if (attributes->headers_to_display) {
 		if (attributes->headers_to_display->header)
 			FreeIfNeeded(*attributes->headers_to_display->header);
 		FreeAndNull(attributes->headers_to_display->header);
 		free(attributes->headers_to_display);
-		attributes->headers_to_display = (struct t_newsheader *) 0;
+		attributes->headers_to_display = NULL;
 	}
 	if (attributes->headers_to_not_display) {
 		if (attributes->headers_to_not_display->header)
 			FreeIfNeeded(*attributes->headers_to_not_display->header);
 		FreeAndNull(attributes->headers_to_not_display->header);
 		free(attributes->headers_to_not_display);
-		attributes->headers_to_not_display = (struct t_newsheader *) 0;
+		attributes->headers_to_not_display = NULL;
 	}
-	free_if_not_default(&attributes->news_quote_format, tinrc.news_quote_format);
-	free_if_not_default(&attributes->organization, default_organization);
-	FreeAndNull(attributes->quick_kill_scope);
-	FreeAndNull(attributes->quick_select_scope);
-	free_if_not_default(&attributes->quote_chars, tinrc.quote_chars);
-	free_if_not_default(&attributes->savedir, tinrc.savedir);
-	FreeAndNull(attributes->savefile);
-	free_if_not_default(&attributes->sigfile, tinrc.sigfile);
+	free_if_not_default(&attributes->news_quote_format, &tinrc.news_quote_format);
+	free_if_not_default(&attributes->organization, &default_organization);
+	free_if_not_default(&attributes->quick_kill_scope, &default_filter_kill_global);
+	free_if_not_default(&attributes->quick_select_scope, &default_filter_select_global);
+	free_if_not_default(&attributes->quote_chars, &tinrc.quote_chars);
+	free_if_not_default(&attributes->savedir, &tinrc.savedir);
+	free_if_not_default(&attributes->savefile, (char **) 0);
+	free_if_not_default(&attributes->sigfile, &tinrc.sigfile);
 #ifdef CHARSET_CONVERSION
-	FreeAndNull(attributes->undeclared_charset);
+	free_if_not_default(&attributes->undeclared_charset, (char **) 0);
 #endif /* CHARSET_CONVERSION */
-	FreeAndNull(attributes->x_headers);
-	FreeAndNull(attributes->x_body);
+	free_if_not_default(&attributes->x_headers, (char **) 0);
+	free_if_not_default(&attributes->x_body, (char **) 0);
+}
+
+static void
+free_tinrc_strings(
+		void)
+{
+	FreeAndNull(tinrc.editor_format);
+	FreeAndNull(tinrc.mailer_format);
+	FreeAndNull(tinrc.default_goto_group);
+	FreeAndNull(tinrc.default_mail_address);
+#ifndef DONT_HAVE_PIPING
+	FreeAndNull(tinrc.default_pipe_command);
+#endif /* DONT_HAVE_PIPING */
+	FreeAndNull(tinrc.default_post_newsgroups);
+	FreeAndNull(tinrc.default_post_subject);
+#ifndef DISABLE_PRINTING
+	FreeAndNull(tinrc.printer);
+#endif /* DISABLE_PRINTING */
+	FreeAndNull(tinrc.default_range_group);
+	FreeAndNull(tinrc.default_range_select);
+	FreeAndNull(tinrc.default_range_thread);
+	FreeAndNull(tinrc.default_pattern);
+	FreeAndNull(tinrc.default_repost_group);
+	FreeAndNull(tinrc.default_save_file);
+	FreeAndNull(tinrc.default_search_art);
+	FreeAndNull(tinrc.default_search_author);
+	FreeAndNull(tinrc.default_search_config);
+	FreeAndNull(tinrc.default_search_group);
+	FreeAndNull(tinrc.default_search_subject);
+	FreeAndNull(tinrc.default_select_pattern);
+	FreeAndNull(tinrc.default_shell_command);
+	FreeAndNull(tinrc.maildir);
+	FreeAndNull(tinrc.mail_address);
+	FreeAndNull(tinrc.mail_quote_format);
+	FreeAndNull(tinrc.metamail_prog);
+#ifndef CHARSET_CONVERSION
+	FreeAndNull(tinrc.mm_charset);
+#endif /* !CHARSET_CONVERSION */
+	FreeAndNull(tinrc.mm_local_charset);
+	FreeAndNull(tinrc.news_headers_to_display);
+	FreeAndNull(tinrc.news_headers_to_not_display);
+	FreeAndNull(tinrc.news_quote_format);
+	FreeAndNull(tinrc.quote_chars);
+#ifdef HAVE_COLOR
+	FreeAndNull(tinrc.quote_regex);
+	FreeAndNull(tinrc.quote_regex2);
+	FreeAndNull(tinrc.quote_regex3);
+	FreeAndNull(tinrc.extquote_regex);
+#endif /* HAVE_COLOR */
+	FreeAndNull(tinrc.slashes_regex);
+	FreeAndNull(tinrc.stars_regex);
+	FreeAndNull(tinrc.underscores_regex);
+	FreeAndNull(tinrc.strokes_regex);
+	FreeAndNull(tinrc.strip_re_regex);
+	FreeAndNull(tinrc.strip_was_regex);
+	FreeAndNull(tinrc.verbatim_begin_regex);
+	FreeAndNull(tinrc.verbatim_end_regex);
+	FreeAndNull(tinrc.hideline_regex);
+	FreeAndNull(tinrc.savedir);
+	FreeAndNull(tinrc.sigfile);
+	FreeAndNull(tinrc.spamtrap_warning_addresses);
+	FreeAndNull(tinrc.url_handler);
+	FreeAndNull(tinrc.posted_articles_file);
+	FreeAndNull(tinrc.inews_prog);
+	FreeAndNull(tinrc.select_format);
+	FreeAndNull(tinrc.group_format);
+	FreeAndNull(tinrc.thread_format);
+	FreeAndNull(tinrc.attachment_format);
+	FreeAndNull(tinrc.page_mime_format);
+	FreeAndNull(tinrc.page_uue_format);
+	FreeAndNull(tinrc.page_yenc_format);
+	FreeAndNull(tinrc.date_format);
+	FreeAndNull(tinrc.xpost_quote_format);
+#	ifdef NNTPS_ABLE
+		FreeAndNull(tinrc.tls_ca_cert_file);
+#	endif /* NNTPS_ABLE */
+	FreeAndNull(tinrc.serverrc_add_cmd_line_opts);
+	FreeAndNull(tinrc.serverrc_disabled_nntp_cmds);
+}
+
+
+static void
+free_serverrc_strings(
+	void)
+{
+#if 0
+	FreeAndNull(serverrc.version);
+	FreeAndNull(serverrc.last_newnews);
+#endif /* 0 */
+	FreeAndNull(serverrc.add_cmd_line_opts);
+	FreeAndNull(serverrc.disabled_nntp_cmds);
+}
+
+
+static void
+free_cmdline_args(
+	void)
+{
+	FreeAndNull(cmdline.maildir);
+	FreeAndNull(cmdline.nntpserver);
+	FreeAndNull(cmdline.savedir);
+	FreeAndNull(cmdline.msgid);
 }
 
 
@@ -430,6 +557,27 @@ free_scopes_arrays(
 		free_scope(--num_scope);
 	FreeAndNull(scopes);
 	num_scope = -1;
+	FreeAndNull(default_filter_kill_global);
+	FreeAndNull(default_filter_select_global);
+	FreeAndNull(default_organization);
+	FreeAndNull(default_mime_types_to_save);
+}
+
+
+void
+free_scopes_list(
+	struct t_group *group)
+{
+	struct t_scope_list *p, *q;
+
+	if (!group || !group->scopes_list)
+		return;
+
+	for (p = group->scopes_list; p != NULL; p = q) {
+		q = p->next;
+		free(p);
+	}
+	group->scopes_list = NULL;
 }
 
 
@@ -454,6 +602,7 @@ free_active_arrays(
 				free(active[i].attribute);
 				active[i].attribute = (struct t_attribute *) 0;
 			}
+			free_scopes_list(&active[i]);
 		}
 		FreeAndNull(active);
 	}
@@ -504,6 +653,25 @@ free_input_history(
 }
 
 
+#ifdef NNTP_ABLE
+static void
+free_distrib_pats(
+	void)
+{
+	t_distrib_pat *p, *q;
+
+	for (p = nntp_caps.distrib_pats; p != NULL; p = q) {
+		q = p->next;
+		FreeIfNeeded(p->pattern);
+		free(p->distribution);
+		FreeIfNeeded(p->description);
+		free(p);
+	}
+	nntp_caps.distrib_pats = NULL;
+}
+#endif /* NNTP_ABLE */
+
+
 void *
 my_malloc1(
 	const char *file,
@@ -550,6 +718,13 @@ my_calloc1(
 }
 
 
+/*
+ * TODO: add fallback code with
+ *        q = malloc(nmemb * size); memmove(q, p, size); free(p)?
+ *
+ * for size = 0 and p != NULL, we do what glibc does and
+ * ignore ISO C 23  7.24.3.7 ("undefined behavior").
+ */
 void *
 my_realloc1(
 	const char *file,
@@ -603,7 +778,7 @@ my_memmove(
 		d += n;
 		c += n;
 		while (n--)
-			*--d= *--c;
+			*--d = *--c;
 	} else {
 		while (n--)
 			*d++ = *c++;
