@@ -3,7 +3,7 @@
  *  Module    : my_tmpfile.c
  *  Author    : Urs Janssen <urs@tin.org>
  *  Created   : 2001-03-11
- *  Updated   : 2024-11-23
+ *  Updated   : 2024-12-03
  *  Notes     :
  *
  * Copyright (c) 2001-2025 Urs Janssen <urs@tin.org>
@@ -79,16 +79,18 @@ my_mktmp(
 
 		if (base_dir) {
 #ifdef HAVE_LONG_FILE_NAMES
+			const char *nodenamebuf = BlankIfNull(get_host_name());
+
 			/*
 			 * base_dir may reside on a shared network device
 			 * so add hostname/pid if allowed.
 			 */
-			if ((n = snprintf(NULL, 0, "tin-%s-%ld-XXXXXX", get_host_name(), (long) process_id)) < 0)
+			if ((n = snprintf(NULL, 0, "tin-%s-%ld-XXXXXX", nodenamebuf, (long) process_id)) < 0)
 				goto error_out;
 
 			len = (size_t) n + 1;
 			buf = my_malloc(len);
-			if (snprintf(buf, len, "tin-%s-%ld-XXXXXX", get_host_name(), (long) process_id) != n) {
+			if (snprintf(buf, len, "tin-%s-%ld-XXXXXX", nodenamebuf, (long) process_id) != n) {
 				free(buf);
 				goto error_out;
 			}
@@ -155,7 +157,7 @@ wait_message(5, "snprintf(%s, %d, \"tin_XXXXXX\")=%d", BlankIfNull(buf), len, n)
 		fd = mkstemp(filename);
 		sverrno = errno;
 #	ifdef DEBUG
-		if (fd == -1 || sverrno)
+		if (fd == -1)
 			perror_message("HAVE_MKSTEMP mkstemp(%s)==%d", filename, fd);
 #	endif /* DEBUG */
 #else
@@ -166,7 +168,7 @@ wait_message(5, "snprintf(%s, %d, \"tin_XXXXXX\")=%d", BlankIfNull(buf), len, n)
 			if ((fd = open(buf, (O_RDWR|O_CREAT|O_EXCL), (mode_t) (S_IRUSR|S_IWUSR))) == -1) {
 				sverrno = errno;
 #		ifdef DEBUG
-				if (fd == -1 || sverrno)
+				if (fd == -1)
 					perror_message("HAVE_MKTEMP open(%s)==%d", filename, fd);
 #		endif /* DEBUG */
 			}
