@@ -3,10 +3,10 @@
  *  Module    : newsrc.c
  *  Author    : I. Lea & R. Skrenta
  *  Created   : 1991-04-01
- *  Updated   : 2024-10-31
+ *  Updated   : 2025-06-15
  *  Notes     : ArtCount = (ArtMax - ArtMin) + 1  [could have holes]
  *
- * Copyright (c) 1991-2024 Iain Lea <iain@bricbrac.de>, Rich Skrenta <skrenta@pbm.com>
+ * Copyright (c) 1991-2025 Iain Lea <iain@bricbrac.de>, Rich Skrenta <skrenta@pbm.com>
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -90,7 +90,7 @@ read_newsrc(
 	FILE *fp;
 	char *grp, *seq;
 	int sub, i;
-	signed long line_count = 0;
+	signed long line_count = 0L;
 	struct stat statbuf;
 
 	if (allgroups)
@@ -152,10 +152,16 @@ read_newsrc(
 
 	while ((grp = tin_fgets(fp, FALSE)) != NULL) {
 		strip_line(grp);
+
+		/* mixed up -f / -F ? */
+		if (*grp && !line_count && !strncmp(grp, txt_filter_file_version, 15)) { /* version number starts after pos. 15 */
+			fclose(fp);
+			tin_done(EXIT_FAILURE, _(txt_error_mixed_up_opt), "-f", newsrc_file);
+		}
 		if (*grp == '#' || *grp == '\0')	/* skip comments and empty lines */
 			continue;
 
-		line_count++;	/* but count all other lines (incl. bogus ones) */
+		++line_count;	/* but count all other lines (incl. bogus ones) */
 		seq = parse_newsrc_line(grp, &sub);
 
 		if (sub == SUBSCRIBED) {
@@ -353,19 +359,24 @@ static FILE *
 open_subscription_fp(
 	void)
 {
-	if (!read_saved_news) {
+	if (read_saved_news)
+		return (FILE *) 0;
+	else {
 #ifdef NNTP_ABLE
 		if (read_news_via_nntp) {
 			/* RFC 6048 2.6 */
-			if (nntp_caps.type == CAPABILITIES && !nntp_caps.list_subscriptions)
-				return NULL;
+			if (!nntp_caps.list_subscriptions)
+				return (FILE *) 0;
 			else
 				return (nntp_command("LIST SUBSCRIPTIONS", OK_GROUPS, NULL, 0));
-		} else
+		}
 #endif /* NNTP_ABLE */
-			return (fopen(subscriptions_file, "r"));
-	} else
-		return NULL;
+#ifndef NNTP_ONLY
+		return (fopen(subscriptions_file, "r"));
+#else
+		return (FILE *) 0;
+#endif /* !NNTP_ONLY */
+	}
 }
 
 
@@ -442,26 +453,122 @@ backup_newsrc(
 	char dirbuf[PATH_LEN];
 	char filebuf[PATH_LEN];
 	struct stat statbuf;
+	FILE *fp_in, *fp_out;
 
-#ifdef NNTP_ABLE
-	if (read_news_via_nntp && !read_saved_news && nntp_tcp_port != IPPORT_NNTP)
-		snprintf(filebuf, sizeof(filebuf), "%s:%u", nntp_server, nntp_tcp_port);
-	else
-#endif /* NNTP_ABLE */
-	{
-		STRCPY(filebuf, quote_space_to_dash(nntp_server));
-	}
+	STRCPY(filebuf, quote_space_to_dash(nntp_server));
 	joinpath(dirbuf, sizeof(dirbuf), rcdir, filebuf);
 	joinpath(filebuf, sizeof(filebuf), dirbuf, OLDNEWSRC_FILE);
 
 	if (stat(dirbuf, &statbuf) == -1) {
 		if (my_mkdir(dirbuf, (mode_t) (S_IRWXU)) == -1)
-			/* Can't create directory: Fall back on Homedir */
+			/* Can't create directory: Fall back to homedir */
 			joinpath(filebuf, sizeof(filebuf), homedir, OLDNEWSRC_FILE);
 	}
 
+#if defined(HAVE_REALPATH) && (defined(O_NOFOLLOW) || defined(HAVE_LSTAT))
+	/*
+	 * preserve symlink if target is below homedir (dedian bug #151113)
+	 *
+	 * requires realpath(3) as readlink(3) doesn't canonicalize;
+	 * prefer the O_NOFOLLOW path as that avoids TOCTOU
+	 *
+	 * TODO: return errno
+	 */
+	{
+		t_bool islink = FALSE;
+#	ifdef O_NOFOLLOW
+		int fd;
+#	endif /* O_NOFOLLOW */
+
+		if ((fp_in = tin_fopen(newsrc, "r")) == NULL)
+			return;
+
+#	ifdef O_NOFOLLOW
+		if ((fd = open(filebuf, (O_RDWR|O_CREAT|O_APPEND|O_NOFOLLOW), (S_IRUSR|S_IWUSR))) == -1) {
+			if ((fd = open(filebuf, (O_RDWR|O_CREAT|O_APPEND), (S_IRUSR|S_IWUSR))) == -1) {
+				fclose(fp_in);
+				perror_message(_(txt_cannot_open), filebuf);
+				return;
+			}
+			islink = TRUE;
+		}
+
+		if ((fp_out = fdopen(fd, "a+")) == NULL) {
+#		ifdef HAVE_POSIX_CLOSE
+			posix_close(fd, 0);
+#		else
+			close(fd);
+#		endif /* HAVE_POSIX_CLOSE */
+			fclose(fp_in);
+			perror_message(_(txt_cannot_open), filebuf);
+			return;
+		}
+#	else
+		/* TOCTOU */
+		if ((fp_out = fopen(filebuf, "a+")) == NULL) {
+			fclose(fp_in);
+			perror_message(_(txt_cannot_open), filebuf);
+			return;
+		}
+
+		if (lstat(filebuf, &statbuf) != 0) {
+			fclose(fp_in);
+			fclose(fp_out);
+			perror_message(_(txt_cannot_open), filebuf);
+			return;
+		}
+
+		if (S_ISLNK(statbuf.st_mode))
+			islink = TRUE;
+
+#	endif /* O_NOFOLLOW */
+
+		if (islink) {
+			char target[PATH_LEN] = { '\0' };
+
+			if (realpath(filebuf, target) == NULL) {
+				fclose(fp_in);
+				fclose(fp_out);
+				return;
+			} else {
+				joinpath(dirbuf, sizeof(dirbuf), homedir, ""); /* ensure homedir is '/'-terminated */
+
+				if (strncmp(dirbuf, target, strlen(dirbuf))) { /* disallow symlinks outside homedir */
+#		ifdef DEBUG
+					if (debug & DEBUG_NEWSRC) {
+						char *debug_mesg;
+						int n;
+
+						if ((n = snprintf(NULL, 0, "overwriting %s-link \"%s\", target \"%s\" not below \"%s\"", OLDNEWSRC_FILE, filebuf, target, dirbuf)) > 0) {
+							debug_mesg = my_malloc(++n);
+							snprintf(debug_mesg, n, "overwriting %s-link \"%s\", target \"%s\" not below \"%s\"", OLDNEWSRC_FILE, filebuf, target, dirbuf);
+							debug_print_comment(debug_mesg);
+							error_message(2, "Warning: %s", debug_mesg);
+							free(debug_mesg);
+						}
+					}
+#		endif /* DEBUG */
+					fclose(fp_in);
+					fclose(fp_out);
+					if (!backup_file(newsrc, filebuf))
+						error_message(2, _(txt_filesystem_full_backup), newsrc);
+					return;
+				}
+			}
+		}
+
+		if (ftruncate(fileno(fp_out), 0L) == 0)
+			copy_fp(fp_in, fp_out);
+
+		fclose(fp_in);
+		fclose(fp_out);
+	}
+#else
 	if (!backup_file(newsrc, filebuf))
 		error_message(2, _(txt_filesystem_full_backup), newsrc);
+#endif /* HAVE_REALPATH && (O_NOFOLLOW || HAVE_LSTAT) */
+
+	return;
 }
 
 
@@ -484,6 +591,7 @@ group_get_art_info(
 	if (read_news_via_nntp && grouptype == GROUP_TYPE_NEWS) {
 #ifdef NNTP_ABLE
 		char line[NNTP_STRLEN];
+		int respcode;
 
 		snprintf(line, sizeof(line), "GROUP %s", groupname);
 #	ifdef DEBUG
@@ -491,8 +599,9 @@ group_get_art_info(
 			debug_print_file("NNTP", "group_get_art_info %s", line);
 #	endif /* DEBUG */
 		put_server(line, FALSE);
+		respcode = get_respcode(line, sizeof(line));
 
-		switch (get_respcode(line, sizeof(line))) {
+		switch (respcode) {
 			case OK_GROUP:
 				if (sscanf(line, "%"T_ARTNUM_SFMT" %"T_ARTNUM_SFMT" %"T_ARTNUM_SFMT, art_count, art_min, art_max) != 3) {
 #	ifdef DEBUG
@@ -509,7 +618,7 @@ group_get_art_info(
 				return -ERR_NOGROUP;
 
 			case ERR_ACCESS:
-				tin_done(NNTP_ERROR_EXIT, "%s", line);
+				tin_done(NNTP_ERROR_EXIT, "%d %s (%s)", respcode, line, groupname);
 				/* keep lint quiet: */
 				/* NOTREACHED */
 				break;
@@ -812,7 +921,7 @@ reset_newsrc(
  */
 void
 delete_group(
-	char *group)
+	const char *group)
 {
 	FILE *fp;
 	FILE *newfp;
@@ -984,13 +1093,13 @@ parse_bitmap_seq(
 	 */
 	ptr = seq;
 	while (ptr && *ptr && (*ptr < '0' || *ptr > '9'))
-		ptr++;
+		++ptr;
 
 #ifdef DEBUG
 	if (debug & DEBUG_NEWSRC) {
 		char buf[NEWSRC_LINE];
 
-		snprintf(buf, sizeof(buf), "Parsing [%s%c %.*s]", group->name, SUB_CHAR(group->subscribed), (int) (NEWSRC_LINE - strlen(group->name) - 14), BlankIfNull(ptr));
+		snprintf(buf, sizeof(buf), "\nParsing [%s%c %.*s]", group->name, SUB_CHAR(group->subscribed), (int) (NEWSRC_LINE - strlen(group->name) - 14), BlankIfNull(ptr));
 		debug_print_comment(buf);
 		debug_print_bitmap(group, NULL);
 	}
@@ -1174,13 +1283,13 @@ parse_get_seq(
 	*low = strtoartnum(seq, &seq, 10);
 
 	if (*seq == '-') {	/* Range of articles */
-		seq++;
+		++seq;
 		*high = strtoartnum(seq, &seq, 10);
 	} else	/* Single article */
 		*high = *low;
 
 	while (*seq && (*seq < '0' || *seq > '9'))
-		seq++;
+		++seq;
 
 	return seq;
 }
@@ -1231,7 +1340,7 @@ parse_unread_arts(
 		}
 		while (j < min) {
 			NSET1(newbitmap, j - bitmin);
-			j++;
+			++j;
 		}
 	}
 
@@ -1259,7 +1368,7 @@ parse_unread_arts(
 			if (arts[i].artnum <= group->xmax)
 #endif /* 0 */
 				NSET1(newbitmap, arts[i].artnum - bitmin);
-			unread++;
+			++unread;
 		}
 	}
 
@@ -1314,7 +1423,7 @@ print_bitmap_seq(
 					fprintf(fp, "1");
 				}
 				while (i < group->newsrc.xmax && NTEST(group->newsrc.xbitmap, (i + 1) - group->newsrc.xmin) == ART_READ)
-					i++;
+					++i;
 
 				if (artnum != i)
 					fprintf(fp, "-%"T_ARTNUM_PFMT, i);
@@ -1331,7 +1440,7 @@ print_bitmap_seq(
 			if (group->newsrc.xmax == i)
 				break;
 
-			i++;
+			++i;
 		}
 	}
 
@@ -1357,7 +1466,7 @@ print_bitmap_seq(
  */
 t_bool
 pos_group_in_newsrc(
-	struct t_group *group,
+	const struct t_group *group,
 	int pos)
 {
 	FILE *fp_in = NULL, *fp_out = NULL;
@@ -1501,7 +1610,7 @@ pos_group_in_newsrc(
 
 		fprintf(fp_out, "%s\n", line);
 
-		subscribed_pos++;
+		++subscribed_pos;
 	}
 
 	if (!repositioned)
