@@ -3,9 +3,9 @@
  *  Module    : nntps.c
  *  Author    : E. Berkhan
  *  Created   : 2022-09-10
- *  Updated   : 2024-05-10
+ *  Updated   : 2025-08-14
  *  Notes     : simple abstraction for various TLS implementations
- *  Copyright : (c) Copyright 2022-2024 Enrik Berkhan <Enrik.Berkhan@inka.de>
+ *  Copyright : (c) Copyright 2022-2026 Enrik Berkhan <Enrik.Berkhan@inka.de>
  *              Permission is hereby granted to copy, reproduce, redistribute
  *              or otherwise use this software  as long as: there is no
  *              monetary  profit  gained  specifically  from the use or
@@ -33,9 +33,11 @@ static struct tls_config *libtls_config = NULL;
 static gnutls_certificate_credentials_t tls_xcreds = NULL;
 static char *gnutls_servername = NULL;
 static unsigned gnutls_verification_status = 0;
+static gnutls_datum_t gtls_alpns[] = { { (unsigned char *) NNTP_TCP_NAME, 4 } };
 #	else
 #		ifdef USE_OPENSSL
 static SSL_CTX *openssl_ctx = NULL;
+static unsigned char nntp_alpn[] = { 4, 'n', 'n', 't', 'p' };
 #		endif /* USE_OPENSSL */
 #	endif /* USE_GNUTLS */
 #endif /* USE_LIBTLS */
@@ -68,24 +70,23 @@ int
 tintls_init(
 	void)
 {
-	int result = 0;
 	char *ca_cert_file = ca_cert_file_expanded;
+#ifdef USE_GNUTLS
+	int result;
+#endif /* USE_GNUTLS */
 
-	if (tinrc.tls_ca_cert_file[0] != '\0') {
-		result = strfpath(tinrc.tls_ca_cert_file, ca_cert_file_expanded, sizeof(ca_cert_file_expanded), NULL, FALSE);
-		if (result == 0)
+	if (tinrc.tls_ca_cert_file && *tinrc.tls_ca_cert_file) {
+		if (strfpath(tinrc.tls_ca_cert_file, ca_cert_file_expanded, sizeof(ca_cert_file_expanded), NULL, FALSE) == 0)
 			return -EINVAL;
 	} else
 		ca_cert_file_expanded[0] = '\0';
 
 #ifdef USE_LIBTLS
-
 	/*
 	 * libtls does not support compression, no actions needed to disable.
 	 */
 
-	libtls_config = tls_config_new();
-	if (!libtls_config) {
+	if ((libtls_config = tls_config_new()) == NULL) {
 		error_message(2, "tls_config_new: out of memory!\n");
 		return -ENOMEM;
 	}
@@ -97,9 +98,8 @@ tintls_init(
 	 * can be triggered.
 	 * (see https://git.causal.agency/libretls/about/#Compatibility)
 	 */
-	if (ca_cert_file[0] != '\0') {
-		result = tls_config_set_ca_file(libtls_config, ca_cert_file);
-		if (result != 0) {
+	if (*ca_cert_file) {
+		if (tls_config_set_ca_file(libtls_config, ca_cert_file) != 0) {
 			error_message(2, "%s!\n", tls_config_error(libtls_config));
 			tls_config_free(libtls_config);
 			libtls_config = NULL;
@@ -113,6 +113,14 @@ tintls_init(
 		tls_config_insecure_noverifytime(libtls_config);
 	}
 
+	/* ALPN - Application-Layer Protocol Negotiation, RFC 7301 */
+	if (tls_config_set_alpn(libtls_config, NNTP_TCP_NAME)) {
+		error_message(2, "%s!\n", tls_config_error(libtls_config));
+		tls_config_free(libtls_config);
+		libtls_config = NULL;
+		return -EINVAL;
+	}
+
 #else
 #	ifdef USE_GNUTLS
 
@@ -130,23 +138,20 @@ tintls_init(
 	}
 #		endif /* DEBUG */
 
-	result = gnutls_certificate_allocate_credentials(&tls_xcreds);
-	if (result < 0) {
+	if (gnutls_certificate_allocate_credentials(&tls_xcreds) < 0) {
 		error_message(2, "gnutls_certificate_allocate_credentials: out of memory!\n");
 		return -ENOMEM;
 	}
 
-	if (ca_cert_file[0] == '\0') {
-		result = gnutls_certificate_set_x509_system_trust(tls_xcreds);
-		if (result < 0) {
+	if (!*ca_cert_file) {
+		if ((result = gnutls_certificate_set_x509_system_trust(tls_xcreds)) < 0) {
 			error_message(2, "gnutls_certificate_set_x509_system_trust: %s (%d)!\n", gnutls_strerror(result), result);
 			gnutls_certificate_free_credentials(tls_xcreds);
 			tls_xcreds = NULL;
 			return -EINVAL;
 		}
 	} else {
-		result = gnutls_certificate_set_x509_trust_file(tls_xcreds, ca_cert_file, GNUTLS_X509_FMT_PEM);
-		if (result < 0) {
+		if ((result = gnutls_certificate_set_x509_trust_file(tls_xcreds, ca_cert_file, GNUTLS_X509_FMT_PEM)) < 0) {
 			error_message(2, "gnutls_certificate_set_x509_trust_file: %s (%d): \"%s\"!\n", gnutls_strerror(result), result, ca_cert_file);
 			gnutls_certificate_free_credentials(tls_xcreds);
 			tls_xcreds = NULL;
@@ -158,14 +163,12 @@ tintls_init(
 #		ifdef USE_OPENSSL
 	ERR_clear_error();
 
-	result = RAND_status();
-	if (result != 1) {
+	if (RAND_status() != 1) {
 		show_errors("RAND_status: %s!\n");
 		return -EINVAL;
 	}
 
-	openssl_ctx = SSL_CTX_new(TLS_method());
-	if (!openssl_ctx) {
+	if ((openssl_ctx = SSL_CTX_new(TLS_method())) == NULL) {
 		show_errors("SSL_CTX_new: %s!\n");
 		return -ENOMEM;
 	}
@@ -178,17 +181,15 @@ tintls_init(
 	 */
 	SSL_CTX_set_options(openssl_ctx, SSL_OP_NO_COMPRESSION);
 
-	if (ca_cert_file[0] == '\0') {
-		result = SSL_CTX_set_default_verify_paths(openssl_ctx);
-		if (result != 1) {
+	if (!*ca_cert_file) {
+		if (SSL_CTX_set_default_verify_paths(openssl_ctx) != 1) {
 			SSL_CTX_free(openssl_ctx);
 			openssl_ctx = NULL;
 			show_errors("SSL_CTX_set_default_verify_paths: %s!\n");
 			return -EINVAL;
 		}
 	} else {
-		result = SSL_CTX_load_verify_locations(openssl_ctx, ca_cert_file, NULL);
-		if (result != 1) {
+		if (SSL_CTX_load_verify_locations(openssl_ctx, ca_cert_file, NULL) != 1) {
 			SSL_CTX_free(openssl_ctx);
 			openssl_ctx = NULL;
 			show_errors("SSL_CTX_load_verify_locations: %s!\n");
@@ -196,6 +197,14 @@ tintls_init(
 		}
 	}
 
+	/* ALPN - Application-Layer Protocol Negotiation, RFC 7301 */
+	if (SSL_CTX_set_alpn_protos(openssl_ctx, nntp_alpn, (int) sizeof(nntp_alpn))) {
+		SSL_CTX_free(openssl_ctx);
+		openssl_ctx = NULL;
+		show_errors("SSL_CTX_set_alpn_protos: %s!\n");
+		return -EINVAL;
+	}
+
 	if (insecure_nntps)
 		SSL_CTX_set_verify(openssl_ctx, SSL_VERIFY_NONE, NULL);
 	else
@@ -248,7 +257,6 @@ tintls_open(
 	int fd,
 	void **session_ctx)
 {
-	int result;
 #ifdef USE_LIBTLS
 	struct tls *client;
 
@@ -257,18 +265,15 @@ tintls_open(
 
 	*session_ctx = NULL;
 
-	client = tls_client();
-	if (!client)
+	if ((client = tls_client()) == NULL)
 		return -ENOMEM;
 
-	result = tls_configure(client, libtls_config);
-	if (result == -1) {
+	if (tls_configure(client, libtls_config) == -1) {
 		tls_free(client);
 		return -ENOMEM;
 	}
 
-	result = tls_connect_socket(client, fd, servername);
-	if (result == -1) {
+	if (tls_connect_socket(client, fd, servername) == -1) {
 		tls_free(client);
 		tintls_exit();
 		return -ENOMEM;
@@ -279,38 +284,39 @@ tintls_open(
 #else
 #	ifdef USE_GNUTLS
 	gnutls_session_t client;
-	size_t servername_len;
 
 	if (!session_ctx)
 		return -EINVAL;
 
 	*session_ctx = NULL;
 
-	result = gnutls_init(&client, GNUTLS_CLIENT | GNUTLS_AUTO_REAUTH | GNUTLS_POST_HANDSHAKE_AUTH);
-	if (result < 0)
+	if (gnutls_init(&client, GNUTLS_CLIENT | GNUTLS_AUTO_REAUTH | GNUTLS_POST_HANDSHAKE_AUTH) < 0)
 		return -ENOMEM;
 
-	result = gnutls_server_name_set(client, GNUTLS_NAME_DNS, servername, strlen(servername));
-	if (result < 0) {
-		gnutls_deinit(client);
-		return -ENOMEM;
-	}
-
-	result = gnutls_set_default_priority(client);
-	if (result < 0) {
+	/* ALPN - Application-Layer Protocol Negotiation, RFC 7301 */
+	if (gnutls_alpn_set_protocols(client, gtls_alpns, 1, GNUTLS_ALPN_MANDATORY)) {
+		error_message(1, "%s", "failed setting ALPN");
 		gnutls_deinit(client);
 		return -EINVAL;
 	}
 
-	result = gnutls_credentials_set(client, GNUTLS_CRD_CERTIFICATE, tls_xcreds);
-	if (result < 0) {
+	/* SNI - Server Name Indication, RFC 6066 */
+	if (gnutls_server_name_set(client, GNUTLS_NAME_DNS, servername, strlen(servername)) < 0) {
+		gnutls_deinit(client);
+		return -ENOMEM;
+	}
+
+	if (gnutls_set_default_priority(client) < 0) {
 		gnutls_deinit(client);
 		return -EINVAL;
 	}
 
-	servername_len = strlen(servername) + 1;
-	gnutls_servername = my_malloc(servername_len);
-	strncpy(gnutls_servername, servername, servername_len);
+	if (gnutls_credentials_set(client, GNUTLS_CRD_CERTIFICATE, tls_xcreds) < 0) {
+		gnutls_deinit(client);
+		return -EINVAL;
+	}
+
+	gnutls_servername = my_strdup(servername);
 	gnutls_session_set_verify_function(client, &verification_func);
 
 	gnutls_transport_set_int(client, fd);
@@ -320,7 +326,6 @@ tintls_open(
 
 #	else
 #		ifdef USE_OPENSSL
-	int long_result;
 	SSL *ssl;
 	BIO *sock;
 	BIO *client;
@@ -345,24 +350,31 @@ tintls_open(
 		return -ENOMEM;
 	}
 
-	long_result = BIO_get_ssl(client, &ssl);
-	if (long_result != 1) {
+	if (BIO_get_ssl(client, &ssl) != 1) {
 		BIO_free(client);
 		BIO_free(sock);
 		show_errors("BIO_get_ssl: %s!\n");
 		return -EINVAL;
 	}
 
-	result = SSL_set_tlsext_host_name(ssl, servername);
-	if (result != 1) {
+	/* ALPN - Application-Layer Protocol Negotiation, RFC 7301 */
+	/* should not be needed as we already set it, but it doesn't hurt */
+	if (SSL_set_alpn_protos(ssl, nntp_alpn, (int) sizeof(nntp_alpn))) {
+		BIO_free(client);
+		BIO_free(sock);
+		show_errors("SSL_set_alpn_protos: %s!\n");
+		return -EINVAL;
+	}
+
+	/* SNI - Server Name Indication, RFC 6066 */
+	if (SSL_set_tlsext_host_name(ssl, servername) != 1) {
 		BIO_free(client);
 		BIO_free(sock);
 		show_errors("SSL_set_tlsext_host_name: %s!\n");
 		return -EINVAL;
 	}
 
-	result = SSL_set1_host(ssl, servername);
-	if (result != 1) {
+	if (SSL_set1_host(ssl, servername) != 1) {
 		BIO_free(client);
 		BIO_free(sock);
 		show_errors("SSL_set1_host: %s!\n");
@@ -411,11 +423,10 @@ tintls_handshake(
 		io_buf = BIO_new(BIO_s_mem());
 
 	if (chain && io_buf) {
-		X509 *cert;
-		char **cert_info;
-
 		if (chain_size > 0 && BIO_write(io_buf, chain, chain_size) > 0) {
-			cert = PEM_read_bio_X509(io_buf, NULL, 0, NULL);
+			char **cert_info;
+			X509 *cert = PEM_read_bio_X509(io_buf, NULL, NULL, NULL);
+
 			if (cert && ((cert_info = get_cert_info(cert)))) {
 				wait_message(0, _(txt_conninfo_subject), cert_info[0] ? cert_info[0] : _(txt_retr_subject_failed));
 				wait_message(0, _(txt_conninfo_issuer), cert_info[1] ? cert_info[1] : _(txt_retr_issuer_failed));
@@ -453,6 +464,13 @@ tintls_handshake(
 
 		wait_message(0, _(txt_tls_handshake_done), version, cipher);
 	}
+
+#	ifdef DEBUG
+	if (debug & DEBUG_NNTP) {
+		if (tls_conn_alpn_selected(client) == NULL)
+			wait_message(0, _(txt_tls_no_alpn_negotiated));
+	}
+#	endif /* DEBUG */
 #else
 #	ifdef USE_GNUTLS
 	int result;
@@ -464,14 +482,12 @@ tintls_handshake(
 
 	if (result < 0) {
 		if (gnutls_verification_status != 0) {
-			int status_result;
 			int type;
 			gnutls_datum_t msg;
 
 			type = gnutls_certificate_type_get2(client, GNUTLS_CTYPE_SERVER);
-			status_result = gnutls_certificate_verification_status_print(gnutls_verification_status, type, &msg, 0);
 
-			if (status_result == 0)
+			if (gnutls_certificate_verification_status_print(gnutls_verification_status, type, &msg, 0) == 0)
 				wait_message(0, _(txt_tls_peer_verify_failed), msg.data);
 			else
 				wait_message(0, _(txt_tls_peer_verify_failed), _(txt_tls_unable_to_get_status));
@@ -542,17 +558,18 @@ tintls_handshake(
 
 			if (!batch_mode || verbose) {
 #if defined(MULTIBYTE_ABLE) && !defined(NO_LOCALE)
-				char *sub = my_strdup((char *) subject.data);
-				char *iss = my_strdup((char *) issuer.data);
-				size_t len_s = strlen(sub);
-				size_t len_i = strlen(iss);
+				char *dt = my_strdup((char *) subject.data);
+				size_t len_dt = strlen(dt);
 
-				process_charsets(&sub, &len_s, "UTF-8", tinrc.mm_local_charset, FALSE);
-				process_charsets(&iss, &len_i, "UTF-8", tinrc.mm_local_charset, FALSE);
-				wait_message(0, _(txt_conninfo_subject), sub);
-				wait_message(0, _(txt_conninfo_issuer), iss);
-				free(sub);
-				free(iss);
+				process_charsets(&dt, &len_dt, "UTF-8", tinrc.mm_local_charset, FALSE);
+				wait_message(0, _(txt_conninfo_subject), dt);
+				free(dt);
+
+				dt = my_strdup((char *) issuer.data);
+				len_dt = strlen(dt);
+				process_charsets(&dt, &len_dt, "UTF-8", tinrc.mm_local_charset, FALSE);
+				wait_message(0, _(txt_conninfo_issuer), dt);
+				free(dt);
 #else
 				wait_message(0, _(txt_conninfo_subject), subject.data);
 				wait_message(0, _(txt_conninfo_issuer), issuer.data);
@@ -569,10 +586,18 @@ err_cert:
 		}
 
 		if (!batch_mode || verbose) {
+#		ifdef DEBUG
+			gnutls_datum_t alpn = { NULL, 0 };
+#		endif /* DEBUG */
+
 			if ((desc = gnutls_session_get_desc(client)) != NULL) {
 				wait_message(0, _(txt_tls_handshake_done), desc);
 				gnutls_free(desc);
 			}
+#		ifdef DEBUG
+			if ((debug & DEBUG_NNTP) && gnutls_alpn_get_selected_protocol(client, &alpn))
+				wait_message(0, _(txt_tls_no_alpn_negotiated));
+#		endif /* DEBUG */
 		}
 	}
 #	else
@@ -620,6 +645,17 @@ err_cert:
 		}
 
 		wait_message(0, _(txt_tls_handshake_done), SSL_get_cipher_name(ssl));
+
+#			ifdef DEBUG
+		if (debug & DEBUG_NNTP) {
+			const unsigned char *proto;
+			unsigned int proto_len;
+
+			SSL_get0_alpn_selected(ssl, &proto, &proto_len);
+			if (!proto_len)
+				wait_message(0, txt_tls_no_alpn_negotiated);
+		}
+#			endif /* DEBUG */
 	}
 #		endif /* USE_OPENSSL */
 #	endif /* USE_GNUTLS */
@@ -778,18 +814,35 @@ tintls_close(
 	return 0;
 }
 
-/* TODO: make date-format configurable? */
-#define PRINT_VALID_AFTER(ts, what) do { \
-		if (my_strftime(what, sizeof(what), "%Y-%m-%dT%H:%M%z", ts)) \
-			fprintf(fp, _(txt_valid_not_after), (what)); \
-		else \
+
+/*
+ * TODO:
+ *      - make date-format configurable?
+ *      - format startup time only once, it is static
+ */
+#define PRINT_VALID_AFTER(ts, what, val) do { \
+		if (my_strftime(what, sizeof(what), "%Y-%m-%dT%H:%M%z", ts)) { \
+			fprintf(fp, _(txt_valid_not_after), (what), (val) ? "" : _(txt_valid_no_longer)); \
+			if (!val_time) { \
+				if (my_strftime(what, sizeof(what), "%Y-%m-%dT%H:%M%z", gmtime(&startup_time))) { \
+					fprintf(fp, _(txt_startup_time), (what)); \
+					fprintf(fp, "\n"); \
+				} \
+				val_time = TRUE; \
+			} \
+		} else \
 			fprintf(fp, "%s", txt_conninfo_fmt_error); \
 	} while (0)
 
-#define PRINT_VALID_BEFORE(ts, what) do { \
-		if (my_strftime(what, sizeof(what), "%Y-%m-%dT%H:%M%z", ts)) \
-			fprintf(fp, _(txt_valid_not_before), (what)); \
-		else \
+#define PRINT_VALID_BEFORE(ts, what, val) do { \
+		if (my_strftime(what, sizeof(what), "%Y-%m-%dT%H:%M%z", ts)) { \
+			fprintf(fp, _(txt_valid_not_before), (what), (val) ? "" : _(txt_valid_not_yet)); \
+			if (!val_time) { \
+				if (my_strftime(what, sizeof(what), "%Y-%m-%dT%H:%M%z", gmtime(&startup_time))) \
+					fprintf(fp, _(txt_startup_time), (what)); \
+				val_time = TRUE; \
+			} \
+		} else \
 			fprintf(fp, "%s", txt_conninfo_fmt_error); \
 	} while (0)
 
@@ -799,9 +852,9 @@ tintls_conninfo(
 	void *session_ctx,
 	FILE *fp)
 {
+	char fmt_time[22]; /* %Y-%m-%dT%H:%M%z */
 #ifdef USE_LIBTLS
 	struct tls *client = session_ctx;
-	char fmt_time[22]; /* %Y-%m-%dT%H:%M%z */
 #	ifdef HAVE_LIB_CRYPTO
 	BIO *io_buf = NULL;
 	const uint8_t *chain;
@@ -809,6 +862,8 @@ tintls_conninfo(
 
 	fprintf(fp, "%s", _(txt_conninfo_tls_info));
 	fprintf(fp, _(txt_conninfo_libtls_info), tls_conn_version(client), tls_conn_cipher(client), tls_conn_cipher_strength(client));
+	if (tls_conn_alpn_selected(client) == NULL)
+		fprintf(fp, "%s\n", _(txt_tls_no_alpn_negotiated));
 
 	if ((chain = tls_peer_cert_chain_pem(client, &chain_size)))
 		io_buf = BIO_new(BIO_s_mem());
@@ -821,21 +876,22 @@ tintls_conninfo(
 		int i;
 		size_t cl;
 		struct tm tm;
+		t_bool val_time = TRUE;
 
 		fprintf(fp, "%s", _(txt_conninfo_server_cert_info));
 
 		/* string copy of chain */
-		i = snprintf(NULL, 0, "%.*s", (int) chain_size, chain);
+		i = snprintf(NULL, 0, "%.*s", (int) chain_size, (const char*) chain);
 		cl = (size_t) i + 1;
 		wchain = my_malloc(cl);
-		snprintf(wchain, cl, "%.*s", (int) chain_size, chain);
+		snprintf(wchain, cl, "%.*s", (int) chain_size, (const char*) chain);
 		cptr = wchain;
 		i = 0;
 
 		while ((cptr = strstr(cptr, "-----BEGIN CERTIFICATE-----"))) {
 			chain_size = strlen(cptr);
 			if (chain_size > 0 && BIO_write(io_buf, cptr, chain_size) > 0) {
-				cert = PEM_read_bio_X509(io_buf, NULL, 0, NULL);
+				cert = PEM_read_bio_X509(io_buf, NULL, NULL, NULL);
 				if (cert && ((cert_info = get_cert_info(cert)))) {
 					if (i)
 						fputs("\n", fp);
@@ -849,14 +905,24 @@ tintls_conninfo(
 
 					if ((asn1 = X509_get0_notBefore(cert)) != NULL) {
 						if (ASN1_TIME_to_tm(asn1, &tm) == 1) {
-							PRINT_VALID_BEFORE(&tm, fmt_time);
+#	ifdef HAVE_MKTIME
+							if (mktime(&tm) > startup_time)
+								val_time = FALSE;
+#	endif /* HAVE_MKTIME */
+							PRINT_VALID_BEFORE(&tm, fmt_time, val_time);
 						}
 					}
+
 					if ((asn1 = X509_get0_notAfter(cert)) != NULL) {
 						if (ASN1_TIME_to_tm(asn1, &tm) == 1) {
-							PRINT_VALID_AFTER(&tm, fmt_time);
+#	ifdef HAVE_MKTIME
+							if (mktime(&tm) < startup_time)
+								val_time = FALSE;
+#	endif /* HAVE_MKTIME */
+							PRINT_VALID_AFTER(&tm, fmt_time, val_time);
 						}
 					}
+
 					X509_free(cert);
 				}
 			}
@@ -870,37 +936,48 @@ tintls_conninfo(
 	fprintf(fp, "%s", _(txt_conninfo_tls_info));
 	fprintf(fp, _(txt_conninfo_libtls_info), tls_conn_version(client), tls_conn_cipher(client), tls_conn_cipher_strength(client));
 	fprintf(fp, "%s", _(txt_conninfo_server_cert_info));
+	if (tls_conn_alpn_selected(client) == NULL)
+		fprintf(fp, "%s\n", _(txt_tls_no_alpn_negotiated));
 #	endif /* HAVE_LIB_CRYPTO */
 	{
 		struct tm *tm;
 		time_t t;
+		t_bool val_time = TRUE;
 
 		fprintf(fp, _(txt_conninfo_subject), tls_peer_cert_subject(client));
 		fprintf(fp, _(txt_conninfo_issuer), tls_peer_cert_issuer(client));
 
 		if ((t = tls_peer_cert_notbefore(client)) != -1) {
 			tm = gmtime(&t);
-			PRINT_VALID_BEFORE(tm, fmt_time);
+#	ifdef HAVE_MKTIME
+			if (mktime(tm) > startup_time)
+				val_time = FALSE;
+#	endif /* HAVE_MKTIME */
+			PRINT_VALID_BEFORE(tm, fmt_time, val_time);
 		}
 
 		if ((t = tls_peer_cert_notafter(client)) != -1) {
 			tm = gmtime(&t);
-			PRINT_VALID_AFTER(tm, fmt_time);
+#	ifdef HAVE_MKTIME
+			if (mktime(tm) < startup_time)
+				val_time = FALSE;
+#	endif /* HAVE_MKTIME */
+			PRINT_VALID_AFTER(tm, fmt_time, val_time);
 		}
 	}
 #else
 #	ifdef USE_GNUTLS
 	char *desc;
-	char fmt_time[22]; /* %Y-%m-%dT%H:%M%z */
 	int retval = -1;
-	int result;
 	gnutls_session_t client = session_ctx;
 	gnutls_datum_t msg;
+	gnutls_datum_t alpn = { NULL, 0 };
 	const gnutls_datum_t *raw_servercert_chain;
 	unsigned int servercert_chainlen;
 	unsigned int i;
 	time_t t;
 	struct tm *tm;
+	t_bool val_time = TRUE;
 
 	desc = gnutls_session_get_desc(client);
 	fprintf(fp, "%s", _(txt_conninfo_tls_info));
@@ -912,11 +989,9 @@ tintls_conninfo(
 	if (gnutls_verification_status != 0) {
 		int type = gnutls_certificate_type_get2(client, GNUTLS_CTYPE_SERVER);
 
-		result = gnutls_certificate_verification_status_print(gnutls_verification_status, type, &msg, 0);
-
-		if (result == 0)
+		if (!gnutls_certificate_verification_status_print(gnutls_verification_status, type, &msg, 0))
 			fprintf(fp, _(txt_conninfo_verify_failed), msg.data,
-					insecure_nntps ? _(txt_conninfo_error_tolerated) : _(txt_conninfo_error_unexpected));
+				insecure_nntps ? _(txt_conninfo_error_tolerated) : _(txt_conninfo_error_unexpected));
 		else
 			fprintf(fp, "%s", _(txt_conninfo_verify_failed_no_reason));
 
@@ -924,6 +999,9 @@ tintls_conninfo(
 	} else
 		fprintf(fp, "%s", _(txt_conninfo_verify_successful));
 
+	if (gnutls_alpn_get_selected_protocol(client, &alpn))
+		fprintf(fp, "%s\n", _(txt_tls_no_alpn_negotiated));
+
 	raw_servercert_chain = gnutls_certificate_get_peers(client, &servercert_chainlen);
 	if (servercert_chainlen > 0)
 		fprintf(fp, "%s", _(txt_conninfo_server_cert_info));
@@ -932,66 +1010,64 @@ tintls_conninfo(
 		gnutls_x509_crt_t servercert = NULL;
 		gnutls_datum_t subject = { NULL, 0 };
 		gnutls_datum_t issuer = { NULL, 0 };
-#if defined(MULTIBYTE_ABLE) && !defined(NO_LOCALE)
+#		if defined(MULTIBYTE_ABLE) && !defined(NO_LOCALE)
 		char *sub;
 		char *iss;
 		size_t len_s;
 		size_t len_i;
-#endif /* MULTIBYTE_ABLE && !NO_LOCALE */
+#		endif /* MULTIBYTE_ABLE && !NO_LOCALE */
 
 		if (i)
 			fputs("\n", fp);
 		fprintf(fp, _(txt_conninfo_cert), i);
 
-		result = gnutls_x509_crt_init(&servercert);
-		if (result < 0)
+		if (gnutls_x509_crt_init(&servercert) < 0)
 			goto err_cert;
 
-		result = gnutls_x509_crt_import(servercert, &raw_servercert_chain[i], GNUTLS_X509_FMT_DER);
-		if (result < 0)
+		if (gnutls_x509_crt_import(servercert, &raw_servercert_chain[i], GNUTLS_X509_FMT_DER) < 0)
 			goto err_cert;
 
-		result = gnutls_x509_crt_get_dn3(servercert, &subject, 0);
-		if (result < 0)
+		if (gnutls_x509_crt_get_dn3(servercert, &subject, 0) < 0)
 			goto err_cert;
 		else {
-#if defined(MULTIBYTE_ABLE) && !defined(NO_LOCALE)
+#		if defined(MULTIBYTE_ABLE) && !defined(NO_LOCALE)
 			sub = (char *) subject.data;
 			len_s = strlen(sub);
 			process_charsets(&sub, &len_s, "UTF-8", tinrc.mm_local_charset, FALSE);
 			fprintf(fp, _(txt_conninfo_subject), sub);
-#else
+#		else
 			fprintf(fp, _(txt_conninfo_subject), subject.data);
-#endif /* MULTIBYTE_ABLE && !NO_LOCALE */
+#		endif /* MULTIBYTE_ABLE && !NO_LOCALE */
 		}
 
-		result = gnutls_x509_crt_get_issuer_dn3(servercert, &issuer, 0);
-		if (result < 0)
+		if (gnutls_x509_crt_get_issuer_dn3(servercert, &issuer, 0) < 0)
 			goto err_cert;
 		else {
-#if defined(MULTIBYTE_ABLE) && !defined(NO_LOCALE)
+#		if defined(MULTIBYTE_ABLE) && !defined(NO_LOCALE)
 			iss = (char *) issuer.data;
 			len_i = strlen(iss);
 			process_charsets(&iss, &len_i, "UTF-8", tinrc.mm_local_charset, FALSE);
 			fprintf(fp, _(txt_conninfo_issuer), iss);
-#else
+#		else
 			fprintf(fp, _(txt_conninfo_issuer), issuer.data);
-#endif /* MULTIBYTE_ABLE && !NO_LOCALE */
+#		endif /* MULTIBYTE_ABLE && !NO_LOCALE */
 		}
 
-		t = gnutls_x509_crt_get_activation_time(servercert);
-		if (t == -1)
+		if ((t = gnutls_x509_crt_get_activation_time(servercert)) == -1)
 			goto err_cert;
 
 		tm = localtime(&t);
-		PRINT_VALID_BEFORE(tm, fmt_time);
+		if (t > startup_time)
+			val_time = FALSE;
+		PRINT_VALID_BEFORE(tm, fmt_time, val_time);
 
-		t = gnutls_x509_crt_get_expiration_time(servercert);
-		if (t == -1)
+		if ((t = gnutls_x509_crt_get_expiration_time(servercert)) == -1)
 			goto err_cert;
 
 		tm = localtime(&t);
-		PRINT_VALID_AFTER(tm, fmt_time);
+		if (t < startup_time)
+			val_time = FALSE;
+		PRINT_VALID_AFTER(tm, fmt_time, val_time);
 
 		retval = 0;
 
@@ -1005,17 +1081,19 @@ err_cert:
 	}
 
 	return retval;
+	/* NOTREACHED */
 #	else
 
 #		ifdef USE_OPENSSL
-	long long_result;
+	const unsigned char *proto;
+	unsigned int proto_len;
 	long verification_result;
 	BIO *client = session_ctx;
 	SSL *ssl;
 	STACK_OF(X509) *chain;
+	t_bool val_time = TRUE;
 
-	long_result = BIO_get_ssl(client, &ssl);
-	if (long_result != 1)
+	if (BIO_get_ssl(client, &ssl) != 1L)
 		return -1;
 
 	fprintf(fp, "%s", _(txt_conninfo_tls_info));
@@ -1029,6 +1107,10 @@ err_cert:
 	else
 		fprintf(fp, "%s", _(txt_conninfo_verify_successful));
 
+	SSL_get0_alpn_selected(ssl, &proto, &proto_len);
+	if (!proto_len)
+		fprintf(fp, "%s\n", _(txt_tls_no_alpn_negotiated));
+
 	fprintf(fp, "%s", _(txt_conninfo_server_cert_info));
 
 	if (verification_result == X509_V_OK)
@@ -1037,14 +1119,13 @@ err_cert:
 		chain = SSL_get0_verified_chain(ssl);
 
 	if (chain) {
-		char name[22]; /* %Y-%m-%dT%H:%M%z */
 		char **cert_info;
 		const ASN1_TIME *asn1;
 		int i;
 		struct tm tm;
 
 		for (i = 0; i < sk_X509_num(chain); i++) {
-			X509* cert = sk_X509_value(chain, i);
+			X509 *cert = sk_X509_value(chain, i);
 
 			if (i)
 				fputs("\n", fp);
@@ -1060,13 +1141,21 @@ err_cert:
 
 			if ((asn1 = X509_get0_notBefore(cert)) != NULL) {
 				if (ASN1_TIME_to_tm(asn1, &tm) == 1) {
-					PRINT_VALID_BEFORE(&tm, name);
+#			ifdef HAVE_MKTIME
+					if (mktime(&tm) > startup_time)
+						val_time = FALSE;
+#			endif /* HAVE_MKTIME */
+					PRINT_VALID_BEFORE(&tm, fmt_time, val_time);
 				}
 			}
 
 			if ((asn1 = X509_get0_notAfter(cert)) != NULL) {
 				if (ASN1_TIME_to_tm(asn1, &tm) == 1) {
-					PRINT_VALID_AFTER(&tm, name);
+#			ifdef HAVE_MKTIME
+					if (mktime(&tm) < startup_time)
+						val_time = FALSE;
+#			endif /* HAVE_MKTIME */
+					PRINT_VALID_AFTER(&tm, fmt_time, val_time);
 				}
 			}
 		}
@@ -1074,6 +1163,7 @@ err_cert:
 #		endif /* USE_OPENSSL */
 #	endif /* USE_GNUTLS */
 #endif /* USE_LIBTLS */
+
 	return 0;
 }
 
@@ -1100,8 +1190,7 @@ get_cert_info(
 	if (cert && (io_buf = BIO_new(BIO_s_mem()))) {
 		res = my_malloc(sizeof(char *) * 2);
 		if (X509_NAME_print_ex(io_buf, X509_get_subject_name(cert), 0, flags) != -1) {
-			len = BIO_get_mem_data(io_buf, &tmp);
-			if (len > 0) {
+			if ((len = BIO_get_mem_data(io_buf, &tmp)) > 0) {
 				subject = my_malloc((size_t) len + 1);
 				memcpy(subject, tmp, (size_t) len);
 				subject[len] = '\0';
@@ -1111,8 +1200,7 @@ get_cert_info(
 			}
 		}
 		if (BIO_reset(io_buf) != -1 && X509_NAME_print_ex(io_buf, X509_get_issuer_name(cert), 0, flags) != -1) {
-			len = BIO_get_mem_data(io_buf, &tmp);
-			if (len > 0) {
+			if ((len = BIO_get_mem_data(io_buf, &tmp)) > 0) {
 				issuer = my_malloc((size_t) len + 1);
 				memcpy(issuer, tmp, (size_t) len);
 				issuer[len] = '\0';
@@ -1137,9 +1225,8 @@ show_errors(
 {
 	unsigned long next_error;
 
-	while ((next_error = ERR_get_error())) {
+	while ((next_error = ERR_get_error()))
 		error_message(0, msg_fmt, ERR_error_string(next_error, NULL));
-	}
 }
 #endif /* USE_OPENSSL */
 
@@ -1179,14 +1266,14 @@ log_func(
 		return;
 
 	if (msg[msglen - 1] == '\n')
-		msglen--;
+		--msglen;
 
 	debug_print_file("NNTP", "TLS%s%.*s [%d]", logtime(), msglen, msg, level);
 }
 
+
 #	else
 #		ifdef USE_OPENSSL
-
 static void
 info_callback(
 	const SSL *s,
@@ -1226,11 +1313,10 @@ info_callback(
 }
 #		endif /* USE_OPENSSL */
 #	endif /* USE_GNUTLS */
-
 #endif /* DEBUG */
 
-#else
 
+#else
 int
 tintls_init(
 	void)
