@@ -3,13 +3,13 @@
  *  Module    : nrctbl.c
  *  Author    : Sven Paulus <sven@tin.org>
  *  Created   : 1996-10-06
- *  Updated   : 2024-03-30
+ *  Updated   : 2024-12-05
  *  Notes     : This module does the NNTP server name lookup in
  *              ~/.tin/newsrctable and returns the real hostname
  *              and the name of the newsrc file for a given
  *              alias of the server.
  *
- * Copyright (c) 1996-2024 Sven Paulus <sven@tin.org>
+ * Copyright (c) 1996-2025 Sven Paulus <sven@tin.org>
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -71,7 +71,7 @@ write_newsrctable_file(
 {
 	FILE *fp;
 
-	if ((fp = fopen(local_newsrctable_file, "w")) == NULL)
+	if ((fp = fopen(newsrctable_file, "w")) == NULL)
 		return;
 
 	fprintf(fp, _(txt_nrctbl_info), PRODUCT, VERSION);
@@ -79,14 +79,13 @@ write_newsrctable_file(
 	fchmod(fileno(fp), (mode_t) (S_IRUSR|S_IWUSR));
 #else
 #	ifdef HAVE_CHMOD
-	chmod(local_newsrctable_file, (mode_t) (S_IRUSR|S_IWUSR));
+	chmod(newsrctable_file, (mode_t) (S_IRUSR|S_IWUSR));
 #	endif /* HAVE_CHMOD */
 #endif /* HAVE_FCHMOD */
 	fclose(fp);
 }
 
 
-#ifdef NNTP_ABLE
 /*
  * get_nntpserver()
  * returns the FQDN of NNTP server by looking up a given
@@ -107,13 +106,13 @@ get_nntpserver(
 	int line_entry_counter;
 	t_bool found = FALSE;
 
-	if ((fp = fopen(local_newsrctable_file, "r")) != NULL) {
+	if ((fp = tin_fopen(newsrctable_file, "r")) != NULL) {
 		while ((fgets(line, sizeof(line), fp) != NULL) && !found) {
 			line_entry_counter = 0;
 
 			if (!strchr("# ;", line[0])) {
 				while ((line_entry = strtok(line_entry_counter ? NULL : line, " \t\n")) != NULL) {
-					line_entry_counter++;
+					++line_entry_counter;
 
 					if (line_entry_counter == 1)
 						STRCPY(name_found, line_entry);
@@ -131,7 +130,6 @@ get_nntpserver(
 	}
 	nntpserver_name[nntpserver_name_len - 1] = '\0';
 }
-#endif /* NNTP_ABLE */
 
 
 /*
@@ -146,49 +144,49 @@ get_newsrcname(
 	const char *nntpserver_name) /* return value is always ignored */
 {
 	FILE *fp;
-	char *line_entry;
-	char line[LEN];
-	char name_found[PATH_LEN] = { '\0' };
-	int line_entry_counter;
-	int found = 0;
-	t_bool do_cpy = FALSE;
 
-	if ((fp = fopen(local_newsrctable_file, "r")) != NULL) {
-		char *ns, *nsp, *p, *q;
+	if ((fp = tin_fopen(newsrctable_file, "r")) != NULL) {
+		char line[LEN];
+		char name_found[PATH_LEN] = { '\0' };
+		char *ns = NULL, *nsp = NULL, *p, *q;
+		char *line_entry;
+		int found = 0;
+		int line_entry_counter;
 		size_t l = strlen(nntpserver_name) + 8; /* []:65535\0 */
-
-		/* add [] and/or port if required */
-		ns = my_strdup(nntpserver_name);
-		nsp = my_malloc(l);
-		sprintf(nsp, "%s:%d", nntpserver_name, nntp_tcp_port);
+		t_bool do_cpy = FALSE;
 
 		if ((p = strchr(nntpserver_name, ':')) != NULL) {
 			if ((q = strrchr(nntpserver_name, ':')) != NULL) {
 				if (p != q) {
-					free(ns);
-					free(nsp);
 					ns = my_malloc(l);
 					nsp = my_malloc(l);
 					sprintf(ns, "[%s]", nntpserver_name);
-					sprintf(nsp, "[%s]:%d", nntpserver_name, nntp_tcp_port);
+					sprintf(nsp, "[%s]:%u", nntpserver_name, nntp_tcp_port);
 				}
 			}
 		}
+		if (!ns)
+			ns = my_strdup(nntpserver_name);
+		if (!nsp) {
+			/* add [] and/or port if required */
+			nsp = my_malloc(l);
+			sprintf(nsp, "%s:%u", nntpserver_name, nntp_tcp_port);
+		}
 
-		while ((fgets(line, (int) sizeof(line), fp) != NULL) && (found != 1)) {
+		while ((fgets(line, sizeof(line), fp) != NULL) && (found != 1)) {
 			if (strchr("# ;", line[0])) /* comment? */
 				continue;
 
 			line_entry_counter = 0;
 			while ((line_entry = strtok(line_entry_counter ? NULL : line, " \t\n")) != NULL) {
-				line_entry_counter++;
+				++line_entry_counter;
 
 				if ((line_entry_counter == 1) && ((!strcasecmp(line_entry,nsp)) || (!strcasecmp(line_entry, ns)))) {
 					found = 1;
 					do_cpy = TRUE;
 				}
 
-				if (!found && (!strcasecmp(line_entry, "default") || !strcmp(line_entry, "*"))) {
+				if (!found && (!strcasecmp(line_entry, "default") || (*line_entry == '*' && *(line_entry + 1) == '\0'))) {
 					found = 2;
 					do_cpy = TRUE;
 				}
