@@ -3,10 +3,10 @@
  *  Module    : page.c
  *  Author    : I. Lea & R. Skrenta
  *  Created   : 1991-04-01
- *  Updated   : 2024-07-30
+ *  Updated   : 2025-07-10
  *  Notes     :
  *
- * Copyright (c) 1991-2024 Iain Lea <iain@bricbrac.de>, Rich Skrenta <skrenta@pbm.com>
+ * Copyright (c) 1991-2025 Iain Lea <iain@bricbrac.de>, Rich Skrenta <skrenta@pbm.com>
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -44,6 +44,13 @@
 #ifndef TCURSES_H
 #	include "tcurses.h"
 #endif /* !TCURSES_H */
+#ifdef HAVE_LIBURIPARSER
+#	include <uriparser/Uri.h>
+#else
+#	ifdef HAVE_LIBCURL
+#		include <curl/curl.h>
+#	endif /* HAVE_LIBCURL */
+#endif /* HAVE_LIBURIPARSER */
 
 
 /*
@@ -77,7 +84,7 @@ static struct t_header *note_h = &pgart.hdr;	/* Easy access to article headers *
 static FILE *info_file;
 static const char *info_title;
 static int curr_info_line;
-static int hide_uue;			/* set when uuencoded sections are 'hidden' */
+static int hide_inline_data;	/* set when inline data sections are 'hidden' */
 static int num_info_lines;
 static int reveal_ctrl_l_lines;	/* number of lines (from top) with de-activated ^L */
 static int rotate;				/* 0=normal, 13=rot13 decode */
@@ -108,7 +115,7 @@ static t_function page_mouse_action(t_function (*left_action) (void), t_function
 static t_function url_left(void);
 static t_function url_right(void);
 static void build_url_line(int i);
-static void draw_page_header(void);
+static void draw_page_header(t_bool tex2iso_conv);
 static void draw_percent_mark(long cur_num, long max_num);
 static void draw_url_arrow(void);
 static void free_url_list(void);
@@ -158,7 +165,7 @@ scroll_page(
 				break;
 
 			case -1:
-				i--;
+				--i;
 				break;
 
 			case -2:
@@ -321,13 +328,13 @@ show_page(
 	char key[MAXKEYLEN];
 	int i, j, n = 0;
 	int art_type = GROUP_TYPE_NEWS;
-	int hide_uue_tmp;
+	int hide_inline_data_tmp;
 	t_artnum old_artnum;
 	t_bool mouse_click_on = TRUE;
 	t_bool repeat_search;
 	t_function func;
 
-	if (group->attribute->mailing_list != NULL)
+	if (group->attribute->mailing_list && *group->attribute->mailing_list)
 		art_type = GROUP_TYPE_MAIL;
 
 	/*
@@ -429,7 +436,7 @@ show_page(
 						curr_line += ((tinrc.scroll_lines == -2) ? ARTLINES / 2 : ARTLINES);
 
 						if (tinrc.scroll_lines == -1)		/* formerly show_last_line_prev_page */
-							curr_line--;
+							--curr_line;
 						draw_page(0);
 					}
 				}
@@ -508,11 +515,43 @@ page_goto_next_unread:
 						return i;
 					}
 				}
+#ifdef NNTP_ABLE
+				else {
+					const char *eyde = input_history[HIST_MESSAGE_ID][(hist_pos[HIST_MESSAGE_ID] - 1 + HIST_SIZE) % HIST_SIZE];
+					int tr = this_resp;
+
+					if (*eyde) {
+						switch (show_article_by_msgid(eyde)) {
+							case LOOKUP_OK:
+								art_close(&pgart);
+								this_resp = tr;
+								if (!index_group(curr_group))
+									return GRP_RETSELECT;
+								if (!art_open(TRUE, &arts[this_resp], group, &pgart, TRUE, _(txt_reading_article))) {
+									if ((i = load_article(this_resp, group)) < 0)
+										return i;
+								} else {
+									XFACE_CLEAR(); /* TODO: needed? */
+									return ART_UNAVAILABLE;
+								}
+								break;
+
+							case LOOKUP_UNAVAIL: /* TODO: check for !reading_saved_news for better error-message? */
+								wait_message(2, _(txt_lookup_func_not_available));
+								break;
+
+							default:
+								wait_message(2, _(txt_art_unavailable));
+								break;
+						}
+					}
+				}
+#endif /* NNTP_ABLE */
 				break;
 
 			case PAGE_GOTO_PARENT:		/* Goto parent of this article */
 			{
-				struct t_msgid *parent = arts[this_resp].refptr->parent;
+				const struct t_msgid *parent = arts[this_resp].refptr->parent;
 
 				if (parent == NULL) {
 					info_message(_(txt_art_parent_none));
@@ -609,7 +648,8 @@ page_goto_next_unread:
 
 			case PAGE_TOP_THREAD:	/* first article in current thread */
 				if (arts[this_resp].prev >= 0) {
-					if ((n = which_thread(this_resp)) >= 0 && base[n] != this_resp) {
+					n = which_thread(this_resp);
+					if (n >= 0 && base[n] != this_resp) {
 						assert(n < grpmenu.max);
 						if ((i = load_article((int) base[n], group)) < 0) {
 							XFACE_CLEAR();
@@ -623,7 +663,7 @@ page_goto_next_unread:
 				for (i = this_resp; i >= 0; i = arts[i].thread)
 					n = i;
 
-				if (n != this_resp) {
+				if (n != this_resp && n >= 0) {
 					if ((i = load_article(n, group)) < 0) {
 						XFACE_CLEAR();
 						return i;
@@ -662,8 +702,8 @@ page_goto_next_unread:
 				break;
 
 			case PAGE_TOGGLE_TEX2ISO:		/* toggle German TeX to ISO latin1 style conversion */
-				if (((group->attribute->tex2iso_conv) = !(group->attribute->tex2iso_conv)))
-					pgart.tex2iso = is_art_tex_encoded(pgart.raw);
+				if ((group->attribute->tex2iso_conv = bool_not(group->attribute->tex2iso_conv)))
+					;
 				else
 					pgart.tex2iso = FALSE;
 
@@ -679,18 +719,27 @@ page_goto_next_unread:
 				info_message(_(txt_toggled_tabwidth), tabwidth);
 				break;
 
-			case PAGE_TOGGLE_UUE:			/* toggle display of uuencoded sections */
-				hide_uue = (hide_uue + 1) % (UUE_ALL + 1);
+			case PAGE_TOGGLE_INLINE_DATA:	/* toggle display of inline data sections */
+				do {
+					hide_inline_data = (hide_inline_data + 1) % (HIDE_ALL + 1);
+				} while (!*txt_hide_inline_data_type[hide_inline_data]); /* a list with gaps? - skip over them */
 				resize_article(TRUE, &pgart);	/* Also recooks it.. */
 				/*
 				 * If we hid uue and are off the end of the article, reposition to
 				 * show last page for neatness
 				 */
-				if (hide_uue && curr_line + ARTLINES > artlines)
+				if (hide_inline_data && curr_line + ARTLINES > artlines)
 					curr_line = artlines - ARTLINES;
 				draw_page(0);
-				/* TODO: use translateable sentences */
-				info_message("%s: %s", _(txt_hide_uue.opt), _(txt_hide_uue_type[hide_uue]));
+				info_message("%s: %s", _(txt_hide_inline_data.opt), _(txt_hide_inline_data_type[hide_inline_data]));
+				break;
+
+			case PAGE_TOGGLE_VERBATIM:			/* toggle verbatim options */
+				if (++group->attribute->verbatim_handling > VERBATIM_HIDE_ALL)
+					group->attribute->verbatim_handling = VERBATIM_NONE;
+				resize_article(TRUE, &pgart);	/* Also recooks it.. */
+				draw_page(0);
+				info_message(_(txt_toggled_verbatim), txt_verbatim_handling_options[group->attribute->verbatim_handling]);
 				break;
 
 			case PAGE_REVEAL:			/* toggle hiding after ^L */
@@ -742,8 +791,8 @@ page_goto_next_unread:
 					if (invoke_editor(filter_file, filter_file_offset, NULL)) {
 						old_artnum = arts[this_resp].artnum;
 						unfilter_articles(group);
-						(void) read_filter_file(filter_file);
-						filter_articles(group);
+						if (read_filter_file(filter_file))
+							filter_articles(group);
 						make_threads(group, FALSE);
 						if ((n = find_artnum(old_artnum)) == -1 || which_thread(n) == -1) /* We have lost the thread */
 							return GRP_KILLED;
@@ -790,9 +839,9 @@ page_goto_next_unread:
 			case MARK_THREAD_UNREAD:
 				thd_mark_unread(group, base[which_thread(this_resp)]);
 				if (group->attribute->thread_articles != THREAD_NONE)
-					info_message(_(txt_marked_as_unread), _(txt_thread_upper));
+					info_message(_(txt_marked_thread_as_unread));
 				else
-					info_message(_(txt_marked_as_unread), _(txt_article_upper));
+					info_message(_(txt_marked_article_as_unread));
 				break;
 
 			case PAGE_CANCEL:			/* cancel an article */
@@ -978,10 +1027,14 @@ return_to_index:
 
 			case MARK_ARTICLE_UNREAD:	/* mark article as unread(to return) */
 				art_mark(group, &arts[this_resp], ART_WILL_RETURN);
-				info_message(_(txt_marked_as_unread), _(txt_article_upper));
+				info_message(_(txt_marked_article_as_unread));
 				break;
 
 			case PAGE_SKIP_INCLUDED_TEXT:	/* skip included text */
+				if (show_raw_article) {
+					info_message(_(txt_info_not_available_in_raw));
+					break;
+				}
 				for (i = j = curr_line; i < artlines; i++) {
 					if (artline[i].flags & (C_QUOTE1 | C_QUOTE2 | C_QUOTE3)) {
 						j = i;
@@ -1012,12 +1065,12 @@ return_to_index:
 
 			case PAGE_VIEW_ATTACHMENTS:
 				XFACE_SUPPRESS();
-				hide_uue_tmp = hide_uue;
-				hide_uue = UUE_NO;
+				hide_inline_data_tmp = hide_inline_data;
+				hide_inline_data = UUE_NO;
 				resize_article(TRUE, &pgart);
 				attachment_page(&pgart);
 				signal_context = cPage;
-				hide_uue = hide_uue_tmp;
+				hide_inline_data = hide_inline_data_tmp;
 				resize_article(TRUE, &pgart);
 				draw_page(0);
 				XFACE_SHOW();
@@ -1236,7 +1289,7 @@ draw_page(
 	 */
 	if ((end - start >= ARTLINES) || (part == 0)) {
 		ClearScreen();
-		draw_page_header();
+		draw_page_header(CURR_GROUP.attribute->tex2iso_conv);
 	} else
 		MoveCursor(0, 0);
 
@@ -1374,10 +1427,9 @@ invoke_metamail(
 		if (!seek_error) {
 #ifdef DONT_HAVE_PIPING
 			char *pbuf;
-			size_t len;
+			size_t len = snprintf(NULL, 0, "%s %s", tinrc.metamail_prog, mimefile);
 
-			len = snprintf(NULL, 0, "%s %s", tinrc.metamail_prog, mimefile);
-			pbuf= my_malloc(++len);
+			pbuf = my_malloc(++len);
 			snprintf(pbuf, len, "%s %s", tinrc.metamail_prog, mimefile);
 			invoke_cmd(pbuf);
 			free(pbuf);
@@ -1409,6 +1461,7 @@ invoke_metamail(
 }
 
 
+/* TODO: add BiDi handling (our layout expects LTR) */
 static char *
 build_from_line(
 	void)
@@ -1416,10 +1469,16 @@ build_from_line(
 	char *tmp_from, *curr_from, *next_from, *p;
 	char addr[HEADER_LEN];
 	char name[HEADER_LEN];
-	char single_from[HEADER_LEN + 3]; /*" <>"*/
+	char single_from[HEADER_LEN + 5]; /*"\"\" <>"*/
 	char *from = NULL;
 	int type, c_needed = 0;
 
+	if (!note_h->from) {
+		from = my_malloc(3);
+		strcpy(from, "<>");
+		return from;
+	}
+
 	curr_from = tmp_from = my_strdup(note_h->from);
 
 	do {
@@ -1428,14 +1487,19 @@ build_from_line(
 			buffer_to_ascii(addr);
 			p = idna_decode(addr);
 
-			if (*name)
-				snprintf(single_from, sizeof(single_from), "%s <%s>", name, p);
-			else {
+			if (*name) {
+				if (CHECK_RFC5322_SPECIALS(name))
+					snprintf(single_from, sizeof(single_from), "\"%s\" <%s>", name, p);
+				else
+					snprintf(single_from, sizeof(single_from), "%s <%s>", name, p);
+			} else {
 				STRCPY(single_from, p);
 			}
 			free(p);
 		} else {
-			STRCPY(single_from, str_trim(curr_from));
+			p = idna_decode(str_trim(curr_from));
+			STRCPY(single_from, p);
+			free(p);
 		}
 		if (c_needed++)
 			from = append_to_string(from, ", ");
@@ -1448,6 +1512,7 @@ build_from_line(
 }
 
 
+#define CCOLS_THIRD (cCOLS / 3 + 1)
 /*
  * PAGE_HEADER defines the size in lines of this header
  *
@@ -1458,17 +1523,16 @@ build_from_line(
  */
 static void
 draw_page_header(
-	void)
+	t_bool tex2iso_conv)
 {
-	char *buf, *from, *tmp, *tmp2;
-	int i;
+	char *buf, *from, *tmp = NULL, *tmp2 = NULL;
+	int i, n;
 	int whichresp, x_resp;
 	int len, right_len, center_pos, cur_pos;
-	size_t line_len;
+	int tex_space;
+	size_t line_len, tlen;
 #if defined(MULTIBYTE_ABLE) && !defined(NO_LOCALE)
-	int n;
-	size_t tlen;
-	wchar_t *fmt_resp = NULL, *fmt_thread, *wtmp, *wtmp2, *wtmp3, *wbuf;
+	wchar_t *wtmp, *wtmp2;
 #endif /* MULTIBYTE_ABLE && !NO_LOCALE */
 
 	whichresp = which_response(this_resp);
@@ -1480,7 +1544,7 @@ draw_page_header(
 	line_len = LEN + 1;
 	buf = my_malloc(line_len);
 
-	if (!my_strftime(buf, line_len, curr_group->attribute->date_format, localtime(&arts[this_resp].date))) {
+	if (!my_strftime(buf, line_len, curr_group->attribute->date_format ? BlankIfNull(*curr_group->attribute->date_format) : "", localtime(&arts[this_resp].date))) {
 		strncpy(buf, BlankIfNull(note_h->date), line_len);
 		buf[line_len - 1] = '\0';
 	}
@@ -1492,35 +1556,22 @@ draw_page_header(
 	/* first line */
 	cur_pos = 0;
 
-#if defined(MULTIBYTE_ABLE) && !defined(NO_LOCALE)
-
-	/* convert to wide-char format string */
-	fmt_thread = char2wchar_t(_(txt_thread_x_of_n));
-
 	/*
 	 * Determine the needed space for the text at the right hand margin.
 	 * The formatting info (%4s) needs 3 positions but we need 4 positions
 	 * on the screen for each counter.
 	 */
-	if (fmt_thread)
-		right_len = wcswidth(fmt_thread, wcslen(fmt_thread)) - 6 + 8;
-	else
-		right_len = 0;
-	FreeIfNeeded(fmt_thread);
+	right_len = strwidth(_(txt_thread_x_of_n)) - 6 + 8;
 
 	/*
 	 * limit right_len to cCOLS / 3
 	 */
-	if (right_len > cCOLS / 3 + 1)
-		right_len = cCOLS / 3 + 1;
+	if (right_len > CCOLS_THIRD)
+		right_len = CCOLS_THIRD;
 
 	/* date */
-	if ((wtmp = char2wchar_t(buf)) != NULL) {
-		my_fputws(wtmp, stdout);
-		if ((n = wcswidth(wtmp, wcslen(wtmp))) > 0)
-			cur_pos += n;
-		free(wtmp);
-	}
+	my_fputs(buf, stdout);
+	cur_pos += strwidth(buf);
 
 	/*
 	 * determine max len for centered group name
@@ -1530,27 +1581,37 @@ draw_page_header(
 		len = 0;
 
 	/* group name */
-	if ((wtmp = char2wchar_t(curr_group->name)) != NULL && len > 0) {
-		/* wconvert_to_printable(wtmp, FALSE); */
-		if (tinrc.abbreviate_groupname)
-			wtmp2 = abbr_wcsgroupname(wtmp, len);
-		else
-			wtmp2 = wstrunc(wtmp, len);
-
-		if ((i = wcswidth(wtmp2, wcslen(wtmp2))) < len)
-			len = i;
-
-		center_pos = (cCOLS - len) / 2;
-
-		/* pad out to left */
-		for (; cur_pos < center_pos; cur_pos++)
-			my_fputc(' ', stdout);
-
-		my_fputws(wtmp2, stdout);
-		cur_pos += wcswidth(wtmp2, wcslen(wtmp2));
-		free(wtmp2);
+	if (len > 0) {
+		if (tinrc.abbreviate_groupname) {
+#if defined(MULTIBYTE_ABLE) && !defined(NO_LOCALE)
+			if ((wtmp = char2wchar_t(curr_group->name)) != NULL) {
+				/* wconvert_to_printable(wtmp, FALSE); */
+				wtmp2 = abbr_wcsgroupname(wtmp, len);
+				tmp = wchar_t2char(wtmp2);
+				free(wtmp);
+				free(wtmp2);
+			}
+#else
+			tmp = abbr_groupname(curr_group->name, len);
+#endif /* MULTIBYTE_ABLE && !NO_LOCALE */
+		} else
+			tmp = strunc(curr_group->name, len);
 	}
-	FreeIfNeeded(wtmp);
+	if (!tmp)
+		tmp = my_strdup("");
+
+	if ((i = strwidth(tmp)) < len)
+		len = i;
+
+	center_pos = (cCOLS - len) / 2;
+
+	/* pad out to left */
+	for (; cur_pos < center_pos; cur_pos++)
+		my_fputc(' ', stdout);
+
+	my_fputs(tmp, stdout);
+	cur_pos += strwidth(tmp);
+	FreeAndNull(tmp);
 
 	/* pad out to right */
 	for (; cur_pos < cCOLS - right_len - 1; cur_pos++)
@@ -1572,15 +1633,15 @@ draw_page_header(
 		tlen = (size_t) n + 1;
 		tmp2 = my_malloc(tlen);
 		if (snprintf(tmp2, tlen, _(txt_thread_x_of_n), buf, buf2) != n) {
-			free(tmp2);
+			FreeAndNull(tmp2);
 			free(buf2);
 			goto skip;
 		}
-		tmp = strunc(tmp2, cCOLS / 3 - 1);
+		tmp = strunc(tmp2, CCOLS_THIRD);
 		my_printf("%s", tmp);
-		free(tmp2);
+		FreeAndNull(tmp);
+		FreeAndNull(tmp2);
 		free(buf2);
-		free(tmp);
 skip:
 		my_fputs(cCRLF, stdout);
 	}
@@ -1595,28 +1656,24 @@ skip:
 	cur_pos = 0;
 
 	/*
-	 * Convert to wide-char format string and determine the needed space
-	 * for the text at the right hand margin. The formatting info (%4s)
-	 * needs 3 positions but we need 4 positions on the screen for each
-	 * counter.
+	 * Determine the needed space for the text at the right hand margin
+	 * the formatting info (%4s) needs 3 positions but we need 4 positions
+	 * on the screen for each counter
 	 */
-
-	right_len = 0;
-	if (whichresp && (fmt_resp = char2wchar_t(_(txt_art_x_of_n)))) {
-		right_len = wcswidth(fmt_resp, wcslen(fmt_resp)) - 6 + 8;
-	} else {
-		if ((!x_resp && (fmt_resp = char2wchar_t(_(txt_no_responses)))) || (x_resp == 1 && (fmt_resp = char2wchar_t(_(txt_1_resp)))))
-			right_len = wcswidth(fmt_resp, wcslen(fmt_resp));
-		else if ((fmt_resp = char2wchar_t(_(txt_x_resp))))
-			right_len = wcswidth(fmt_resp, wcslen(fmt_resp)) - 3 + 4;
+	if (whichresp)
+		right_len = strwidth(_(txt_art_x_of_n)) - 6 + 8;
+	else {
+		if (!x_resp)
+			right_len = strwidth(_(txt_no_responses));
+		else
+			right_len = strwidth(P_(txt_x_resp_sp[0], txt_x_resp_sp[1], x_resp)) - 3 + 4;
 	}
-	FreeIfNeeded(fmt_resp);
 
 	/*
 	 * limit right_len to cCOLS / 3
 	 */
-	if (right_len > cCOLS / 3 + 1)
-		right_len = cCOLS / 3 + 1;
+	if (right_len > CCOLS_THIRD)
+		right_len = CCOLS_THIRD;
 
 	/* line count */
 	if (arts[this_resp].line_count < 0)
@@ -1624,64 +1681,61 @@ skip:
 	else
 		snprintf(buf, line_len, "%-4d", arts[this_resp].line_count);
 
-	{
-		wchar_t *fmt;
-
-		if ((wtmp = char2wchar_t(_(txt_lines))) != NULL) {
-			int tex_space = pgart.tex2iso ? 5 : 0;
-
-			fmt = wstrunc(wtmp, cCOLS / 3 - 1 - tex_space);
-			wtmp = my_realloc(wtmp, sizeof(wchar_t) * line_len);
-			swprintf(wtmp, line_len, fmt, buf);
-			my_fputws(wtmp, stdout);
-			cur_pos += wcswidth(wtmp, wcslen(wtmp));
-			free(fmt);
-			free(wtmp);
+	if ((n = snprintf(NULL, 0, _(txt_lines), buf)) > 0) {
+		tlen = (size_t) n + 1;
+		tmp = my_malloc(tlen);
+		if (snprintf(tmp, tlen, _(txt_lines), buf) == n) {
+			tmp2 = strunc(tmp, CCOLS_THIRD);
+			my_fputs(tmp2, stdout);
+			cur_pos += strwidth(tmp2);
 		}
+		free(tmp);
+		free(tmp2);
 	}
 
-#	ifdef HAVE_COLOR
-	fcol(tinrc.col_subject);
-#	endif /* HAVE_COLOR */
-
 	/* tex2iso */
-	if (pgart.tex2iso) {
-		if ((wtmp = char2wchar_t(_(txt_tex))) != NULL) {
-			wtmp2 = wstrunc(wtmp, 5);
-			my_fputws(wtmp2, stdout);
-			cur_pos += wcswidth(wtmp2, wcslen(wtmp2));
-			free(wtmp);
-			free(wtmp2);
-		}
+	if (pgart.tex2iso && tex2iso_conv) {
+		tex_space = strwidth(_(txt_tex));
+		if (tex_space > CCOLS_THIRD - cur_pos)
+			tex_space = CCOLS_THIRD - cur_pos;
+
+		tmp = strunc(_(txt_tex), tex_space);
+		tex_space = strwidth(tmp);
+		my_fputs(tmp, stdout);
+		cur_pos += tex_space;
+		free(tmp);
 	}
 
 	/* subject */
+#	ifdef HAVE_COLOR
+	fcol(tinrc.col_subject);
+#	endif /* HAVE_COLOR */
 	/*
 	 * TODO: why do we fall back to arts[this_resp].subject if !note_h->subj?
 	 *       if !note_h->subj then the article just has no subject, no matter
 	 *       what the overview says.
 	 *
-	 *       add BiDi handling
+	 *       add BiDi handling (our layout expects LTR)
 	 */
 	strncpy(buf, (note_h->subj ? note_h->subj : arts[this_resp].subject), line_len);
 	buf[line_len - 1] = '\0';
-	if ((wtmp = char2wchar_t(buf)) != NULL) {
-		wbuf = wexpand_tab(wtmp, tabwidth);
-		wtmp2 = wstrunc(wbuf, cCOLS - 2 * right_len - 4);
-		center_pos = (cCOLS - wcswidth(wtmp2, wcslen(wtmp2))) / 2;
 
-		/* pad out to left */
-		for (; cur_pos < center_pos; cur_pos++)
-			my_fputc(' ', stdout);
+	tmp2 = expand_tab(buf, tabwidth);
+	tmp = strunc(tmp2, cCOLS - 2 * (MAX(cur_pos, right_len)) - 4);
 
-		StartInverse();
-		my_fputws(wtmp2, stdout);
-		EndInverse();
-		cur_pos += wcswidth(wtmp2, wcslen(wtmp2));
-		free(wtmp2);
-		free(wtmp);
-		free(wbuf);
-	}
+	n = strwidth(tmp);
+	center_pos = (cCOLS - n) / 2;
+
+	/* pad out to left */
+	for (; cur_pos < center_pos; cur_pos++)
+		my_fputc(' ', stdout);
+
+	StartInverse();
+	my_fputs(tmp, stdout);
+	EndInverse();
+	cur_pos += n;
+	free(tmp);
+	free(tmp2);
 
 #	ifdef HAVE_COLOR
 	fcol(tinrc.col_response);
@@ -1701,29 +1755,25 @@ skip:
 			free(tmp2);
 			goto shrug;
 		}
-		tmp = strunc(tmp2, cCOLS / 3 - 1);
+		tmp = strunc(tmp2, CCOLS_THIRD);
 		my_printf("%s", tmp);
 		free(tmp2);
 		free(tmp);
 	} else {
-		/* TODO: ngettext */
 		if (!x_resp) {
-			tmp = strunc(_(txt_no_responses), cCOLS / 3 - 1);
-			my_printf("%s", tmp);
-		} else if (x_resp == 1) {
-			tmp = strunc(_(txt_1_resp), cCOLS / 3 - 1);
+			tmp = strunc(_(txt_no_responses), CCOLS_THIRD);
 			my_printf("%s", tmp);
 		} else {
-			if ((n = snprintf(NULL, 0, _(txt_x_resp), x_resp)) < 0)
+			if ((n = snprintf(NULL, 0, P_(txt_x_resp_sp[0], txt_x_resp_sp[1], x_resp), x_resp)) < 0)
 				goto shrug;
 
 			tlen = (size_t) n + 1;
 			tmp2 = my_malloc(tlen);
-			if (snprintf(tmp2, tlen, _(txt_x_resp), x_resp) != n) {
+			if (snprintf(tmp2, tlen, P_(txt_x_resp_sp[0], txt_x_resp_sp[1], x_resp), x_resp) != n) {
 				free(tmp2);
 				goto shrug;
 			}
-			tmp = strunc(tmp2, cCOLS / 3 - 1);
+			tmp = strunc(tmp2, CCOLS_THIRD);
 			my_printf("%s", tmp);
 			free(tmp2);
 		}
@@ -1740,225 +1790,7 @@ shrug:
 #	ifdef HAVE_COLOR
 	fcol(tinrc.col_from);
 #	endif /* HAVE_COLOR */
-	/* from */
-	from = build_from_line();
-	snprintf(buf, line_len, "%s", from);
-	free(from);
 
-	/* precalculate min. organization length */
-	wtmp3 = my_calloc(1, sizeof(wchar_t));
-	if (note_h->org) {
-		char *tb;
-		char *tail = NULL;
-
-		if (tinrc.utf8_graphics)
-			tail = wchar_t2char(WTRUNC_TAIL);
-
-		if ((n = snprintf(NULL, 0, _(txt_at_s), tail ? tail : TRUNC_TAIL)) > 0) {
-			size_t olen = (size_t) n + 1;
-
-			tb = my_malloc(olen);
-			if (snprintf(tb, olen, _(txt_at_s), tail ? tail : TRUNC_TAIL) == n) {
-				if ((wtmp = char2wchar_t(tb)) != NULL) {
-					wbuf = wexpand_tab(wtmp, tabwidth);
-					free(wtmp3);
-					wtmp3 = my_wcsdup(wbuf);
-					free(wtmp);
-					free(wbuf);
-				}
-			}
-			free(tb);
-		}
-		FreeIfNeeded(tail);
-	}
-
-	/* from */
-	if ((wtmp = char2wchar_t(buf)) != NULL) {
-		wtmp2 = wstrunc(wtmp, cCOLS - 1 - wcslen(wtmp3));
-		my_fputws(wtmp2, stdout);
-		cur_pos += wcswidth(wtmp2, wcslen(wtmp2));
-		free(wtmp2);
-		free(wtmp);
-	}
-	free(wtmp3);
-
-	/*
-	 * Organization
-	 *
-	 * TODO: - IDNA decoding, see also comment in
-	 *         cook.c:cook_article()
-	 *       - add BiDi handling
-	 */
-	if (note_h->org && cur_pos < cCOLS - 1) {
-		snprintf(buf, line_len, _(txt_at_s), note_h->org);
-
-		if ((wtmp = char2wchar_t(buf)) != NULL) {
-			wbuf = wexpand_tab(wtmp, tabwidth);
-			wtmp2 = wstrunc(wbuf, cCOLS - cur_pos - 1);
-
-			i = cCOLS - wcswidth(wtmp2, wcslen(wtmp2)) - 1;
-			for (; cur_pos < i; cur_pos++)
-				my_fputc(' ', stdout);
-
-			my_fputws(wtmp2, stdout);
-			free(wtmp2);
-			free(wtmp);
-			free(wbuf);
-		}
-	}
-
-#else /* !MULTIBYTE_ABLE || NO_LOCALE */
-
-	/*
-	 * determine the needed space for the text at the right hand margin
-	 * the formatting info (%4s) needs 3 positions but we need 4 positions
-	 * on the screen for each counter
-	 */
-	right_len = strlen(_(txt_thread_x_of_n)) - 6 + 8;
-
-	/* date */
-	my_fputs(buf, stdout);
-	cur_pos += strlen(buf);
-
-	/*
-	 * determine max len for centered group name
-	 * allow one space before and after group name
-	 */
-	len = cCOLS - 2 * MAX(cur_pos, right_len) - 3;
-
-	/* group name */
-	if (tinrc.abbreviate_groupname)
-		tmp = abbr_groupname(curr_group->name, len);
-	else
-		tmp = strunc(curr_group->name, len);
-
-	if ((i = strlen(tmp)) < len)
-		len = i;
-
-	center_pos = (cCOLS - len) / 2;
-
-	/* pad out to left */
-	for (; cur_pos < center_pos; cur_pos++)
-		my_fputc(' ', stdout);
-
-	my_fputs(tmp, stdout);
-	cur_pos += strlen(tmp);
-	free(tmp);
-
-	/* pad out to right */
-	for (; cur_pos < cCOLS - right_len - 1; cur_pos++)
-		my_fputc(' ', stdout);
-
-	/* thread info */
-	/* can't eval tin_ltoa() more than once in a statement due to statics */
-	strcpy(buf, tin_ltoa(which_thread(this_resp) + 1, 4));
-	my_printf(_(txt_thread_x_of_n), buf, tin_ltoa(grpmenu.max, 4));
-
-	my_fputs(cCRLF, stdout);
-
-#	if 0
-	/* display a ruler for layout checking purposes */
-	my_fputs("....|....3....|....2....|....1....|....0....|....1....|....2....|....3....|....\n", stdout);
-#	endif /* 0 */
-
-	/*
-	 * second line
-	 */
-	cur_pos = 0;
-
-	/*
-	 * determine the needed space for the text at the right hand margin
-	 * the formatting info (%4s) needs 3 positions but we need 4 positions
-	 * on the screen for each counter
-	 */
-	if (whichresp) {
-		right_len = strlen(_(txt_art_x_of_n)) - 6 + 8;
-	} else {
-		if (!x_resp)
-			right_len = strlen(_(txt_no_responses));
-		else if (x_resp == 1)
-			right_len = strlen(_(txt_1_resp));
-		else
-			right_len = strlen(_(txt_x_resp)) - 3 + 4;
-	}
-
-	/* line count */
-	/* an accurate line count will appear in the footer anymay */
-	if (arts[this_resp].line_count < 0)
-		strcpy(buf, "?");
-	else
-		snprintf(buf, line_len, "%-4d", arts[this_resp].line_count);
-
-	tmp = my_malloc(line_len);
-	snprintf(tmp, line_len, _(txt_lines), buf);
-	my_fputs(tmp, stdout);
-	cur_pos += strlen(tmp);
-	free(tmp);
-
-#	ifdef HAVE_COLOR
-	fcol(tinrc.col_subject);
-#	endif /* HAVE_COLOR */
-
-	/* tex2iso */
-	if (pgart.tex2iso) {
-		my_fputs(_(txt_tex), stdout);
-		cur_pos += strlen(_(txt_tex));
-	}
-
-	/* subject */
-	/*
-	 * TODO: why do we fall back to arts[this_resp].subject if !note_h->subj?
-	 *       if !note_h->subj then the article just has no subject, no matter
-	 *       what the overview says.
-	 */
-	strncpy(buf, (note_h->subj ? note_h->subj : arts[this_resp].subject), line_len);
-	buf[line_len - 1] = '\0';
-
-	tmp2 = expand_tab(buf, tabwidth);
-	tmp = strunc(tmp2, cCOLS - 2 * right_len - 3);
-
-	center_pos = (cCOLS - strlen(tmp)) / 2;
-
-	/* pad out to left */
-	for (; cur_pos < center_pos; cur_pos++)
-		my_fputc(' ', stdout);
-
-	StartInverse();
-	my_fputs(tmp, stdout);
-	EndInverse();
-	cur_pos += strlen(tmp);
-	free(tmp);
-	free(tmp2);
-
-#	ifdef HAVE_COLOR
-	fcol(tinrc.col_response);
-#	endif /* HAVE_COLOR */
-
-	/* pad out to right */
-	for (; cur_pos < cCOLS - right_len - 1; cur_pos++)
-		my_fputc(' ', stdout);
-
-	if (whichresp)
-		my_printf(_(txt_art_x_of_n), whichresp + 1, x_resp + 1);
-	else {
-		/* TODO: ngettext */
-		if (!x_resp)
-			my_printf("%s", _(txt_no_responses));
-		else if (x_resp == 1)
-			my_printf("%s", _(txt_1_resp));
-		else
-			my_printf(_(txt_x_resp), x_resp);
-	}
-	my_fputs(cCRLF, stdout);
-
-	/*
-	 * third line
-	 */
-	cur_pos = 0;
-
-#	ifdef HAVE_COLOR
-	fcol(tinrc.col_from);
-#	endif /* HAVE_COLOR */
 	/* from */
 	from = build_from_line();
 	snprintf(buf, line_len, "%s", from);
@@ -1966,16 +1798,23 @@ shrug:
 
 	tmp = strunc(buf, cCOLS - 1);
 	my_fputs(tmp, stdout);
-	cur_pos += strlen(tmp);
+	cur_pos += strwidth(tmp);
 	free(tmp);
 
+	/*
+	 * Organization
+	 *
+	 * TODO: - IDNA decoding, see also comment in
+	 *         cook.c:cook_article()
+	 *       - add BiDi handling (our layout expects LTR)
+	 */
 	if (note_h->org && cCOLS - cur_pos - 1 >= (int) strlen(_(txt_at_s)) - 2 + 3) {
 		/* we have enough space to print at least " at ..." */
 		snprintf(buf, line_len, _(txt_at_s), note_h->org);
 
 		tmp2 = expand_tab(buf, tabwidth);
 		tmp = strunc(tmp2, cCOLS - cur_pos - 1);
-		len = cCOLS - (int) strlen(tmp) - 1;
+		len = cCOLS - (int) strwidth(tmp) - 1;
 		for (; cur_pos < len; cur_pos++)
 			my_fputc(' ', stdout);
 		my_fputs(tmp, stdout);
@@ -1983,8 +1822,6 @@ shrug:
 		free(tmp2);
 	}
 
-#endif /* MULTIBYTE_ABLE && !NO_LOCALE */
-
 	my_fputs(cCRLF, stdout);
 	my_fputs(cCRLF, stdout);
 
@@ -1994,6 +1831,7 @@ shrug:
 	fcol(tinrc.col_normal);
 #endif /* HAVE_COLOR */
 }
+#undef CCOLS_THIRD
 
 
 /*
@@ -2015,12 +1853,8 @@ load_article(
 #endif /* DEBUG */
 
 	if (new_respnum != this_resp || art_closed) {
-		int ret;
-
 		art_close(&pgart);			/* close previously opened art in pager */
-		ret = art_open(TRUE, &arts[new_respnum], group, &pgart, TRUE, _(txt_reading_article));
-
-		switch (ret) {
+		switch (art_open(TRUE, &arts[new_respnum], group, &pgart, TRUE, _(txt_reading_article))) {
 			case ART_UNAVAILABLE:
 				art_mark(group, &arts[new_respnum], ART_READ);
 				/* prevent retagging as unread in unfilter_articles() */
@@ -2097,7 +1931,7 @@ load_article(
 	rotate = 0;			/* normal mode, not rot13 */
 	reveal_ctrl_l = FALSE;
 	reveal_ctrl_l_lines = -1;	/* all ^L's active */
-	hide_uue = tinrc.hide_uue;
+	hide_inline_data = group->attribute->hide_inline_data;
 
 	draw_page(0);
 
@@ -2107,7 +1941,7 @@ load_article(
 	if (!note_h->mime || IS_PLAINTEXT(note_h->ext))		/* Text only article */
 		return 0;
 
-	if (*tinrc.metamail_prog == '\0' || getenv("NOMETAMAIL") != NULL)	/* Viewer turned off */
+	if (!tinrc.metamail_prog || !*tinrc.metamail_prog || getenv("NOMETAMAIL") != NULL)	/* Viewer turned off */
 		return 0;
 
 	if (group->attribute->ask_for_metamail) {
@@ -2195,7 +2029,7 @@ process_search(
 	/*
 	 * Highlight found string
 	 */
-	highlight_string(i - *lcurr_line + scroll_region_top, start, end - start);
+	highlight_string(i - *lcurr_line + scroll_region_top, (int) start, (int) (end - start));
 }
 
 
@@ -2236,7 +2070,7 @@ toggle_raw(
 
 				pgart.rawl[j].offset = offset;
 				pgart.rawl[j].flags = 0;
-				j++;
+				++j;
 				if (j >= chunk) {
 					chunk += 50;
 					pgart.rawl = my_realloc(pgart.rawl, sizeof(t_lineinfo) * (size_t) chunk);
@@ -2257,9 +2091,9 @@ toggle_raw(
 						}
 #else
 						if (my_isprint((unsigned char) *p)) {
-							space--;
-							p++;
-							offset++;
+							--space;
+							++p;
+							++offset;
 						}
 #endif /* MULTIBYTE_ABLE && !NO_LOCALE */
 						else if (IS_LOCAL_CHARSET("Big5") && (unsigned char) *p >= 0xa1 && (unsigned char) *p <= 0xfe && *(p + 1)) {
@@ -2271,7 +2105,7 @@ toggle_raw(
 							 */
 							p += 2;
 							offset += 2;
-							space--;
+							--space;
 						} else {
 							/*
 							 * the current character can't be displayed print it as
@@ -2280,8 +2114,8 @@ toggle_raw(
 							 */
 							if ((space -= 4) < 0)
 								break;
-							offset++;
-							p++;
+							++offset;
+							++p;
 						}
 					}
 					/*
@@ -2318,11 +2152,11 @@ toggle_raw(
 
 
 void
-update_hide_uue(
+update_hide_inline_data(
 	void)
 {
-	if (hide_uue != tinrc.hide_uue)
-		hide_uue = tinrc.hide_uue;
+	if (hide_inline_data != curr_group->attribute->hide_inline_data)
+		hide_inline_data = curr_group->attribute->hide_inline_data;
 }
 
 
@@ -2338,7 +2172,7 @@ resize_article(
 	if (artinfo->cooked)
 		fclose(artinfo->cooked);
 
-	if (!cook_article(wrap_lines, artinfo, hide_uue, show_all_headers))
+	if (!cook_article(wrap_lines, artinfo, hide_inline_data, show_all_headers))
 		tin_done(EXIT_FAILURE, _(txt_cook_article_failed_exiting), tin_progname);
 
 	show_raw_article = FALSE;
@@ -2566,14 +2400,14 @@ preprocess_info_message(
 	do {
 		infoline[num_info_lines].offset = ftell(info_fh);
 		infoline[num_info_lines].flags = 0;
-		num_info_lines++;
+		++num_info_lines;
 		if (num_info_lines >= chunk) {
 			chunk += 50;
 			infoline = my_realloc(infoline, sizeof(t_lineinfo) * (size_t) chunk);
 		}
 	} while (tin_fgets(info_fh, FALSE) != NULL);
 
-	num_info_lines--;
+	--num_info_lines;
 	infoline = my_realloc(infoline, sizeof(t_lineinfo) * (size_t) num_info_lines);
 }
 
@@ -2601,7 +2435,7 @@ static void
 show_url_page(
 	void)
 {
-	int i;
+	int i, prev_mark_offset = mark_offset;
 
 	signal_context = cURL;
 	currmenu = &urlmenu;
@@ -2620,6 +2454,7 @@ show_url_page(
 	show_mini_help(URL_LEVEL);
 
 	draw_url_arrow();
+	mark_offset = prev_mark_offset;
 }
 
 
@@ -2727,7 +2562,8 @@ url_page(
 
 			case GLOBAL_TOGGLE_INFO_LAST_LINE:
 				tinrc.info_in_last_line = bool_not(tinrc.info_in_last_line);
-				show_url_page();
+				clear_message();
+				draw_url_arrow();
 				break;
 
 			case GLOBAL_TOGGLE_INVERSE_VIDEO:
@@ -2813,7 +2649,7 @@ build_url_line(
 	 * Allocate line buffer
 	 * make it the same size like in !USE_CURSES case to simplify some code
 	 */
-	sptr = my_malloc(cCOLS + 2);
+	sptr = my_malloc((size_t) cCOLS + 2);
 #else
 	sptr = screen[INDEX2SNUM(i)].col;
 #endif /* USE_CURSES */
@@ -2832,10 +2668,23 @@ static t_bool
 process_url(
 	int n)
 {
-	char *url, *url_esc;
+	char *url;
+	const char *url_esc;
 	int l;
 	size_t len;
 	t_url *lptr;
+#if defined(HAVE_LIBURIPARSER) || defined(HAVE_LIBCURL)
+	char *uri_norm;
+	int ulen = -1;
+#endif /* HAVE_LIBURIPARSER || HAVE_LIBCURL */
+#ifdef HAVE_LIBURIPARSER
+	UriParserStateA state;
+	UriUriA uri;
+#else
+#	ifdef HAVE_LIBCURL
+	CURLU *curl;
+#	endif /* HAVE_LIBCURL */
+#endif /* HAVE_LIBURIPARSER */
 
 	lptr = find_url(n);
 	len = strlen(lptr->url) << 1; /* double size; room for editing URL */
@@ -2845,12 +2694,61 @@ process_url(
 			free(url);
 			return FALSE;
 		}
+
+#ifdef HAVE_LIBURIPARSER
+		/*
+		 * Syntax-Based Normalization RFC 3986 6.2.2
+		 *
+		 * We could keep the error code, but that would be likely
+		 * always URI_ERROR_SYNTAX, so no big win.
+		 * With -DDEBUG MISC|URI we would write some details
+		 * (uri.scheme, uri.hostText, ...) to a log.
+		 * And/or make use of uri(Une|E)scape* ...
+		 */
+		state.uri = &uri;
+		if (uriParseUriA(&state, url) == URI_SUCCESS) {
+			if (uriNormalizeSyntaxA(&uri) == URI_SUCCESS) {
+				if (uriToStringCharsRequiredA(&uri, &ulen) == URI_SUCCESS) {
+					uri_norm = my_malloc(++ulen);
+					if (uriToStringA(uri_norm, &uri, ulen, NULL) == URI_SUCCESS) {
+						free(url);
+						url = uri_norm;
+					} else {
+						free(uri_norm);
+						ulen = -1;
+					}
+				}
+			}
+			uriFreeUriMembersA(&uri);
+		}
+#else
+#	ifdef HAVE_LIBCURL
+		/* same as above but with libcurl, again no detailed error logging yet */
+		if ((curl = curl_url())) {
+			if (curl_url_set(curl, CURLUPART_URL, url, CURLU_URLENCODE) == CURLUE_OK) {
+				if (curl_url_get(curl, CURLUPART_URL, &uri_norm, 0) == CURLUE_OK) {
+					free(url);
+					url = uri_norm;
+					ulen = strlen(url);
+				}
+			}
+			curl_url_cleanup(curl);
+		}
+#	endif /* HAVE_LIBCURL */
+#endif /* HAVE_LIBURIPARSER */
+#if defined(HAVE_LIBURIPARSER) || defined(HAVE_LIBCURL)
+		if (ulen < 0) { /* should only happen if the URL was modified in the prompt above */
+			error_message(2, "URI Normalization failed: %s", url); /* TODO: -> lang.c; _()? */
+			free(url);
+			return FALSE;
+		}
+#endif /* HAVE_LIBURIPARSER || HAVE_LIBCURL */
 		url_esc = escape_shell_meta(url, no_quote);
 		if ((l = snprintf(NULL, 0, "%s %s", tinrc.url_handler, url_esc)) < 0) {
 			free(url);
 			return FALSE;
 		}
-		wait_message(2, _(txt_url_open), url);
+		wait_message(2, _(txt_url_open), url); /* use url_esc in message? */
 		len = (size_t) l + 1;
 		url = my_realloc(url, len);
 		if (snprintf(url, len, "%s %s", tinrc.url_handler, url_esc) != l) {
@@ -2893,11 +2791,11 @@ build_url_list(
 		 */
 		forever {
 			/* any matches left? */
-			if (match_regex_ex(ptr, (REGEX_SIZE) strlen(ptr), 0, 0, &url_regex) >= 0) {
+			if (MATCH_REGEX(url_regex, ptr, strlen(ptr))) {
 				offsets = regex_get_ovector_pointer(&url_regex);
-			} else if (match_regex_ex(ptr, (REGEX_SIZE) strlen(ptr), 0, 0, &mail_regex) >= 0) {
+			} else if (MATCH_REGEX(mail_regex, ptr, strlen(ptr))) {
 				offsets = regex_get_ovector_pointer(&mail_regex);
-			} else if (match_regex_ex(ptr, (REGEX_SIZE) strlen(ptr), 0, 0, &news_regex) >= 0) {
+			} else if (MATCH_REGEX(news_regex, ptr, strlen(ptr))) {
 				offsets = regex_get_ovector_pointer(&news_regex);
 			} else
 				break;
@@ -2959,6 +2857,8 @@ log_article_info(
 	FILE *fp;
 	t_part *ptr;
 #ifdef DEBUG
+	const char *filename_star;
+	const char *filename;
 	const char *name;
 #endif /* DEBUG */
 
@@ -2990,6 +2890,8 @@ log_article_info(
 			fprintf(fp, " / %s: %s", "Unsupported", ptr->mime_hints.subtype);
 		else
 			fprintf(fp, " / %s", ptr->mime_hints.subtype);
+		if (ptr->format)
+			fprintf(fp, "%s", " ; Format=Flowed");
 		fprintf(fp, "\n\t\tUsed:\t\t%s / %s", content_types[ptr->type], ptr->subtype);
 
 		charset = get_param(ptr->params, "charset");
@@ -3026,7 +2928,18 @@ log_article_info(
 			fprintf(fp, "%s", content_encodings[ptr->encoding]);
 		fprintf(fp, "\n\t\tUsed:\t\t%s", content_encodings[ptr->encoding]);
 
+		if (artinfo->tex2iso)
+			fprintf(fp, "\n\n\t\t%s\n", _(txt_tex));
+
 #ifdef DEBUG
+		if ((filename_star = get_param(ptr->params, "filename*")) != NULL) {
+			fprintf(fp, "\n\tFilename*:\n");
+			fprintf(fp, "\t\t%s", filename_star);
+		}
+		if ((filename = get_param(ptr->params, "filename")) != NULL) {
+			fprintf(fp, "\n\tFilename:\n");
+			fprintf(fp, "\t\t%s", filename);
+		}
 		if ((name = get_param(ptr->params, "name")) != NULL) {
 			fprintf(fp, "\n\tName:\n");
 			fprintf(fp, "\t\t%s", name);
@@ -3043,6 +2956,13 @@ log_article_info(
 			fprintf(fp, "\n\tContent Disposition:\n");
 			fprintf(fp, "\t\t%s", ptr->disposition == DISP_INLINE ? content_disposition[DISP_INLINE] : content_disposition[DISP_ATTACHMENT]);
 		}
+		fprintf(fp, "\n");
+		if (filename_star && *filename_star && filename && *filename && strcmp(filename_star, filename))
+			fprintf(fp, "\n!!\tFilename*: %s != Filename: %s", filename_star, filename);
+		if (filename_star && *filename_star && name && *name && strcmp(filename_star, name))
+			fprintf(fp, "\n!!\tFilename*: %s != Name: %s", filename_star, name);
+		if (filename && *filename && name && *name && strcmp(filename, name))
+			fprintf(fp, "\n!!\tFilename: %s != Name: %s", filename, name);
 #endif /* DEBUG */
 		fprintf(fp, "\n");
 	}
