@@ -3,10 +3,10 @@
  *  Module    : pgp.c
  *  Author    : Steven J. Madsen
  *  Created   : 1995-05-12
- *  Updated   : 2024-07-11
+ *  Updated   : 2025-12-06
  *  Notes     : PGP support
  *
- * Copyright (c) 1995-2024 Steven J. Madsen <steve@erinet.com>
+ * Copyright (c) 1995-2026 Steven J. Madsen <steve@erinet.com>
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -85,6 +85,7 @@
 #		define PGPNAME		PATH_GPG
 #		define PGPDIR		".gnupg"
 #		define PGP_PUBRING	"pubring.gpg"
+#		define PGP_PUBRING_KBX	"pubring.kbx"	/* since GnuPG 2.1 */
 #		if 0 /* gpg 1.4.11 doesn't like this */
 #			define CHECK_SIGN	"%s %s --no-batch --decrypt <%s %s"
 #		else
@@ -110,9 +111,6 @@ PGPNAME, pgpopts, pt, mailto, mailfrom, pt
 #	endif /* HAVE_GPG */
 
 /* RFC 4880 6.2 */
-#	define PGP_SIG_TAG "-----BEGIN PGP SIGNED MESSAGE-----\n"
-#	define PGP_KEY_TAG "-----BEGIN PGP PUBLIC KEY BLOCK-----\n"
-
 #	define HEADERS	"tin-%ld.h"
 #	ifdef HAVE_LONG_FILE_NAMES
 #		define PLAINTEXT	"tin-%ld.pt"
@@ -131,7 +129,7 @@ PGPNAME, pgpopts, pt, mailto, mailfrom, pt
 static t_bool pgp_available(void);
 static void do_pgp(t_function what, const char *file, const char *mail_to);
 static void join_files(const char *file);
-static void pgp_append_public_key(char *file);
+static void pgp_append_public_key(const char *file);
 static void split_file(const char *file);
 
 static char pgp_data[PATH_LEN];
@@ -221,10 +219,10 @@ split_file(
 	if ((plaintext = fopen(pt, "w")) == NULL)
 		goto err_hdr;
 
-	if (fgets(buf, LEN, art) != NULL) {			/* Copy the hdr up to and including the \n */
+	if (fgets(buf, sizeof(buf), art) != NULL) {			/* Copy the hdr up to and including the \n */
 		t_bool success = TRUE;
 
-		while (strcmp(buf, "\n")) {
+		while (*buf != '\n') {
 			fputs(buf, header);
 			if (fgets(buf, LEN, art) == NULL) {
 				success = FALSE;
@@ -264,8 +262,8 @@ do_pgp(
 	/*
 	 * <mailfrom> is valid only when signing and a local address exists
 	 */
-	if ((CURR_GROUP.attribute->from) != NULL)
-		strip_name(CURR_GROUP.attribute->from, mailfrom);
+	if (CURR_GROUP.attribute->from != NULL && *CURR_GROUP.attribute->from && strchr(*CURR_GROUP.attribute->from, '@'))
+		strip_name(*CURR_GROUP.attribute->from, mailfrom);
 
 	switch (what) {
 		case PGP_KEY_SIGN:
@@ -299,16 +297,16 @@ do_pgp(
 
 static void
 pgp_append_public_key(
-	char *file)
+	const char *file)
 {
 	FILE *fp, *key;
 	char cmd[LEN], buf[LEN];
 	char keyfile[PATH_LEN], tmp[PATH_LEN];
 
-	if ((CURR_GROUP.attribute->from) != NULL && strlen(CURR_GROUP.attribute->from))
-		strip_name(CURR_GROUP.attribute->from, buf);
+	if (CURR_GROUP.attribute->from != NULL && *CURR_GROUP.attribute->from && strchr(*CURR_GROUP.attribute->from, '@'))
+		strip_name(*CURR_GROUP.attribute->from, buf);
 	else /* FIXME: avoid hardcoded length */
-		snprintf(buf, sizeof(buf), "%.255s@%.765s", userid, BlankIfNull(get_host_name()));
+		snprintf(buf, sizeof(buf), "%.*s@%.765s", LOGIN_NAME_MAX - 1, userid, BlankIfNull(get_host_name()));
 
 	snprintf(tmp, sizeof(tmp), KEYFILE, (long) process_id);
 	joinpath(keyfile, sizeof(keyfile), tmpdir, tmp);
@@ -342,21 +340,30 @@ pgp_available(
 	FILE *fp;
 	char keyring[PATH_LEN];
 
+#ifdef HAVE_GPG
+	/* GnuPG >= 2.1 may use PGP_PUBRING_KBX */
+	joinpath(keyring, sizeof(keyring), pgp_data, PGP_PUBRING_KBX);
+	if ((fp = fopen(keyring, "r")) != NULL) {
+		fclose(fp);
+		return TRUE;
+	}
+#endif /* HAVE_GPG */
+
 	joinpath(keyring, sizeof(keyring), pgp_data, PGP_PUBRING);
-	if ((fp = fopen(keyring, "r")) == NULL) {
-		wait_message(2, _(txt_pgp_not_avail), keyring);
-		return FALSE;
+	if ((fp = fopen(keyring, "r")) != NULL) {
+		fclose(fp);
+		return TRUE;
 	}
 
-	fclose(fp);
-	return TRUE;
+	wait_message(2, _(txt_pgp_not_avail), keyring);
+	return FALSE;
 }
 
 
 void
 invoke_pgp_mail(
 	const char *nam,
-	char *mail_to)
+	const char *mail_to)
 {
 	char keyboth[MAXKEYLEN], keyencrypt[MAXKEYLEN], keyquit[MAXKEYLEN];
 	char keysign[MAXKEYLEN];
@@ -401,7 +408,7 @@ invoke_pgp_mail(
 
 void
 invoke_pgp_news(
-	char *artfile)
+	const char *artfile)
 {
 	char keyinclude[MAXKEYLEN], keyquit[MAXKEYLEN], keysign[MAXKEYLEN];
 	t_function func, default_func = PGP_KEY_SIGN;
@@ -452,14 +459,15 @@ pgp_check_article(
 	int n;
 	size_t len;
 	t_bool pgp_signed = FALSE;
-	t_bool pgp_key = FALSE;
+	int pgp_key = 0;
 
 	if (!pgp_available())
 		return FALSE;
 
 	joinpath(artfile, sizeof(artfile), homedir, TIN_ARTICLE_NAME);
 #	ifdef APPEND_PID
-	snprintf(artfile + strlen(artfile), sizeof(artfile) - strlen(artfile), ".%ld", (long) process_id);
+	n = strlen(artfile);
+	snprintf(artfile + n, sizeof(artfile) - n, ".%ld", (long) process_id);
 #	endif /* APPEND_PID */
 	if ((art = fopen(artfile, "w")) == NULL) {
 		info_message(_(txt_cannot_open), artfile);
@@ -472,18 +480,18 @@ pgp_check_article(
 	}
 
 	while (!feof(artinfo->raw) && !ferror(artinfo->raw)) {
-		if (fgets(buf, LEN, artinfo->raw) != NULL) {
+		if (fgets(buf, sizeof(buf), artinfo->raw) != NULL) {
 			if (!pgp_signed && !strcmp(buf, PGP_SIG_TAG))
 				pgp_signed = TRUE;
-			if (!pgp_key && !strcmp(buf, PGP_KEY_TAG))
-				pgp_key = TRUE;
+			if (!strcmp(buf, PGP_PUBLIC_KEY_TAG))
+				++pgp_key;
 			fputs(buf, art);
 		} else
 			break;
 	}
 	fclose(art);
 
-	if (!(pgp_signed || pgp_key)) {
+	if (!pgp_signed && !pgp_key) {
 		info_message(_(txt_pgp_nothing));
 		return FALSE;
 	}
@@ -515,10 +523,10 @@ pgp_check_article(
 	InitWin();
 #	endif /* !USE_CURSES */
 	if (pgp_key) {
-		if (prompt_yn(_(txt_pgp_add), FALSE) == 1) {
+		if (prompt_yn(P_(txt_pgp_add_sp[0], txt_pgp_add_sp[1], pgp_key), FALSE) == 1) {
 			Raw(FALSE);
 			n = snprintf(NULL, 0, ADD_KEY, PGPNAME, pgpopts, artfile);
-			len = (size_t) (n << 1) + 1; /* double size for quoting */
+			len = ((size_t) n << 1) + 1; /* double size for quoting */
 			cmd = my_malloc(len);
 			sh_format(cmd, len, ADD_KEY, PGPNAME, pgpopts, artfile);
 			invoke_cmd(cmd);
