@@ -3,10 +3,10 @@
  *  Module    : prompt.c
  *  Author    : I. Lea
  *  Created   : 1991-04-01
- *  Updated   : 2024-03-16
+ *  Updated   : 2025-12-17
  *  Notes     :
  *
- * Copyright (c) 1991-2024 Iain Lea <iain@bricbrac.de>
+ * Copyright (c) 1991-2026 Iain Lea <iain@bricbrac.de>
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -53,8 +53,11 @@ static char *prompt_yn_choice;
 /*
  * Local prototypes
  */
-static int prompt_list(int row, int col, int var, constext *help_text, constext *prompt_text, constext *list[], int size);
-
+static int prompt_list(int row, int col, int *var, constext *help_text, constext *prompt_text, constext * const list[], int size);
+#ifdef HAVE_COLOR
+	static void show_color_help(void);
+	static void clear_color_help(void);
+#endif /* HAVE_COLOR */
 
 /*
  *  prompt_num
@@ -74,7 +77,7 @@ prompt_num(
 	snprintf(tmp, sizeof(tmp), "%c", ch);
 	if ((p = tin_getline(prompt, 1, tmp, 0, FALSE, HIST_OTHER)) != NULL) {
 		STRCPY(tmp, p);
-		num = atoi(tmp);
+		num = strtol(tmp, NULL, 10);
 	} else
 		num = -1;
 
@@ -95,7 +98,7 @@ prompt_string(
 	char *buf,
 	int which_hist)
 {
-	return prompt_default_string(prompt, buf, 0, (char *) NULL, which_hist);
+	return prompt_default_string(prompt, buf, 0, NULL, which_hist);
 }
 
 
@@ -109,7 +112,7 @@ prompt_default_string(
 	const char *prompt,
 	char *buf,
 	int buf_len,
-	char *default_prompt,
+	const char *default_prompt,
 	int which_hist)
 {
 	char *p;
@@ -135,7 +138,7 @@ t_bool
 prompt_menu_string(
 	int line,
 	const char *prompt,
-	char *var)
+	char **var)
 {
 	char *p;
 
@@ -146,10 +149,11 @@ prompt_menu_string(
 	 */
 	/* fflush(stdin); */
 	MoveCursor(line, 0);
-	if ((p = tin_getline(prompt, 0, var, 0, FALSE, HIST_OTHER)) == NULL)
+	if ((p = tin_getline(prompt, 0, *var, 0, FALSE, HIST_OTHER)) == NULL)
 		return FALSE;
 
-	strcpy(var, p);
+	FreeIfNeeded(*var);
+	*var = my_strdup(p);
 	return TRUE;
 }
 
@@ -160,6 +164,9 @@ prompt_menu_string(
  * last line giving the default answer "default_answer".
  * The function returns 1 if the user decided "yes", -1 if the user wanted
  * to escape, or 0 for any other decision.
+ *
+ * TODO: add nl_langinfo(YESEXPR) / nl_langinfo(NOEXPR), but use pcre
+ *       instead of rpmatch()
  */
 int
 prompt_yn(
@@ -244,7 +251,11 @@ prompt_yn(
 			default:
 				break;
 		}
+#if defined(MULTIBYTE_ABLE) && !defined(NO_LOCALE)
 		func = key_to_func((wchar_t) ch, prompt_keys);
+#else
+		func = key_to_func(ch, prompt_keys);
+#endif /* MULTIBYTE_ABLE && !NO_LOCALE */
 	} while (func == NOT_ASSIGNED);
 
 	input_context = cNone;
@@ -290,6 +301,92 @@ prompt_yn_redraw(
 }
 
 
+#ifdef HAVE_COLOR
+static void
+show_color_help(
+	void)
+{
+	char *buf;
+	int i, row, col, head, tail, len, col_width = cCOLS / 6;
+	int col_normal_orig = tinrc.col_normal;
+	int col_back_orig = tinrc.col_back;
+
+	clear_color_help();
+	for (row = 2, col = 0; row >= 0; row--, col += 6) {
+		MoveCursor(cLINES - row, 0);
+		for (i = col; i <= col + 5 && txt_colors[i]; i++) {
+			if (i == 0)
+				fcol(-1);
+			else if (i < 6)
+				fcol(15);
+			else
+				fcol(0);
+			bcol(i - 1);
+			len = strwidth(_(txt_colors[i]));
+			if (col_width > len) {
+				head = (col_width - len) / 2;
+				tail = col_width - len - head;
+				if (tail > 1 || (head == 1 && tail == 1))
+					/*
+					 * we cannot use %-*s for color names, because names
+					 * with multibyte chars would destroy the layout
+					 */
+					my_printf("%*c%s%*c", head, ' ', _(txt_colors[i]), tail, ' ');
+				else
+					my_printf("%s ", _(txt_colors[i]));
+			} else {
+				buf = strunc(_(txt_colors[i]), col_width);
+				my_printf("%s", buf);
+				free(buf);
+			}
+		}
+		bcol(col_back_orig);
+	}
+	fcol(col_normal_orig);
+}
+
+
+static void
+clear_color_help(
+	void)
+{
+	MoveCursor(cLINES - 2, 0);
+	CleartoEOS();
+}
+#endif /* HAVE_COLOR */
+
+
+#ifdef HAVE_COLOR
+#	define SET_OPT_COLOR() do { \
+		if (var == &tinrc.col_invers_bg || var == &tinrc.col_invers_fg) { \
+			int orig_var = *var; \
+			*var = idx - adjust; \
+			StartInverse(); \
+			*var = orig_var; \
+		} else if (var == &tinrc.col_back) \
+			bcol(idx - adjust); \
+		else \
+			fcol(idx - adjust); \
+	} while (0)
+
+#	define RESET_OPT_COLOR() do { \
+		if (var != &tinrc.col_back) \
+			bcol(old_bcol); \
+		if (var != &tinrc.col_normal) \
+			fcol(old_fcol); \
+		if (var == &tinrc.col_invers_bg || var == &tinrc.col_invers_fg) \
+			EndInverse(); \
+	} while (0)
+
+#	define RESTORE_OPT_COLOR() do { \
+		if (var == &tinrc.col_back) \
+			bcol(old_bcol); \
+		else if (var == &tinrc.col_normal) \
+			fcol(old_fcol); \
+		else if (var == &tinrc.col_invers_bg || var == &tinrc.col_invers_fg) \
+			EndInverse(); \
+	} while (0)
+#endif /* HAVE_COLOR */
 /*
  * help_text is displayed near the bottom of the screen.
  * var is an index into a list containing size elements.
@@ -302,22 +399,25 @@ static int
 prompt_list(
 	int row,
 	int col,
-	int var,
+	int *var,
 	constext *help_text,
 	constext *prompt_text,
-	constext *list[],
+	constext * const list[],
 	int size)
 {
-	int ch, var_orig;
-	int i, offset, width = 0;
-	int change;
+	int ch, change, i, idx, offset, width = 0;
 	int adjust = (strcasecmp(_(list[0]), _(txt_default)) == 0);
 #if defined(MULTIBYTE_ABLE) && !defined(NO_LOCALE)
 	char *buf;
 #endif /* MULTIBYTE_ABLE && !NO_LOCALE */
+#ifdef HAVE_COLOR
+	/* initialization to silence compiler warnings */
+	t_bool color_opt = use_color && (list == txt_colors || list == txt_backcolors);
+	int old_bcol = tinrc.col_back;
+	int old_fcol = tinrc.col_normal;
+#endif /* HAVE_COLOR */
 
-	var += adjust;
-	var_orig = var;
+	idx = *var + adjust;
 
 	/*
 	 * Find the length of longest printable text
@@ -325,7 +425,13 @@ prompt_list(
 	for (i = 0; i < size; i++)
 		width = MAX(width, strwidth(_(list[i])));
 
-	show_menu_help(help_text);
+#ifdef HAVE_COLOR
+	if (color_opt) {
+		(void) help_text;
+		show_color_help();
+	} else
+#endif /* HAVE_COLOR */
+		show_menu_help(help_text);
 	cursoron();
 
 	offset = strwidth(_(prompt_text));
@@ -336,10 +442,35 @@ prompt_list(
 	if (offset + width >= cCOLS)
 		width = cCOLS - offset - 1;
 
+#ifdef HAVE_COLOR
+	if (color_opt) {
+		MoveCursor(row, col + offset);
+		SET_OPT_COLOR();
+#	if defined(MULTIBYTE_ABLE) && !defined(NO_LOCALE)
+		if ((buf = spart(_(list[idx]), width, TRUE)) != NULL) {
+			my_printf("%s", buf);
+			free(buf);
+		} else
+#	endif /* MULTIBYTE_ABLE && !NO_LOCALE */
+			my_printf("%-*s", width, _(list[idx]));
+		RESET_OPT_COLOR();
+		my_flush();
+	}
+#	endif /* HAVE_COLOR */
+
+	input_context = cPromptList;
+
 	do {
 		MoveCursor(row, col + offset);
 		ch = (char) ReadCh();
 
+#ifdef USE_CURSES
+		if (pending_sactions && ch != EOF) {
+/*			ungetch('\r'); */ /* reenter option, would differ from !USE_CURSES */
+			ungetch(ESC);
+		}
+#endif /* USE_CURSES */
+
 		/*
 		 * change:
 		 *   1 = move to the next list element
@@ -380,42 +511,69 @@ prompt_list(
 				break;
 		}
 
-		if (change) {
+		if (change && !pending_sactions) {
 			/*
 			 * increment or decrement list, loop around at the limits
 			 */
-			var += change;
-			if (var < 0)
-				var = size - 1;
-			else
-				var %= (size ? size : 1);
+			do {
+				idx += change;
+				if (idx < 0)
+					idx = size - 1;
+				else
+					idx %= (size ? size : 1);
+			} while (!*list[idx]); /* allow gaps in lists */
 
+#	ifdef HAVE_COLOR
+			if (color_opt)
+				SET_OPT_COLOR();
+#	endif /* HAVE_COLOR */
 #if defined(MULTIBYTE_ABLE) && !defined(NO_LOCALE)
-			if ((buf = spart(_(list[var]), width, TRUE)) != NULL) {
+			if ((buf = spart(_(list[idx]), width, TRUE)) != NULL) {
 				my_printf("%s", buf);
 				free(buf);
 			} else
 #endif /* MULTIBYTE_ABLE && !NO_LOCALE */
-				my_printf("%-*s", width, _(list[var]));
+				my_printf("%-*s", width, _(list[idx]));
+#	ifdef HAVE_COLOR
+			if (color_opt)
+				RESET_OPT_COLOR();
+#	endif /* HAVE_COLOR */
 			my_flush();
 		}
-	} while (ch != '\r' && ch != '\n' && ch != ESC);
+	} while (ch != '\r' && ch != '\n' && ch != ESC && !pending_sactions);
 
-	if (ch == ESC) {
-		var = var_orig;
+	input_context = cNone;
+
+	if (ch == ESC || pending_sactions) {
+		pending_sactions = 0;
+#	ifdef HAVE_COLOR
+		if (color_opt)
+			RESTORE_OPT_COLOR();
+#	endif /* HAVE_COLOR */
 #if defined(MULTIBYTE_ABLE) && !defined(NO_LOCALE)
-		if ((buf = spart(_(list[var]), width, TRUE)) != NULL) {
+		if ((buf = spart(_(list[*var]), width, TRUE)) != NULL) {
 			my_printf("%s", buf);
 			free(buf);
 		} else
 #endif /* MULTIBYTE_ABLE && !NO_LOCALE */
-			my_printf("%-*s", width, _(list[var]));
+			my_printf("%-*s", width, _(list[*var]));
 		my_flush();
-	}
+	} else
+		*var = idx - adjust;
+
+#ifdef HAVE_COLOR
+	if (color_opt)
+		clear_color_help();
+#endif /* HAVE_COLOR */
 
 	cursoroff();
-	return (var - adjust);
+	return *var;
 }
+#ifdef HAVE_COLOR
+#	undef SET_OPT_COLOR
+#	undef RESET_OPT_COLOR
+#	undef RESTORE_OPT_COLOR
+#endif /* HAVE_COLOR */
 
 
 /*
@@ -431,7 +589,7 @@ prompt_option_on_off(
 	t_bool old_value = *variable;
 
 	fmt_option_prompt(prompt, sizeof(prompt), TRUE, option);
-	*variable = prompt_list(option_row(option), 0, (int) *variable, option_table[option].txt->help, prompt, txt_onoff, 2) ? TRUE : FALSE;
+	*variable = prompt_list(option_row(option), 0, (int *) variable, option_table[option].txt->help, prompt, txt_onoff, 2) ? TRUE : FALSE;
 	return bool_not(bool_equal(*variable, old_value));
 }
 
@@ -451,7 +609,7 @@ prompt_option_list(
 	while (option_table[option].opt_list[opt_count] != NULL)
 		++opt_count;
 	fmt_option_prompt(prompt, sizeof(prompt), TRUE, option);
-	*variable = prompt_list(option_row(option), 0, *variable, option_table[option].txt->help, prompt, option_table[option].opt_list, opt_count);
+	*variable = prompt_list(option_row(option), 0, variable, option_table[option].txt->help, prompt, option_table[option].opt_list, opt_count);
 	return *variable != old_value;
 }
 
@@ -466,19 +624,35 @@ prompt_option_list(
  */
 t_bool
 prompt_option_string(
-	enum option_enum option) /* return value is always ignored */
+	enum option_enum option)
 {
-	char *variable = OPT_STRING_list[option_table[option].var_index];
+	char *variable;
+	char *old_value;
 	char prompt[LEN];
-	char old_value[LEN];
+	t_bool is_same;
 
-	STRCPY(old_value, variable);
+	if (*OPT_STRING_list[option_table[option].var_index])
+		variable = my_strdup(*OPT_STRING_list[option_table[option].var_index]);
+	else
+		variable = my_strdup("");
+	old_value = my_strdup(variable);
 	show_menu_help(option_table[option].txt->help);
 	fmt_option_prompt(prompt, sizeof(prompt) - 1, TRUE, option);
-	if (prompt_menu_string(option_row(option), prompt, variable))
-		return strcmp(old_value, variable) ? TRUE : FALSE;
-	else
+	if (prompt_menu_string(option_row(option), prompt, &variable)) {
+		if (!option_is_default(option))
+			FreeIfNeeded(*OPT_STRING_list[option_table[option].var_index]);
+		*OPT_STRING_list[option_table[option].var_index] = my_strdup(variable);
+		is_same = strcmp(old_value, variable) ? TRUE : FALSE;
+		free(old_value);
+		free(variable);
+		cursoroff();
+		return is_same;
+	} else {
+		free(old_value);
+		free(variable);
+		cursoroff();
 		return FALSE;
+	}
 }
 
 
@@ -504,13 +678,20 @@ prompt_option_num(
 	fmt_option_prompt(prompt, sizeof(prompt) - 1, TRUE, option);
 	snprintf(&number[0], sizeof(number), "%d", *(option_table[option].variable));
 
-	if ((p = tin_getline(prompt, 2, number, 0, FALSE, HIST_OTHER)) == NULL)
+	if ((p = tin_getline(prompt, 2, number, 0, FALSE, HIST_OTHER)) == NULL) {
+		cursoroff();
 		return FALSE;
+	}
 
 	STRCPY(number, p);
-	num = atoi(number);
+	num = s2i(number, INT_MIN, INT_MAX);
+	/*
+	 * we don't check errno here as
+	 * - EINVAL shouldn't happen as it is already caught by tin_getline()
+	 * - ERANGE the default behaviour of s2i() is fine here
+	 */
 	*(option_table[option].variable) = num;
-	clear_message();
+	cursoroff();
 	return TRUE;
 }
 
@@ -549,18 +730,18 @@ prompt_option_char(
 	if ((curr_val = wchar_t2char(input))) {
 #endif /* MULTIBYTE_ABLE && !NO_LOCALE */
 
-	do {
-		show_menu_help(option_table[option].txt->help);
-		MoveCursor(option_row(option), 0);
-		fmt_option_prompt(prompt, sizeof(prompt) - 1, TRUE, option);
+		do {
+			show_menu_help(option_table[option].txt->help);
+			MoveCursor(option_row(option), 0);
+			fmt_option_prompt(prompt, sizeof(prompt) - 1, TRUE, option);
 
-		if ((p = tin_getline(prompt, 0, curr_val, max_chars, FALSE, HIST_OTHER)) == NULL) {
-			clear_message();
-			return FALSE;
-		}
-		if (!*p)
-			info_message(_(txt_info_enter_valid_character));
-	} while (!*p);
+			if ((p = tin_getline(prompt, 0, curr_val, max_chars, FALSE, HIST_OTHER)) == NULL) {
+				clear_message();
+				return FALSE;
+			}
+			if (!*p)
+				info_message(_(txt_info_enter_valid_character));
+		} while (!*p);
 
 #if defined(MULTIBYTE_ABLE) && !defined(NO_LOCALE)
 		if ((wp = char2wchar_t(p))) {
@@ -599,10 +780,11 @@ prompt_string_default(
 		return NULL;
 	}
 
-	if (pattern[0] != '\0')			/* got a string - make it the default */
-		my_strncpy(def, pattern, LEN);
-	else {
-		if (def[0] == '\0') {		/* no default - give up */
+	if (*pattern) {			/* got a string - make it the default */
+		FreeIfNeeded(def);
+		def = my_strdup(pattern);
+	} else {
+		if (!def || !*def) {		/* no default - give up */
 			error_message(2, "%s", failtext);
 			return NULL;
 		}
@@ -612,6 +794,36 @@ prompt_string_default(
 }
 
 
+char *
+prompt_string_ptr_default(
+	const char *prompt,
+	char **def,
+	const char *failtext,
+	int history)
+{
+	char pattern[LEN];
+
+	clear_message();
+
+	if (!prompt_string(prompt, pattern, history)) {
+		clear_message();
+		return NULL;
+	}
+
+	if (*pattern) {		/* got a string - make it the default */
+		FreeIfNeeded(*def);
+		*def = my_strdup(pattern);
+	} else {
+		if (!*def || **def == '\0') {		/* no default - give up */
+			error_message(2, "%s", failtext);
+			return NULL;
+		}
+	}
+
+	return *def;					/* use the default */
+}
+
+
 /*
  * Get a message ID for the 'L' command. Add <> if needed
  * If the msgid exists and is reachable, return its index
@@ -653,7 +865,17 @@ prompt_msgid(
 		 * If the article is no longer part of a thread, then there is
 		 * no way to display it
 		 */
-		if (which_thread(msgid->article) == -1) {
+		/*
+		 * TODO: Find a better solution, as we end up in the select
+		 *       level after leaving the thread or page level with
+		 *       show_only_unread_arts = TRUE.
+		 *
+		 *       Without !curr_group->attribute->show_only_unread_arts
+		 *       read articles are not found if
+		 *       show_only_unread_arts = TRUE.
+		 */
+		/* if (which_thread(msgid->article) == -1) { */
+		if (!curr_group->attribute->show_only_unread_arts && curr_group->attribute->thread_articles && which_thread(msgid->article) == -1) {
 			info_message(_(txt_no_last_message));
 			return ART_UNAVAILABLE;
 		}
@@ -678,9 +900,7 @@ sized_message(
 	const char *subject)
 {
 	char *buf;
-	int max_len;
-
-	max_len = cCOLS - strwidth(format) + 2 - 1;	/* The formatting info (%s) wastes 2 chars, but our prompt needs 1 char */
+	int max_len = cCOLS - strwidth(format) + 2 - 1;	/* The formatting info (%s) wastes 2 chars, but our prompt needs 1 char */
 
 	buf = strunc(subject, max_len);
 
@@ -778,7 +998,7 @@ prompt_slk_response(
 				case KEYMAP_PAGE_UP:
 				case KEYMAP_HOME:
 				case KEYMAP_END:
-					ch = '\0';
+					ch = '\0'; /* <- do not remove this! NOLINT(clang-analyzer-deadcode.DeadStores) */
 					func = NOT_ASSIGNED;
 					break;
 
@@ -823,7 +1043,7 @@ prompt_continue(
 	void)
 {
 	int ch;
-	int save_signal_context = signal_context;
+	enum context save_signal_context = signal_context;
 
 	cmd_line = TRUE;
 	info_message(_(txt_return_key));
