@@ -3,11 +3,11 @@
  *  Module    : regex.c
  *  Author    : Jason Faultless <jason@altarstone.com>
  *  Created   : 1997-02-21
- *  Updated   : 2024-02-28
+ *  Updated   : 2025-07-09
  *  Notes     : Regular expression subroutines
  *  Credits   :
  *
- * Copyright (c) 1997-2024 Jason Faultless <jason@altarstone.com>
+ * Copyright (c) 1997-2026 Jason Faultless <jason@altarstone.com>
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -116,9 +116,8 @@ match_regex_ex(
 	struct regex_cache *regex)
 {
 #ifndef HAVE_LIB_PCRE2
-	int error;
+	int error = pcre_exec(regex->re, regex->extra, string, length, offset, options, regex->ovector, regex->ovecalloc);
 
-	error = pcre_exec(regex->re, regex->extra, string, length, offset, options, regex->ovector, regex->ovecalloc);
 	if (error >= 0) {
 		/* error == 0 means 'matched, but not enough space in ovector' */
 		regex->oveccount = error;
@@ -171,12 +170,15 @@ compile_regex(
 	struct regex_cache *cache,
 	REGEX_OPTIONS options)
 {
+	char curr_error[8192];
+	static char last_error[8192] = { '\0' };
+	t_bool different;
 #ifdef HAVE_LIB_PCRE2
 	int regex_errcode;
 	PCRE2_SIZE regex_errpos;
 
 	if (regex_use_utf8())
-		options |= PCRE2_UTF;
+		options |= PCRE2_UTF; /* TODO: add PCRE2_UCP? */
 
 	cache->re = pcre2_compile_8((const PCRE2_UCHAR8 *) regex, PCRE2_ZERO_TERMINATED, options,
 			&regex_errcode, &regex_errpos, NULL);
@@ -184,14 +186,16 @@ compile_regex(
 		PCRE2_UCHAR8 regex_errmsg[256];
 
 		pcre2_get_error_message_8(regex_errcode, regex_errmsg, sizeof(regex_errmsg));
-		error_message(2, _(txt_pcre_error_at), regex_errmsg, regex_errpos, regex);
+		snprintf(curr_error, sizeof(curr_error), _(txt_pcre_error_at), regex_errmsg, regex_errpos, regex);
+		different = (strcmp(last_error, curr_error) != 0);
+		error_message(different ? 2 : 0, "%s", curr_error);
+		if (different)
+			strcpy(last_error, curr_error);
 	} else {
 		cache->match = pcre2_match_data_create_from_pattern_8(cache->re, NULL);
-		if (cache->match == NULL) {
-			/* out of memory ... */
+		if (cache->match == NULL) /* out of memory ... */
 			regex_cache_destroy(cache);
-			regex_cache_init(cache);
-		} else
+		else
 			return TRUE;
 	}
 
@@ -204,20 +208,22 @@ compile_regex(
 	if (regex_use_utf8())
 		options |= PCRE_UTF8;
 
-	if ((cache->re = pcre_compile(regex, options, &regex_errmsg, &regex_errpos, NULL)) == NULL)
-		error_message(2, _(txt_pcre_error_at), regex_errmsg, regex_errpos, regex);
-	else {
+	if ((cache->re = pcre_compile(regex, options, &regex_errmsg, &regex_errpos, NULL)) == NULL) {
+		snprintf(curr_error, sizeof(curr_error), _(txt_pcre_error_at), regex_errmsg, regex_errpos, regex);
+		different = (strcmp(last_error, curr_error) != 0);
+		error_message(different ? 2 : 0, "%s", curr_error);
+		if (different)
+			strcpy(last_error, curr_error);
+	} else {
 		cache->extra = pcre_study(cache->re, 0, &regex_errmsg);
 		if (regex_errmsg != NULL) {
 			/* we failed, clean up */
 			regex_cache_destroy(cache);
-			regex_cache_init(cache);
 			error_message(2, _(txt_pcre_error_text), regex_errmsg);
 		} else {
 			int n;
-			int error;
+			int error = pcre_fullinfo(cache->re, cache->extra, PCRE_INFO_CAPTURECOUNT, &n);
 
-			error = pcre_fullinfo(cache->re, cache->extra, PCRE_INFO_CAPTURECOUNT, &n);
 			if (error != 0)
 				error_message(2, _(txt_pcre_error_num), error);
 			else {
@@ -249,14 +255,15 @@ highlight_regexes(
 	int color)
 {
 	char *ptr;
-#ifdef USE_CURSES
-	char buf[LEN];
-#else
 	char *buf;
-#endif /* USE_CURSES */
 
 	/* Get contents of line from the screen */
 #ifdef USE_CURSES
+#	if defined(MULTIBYTE_ABLE) && !defined(NO_LOCALE)
+	buf = my_malloc(MB_CUR_MAX * (size_t) (cCOLS + 1));
+#	else
+	buf = my_malloc(cCOLS + 1);
+#	endif /* MULTIBYTE_ABLE && !NO_LOCALE */
 	screen_contents(row, 0, buf);
 #else
 	buf = screen[row].col;
@@ -268,10 +275,10 @@ highlight_regexes(
 		REGEX_SIZE *offsets = regex_get_ovector_pointer(regex);
 		/* we have a match */
 		if (color >= 0) /* color the matching text */
-			word_highlight_string(row, (int) ((ptr - buf) + offsets[0]), offsets[1] - offsets[0], color);
+			word_highlight_string(row, (int) ((ptr - buf) + offsets[0]), (int) (offsets[1] - offsets[0]), color);
 		else
 			/* inverse the matching text */
-			highlight_string(row, (int) ((ptr - buf) + offsets[0]), offsets[1] - offsets[0]);
+			highlight_string(row, (int) ((ptr - buf) + offsets[0]), (int) (offsets[1] - offsets[0]));
 
 		if (!tinrc.word_h_display_marks) {
 #ifdef USE_CURSES
@@ -281,6 +288,9 @@ highlight_regexes(
 		} else
 			ptr += offsets[1];
 	}
+#ifdef USE_CURSES
+	free(buf);
+#endif /* USE_CURSES */
 }
 
 
@@ -288,11 +298,10 @@ void
 regex_cache_init(
 	struct regex_cache *regex)
 {
-#ifdef HAVE_LIB_PCRE2
 	regex->re = NULL;
+#ifdef HAVE_LIB_PCRE2
 	regex->match = NULL;
 #else
-	regex->re = NULL;
 	regex->extra = NULL;
 	regex->ovector = NULL;
 	regex->ovecalloc = 0;
@@ -322,6 +331,52 @@ regex_cache_destroy(
 }
 
 
+/*
+ * returns freshly allocated mem which holds the matched portion
+ * of the named subpattern on success and NULL on error
+ */
+char *
+regex_get_substring_by_name(
+	struct regex_cache *re,		/* precompiled regex */
+	const char *sname,			/* name of subpattern */
+	char *subject)				/* data to match against */
+{
+	char *ms;
+	int snum;
+
+#ifdef HAVE_LIB_PCRE2
+	PCRE2_UCHAR8 *buf = NULL;
+	PCRE2_SIZE buflen = 0;
+
+	(void) subject;
+	if ((snum = pcre2_substring_number_from_name_8(re->re, (PCRE2_SPTR8) sname)) < 1)
+		return NULL;
+
+	if (pcre2_substring_get_bynumber_8(re->match, snum, &buf, &buflen) < 0) {
+		pcre2_substring_free_8(buf);
+		return NULL;
+	}
+	ms = my_strdup((const char *) buf);
+	pcre2_substring_free_8(buf);
+	return ms;
+
+#else
+	const char *buf = NULL;
+
+	regex_get_ovector_pointer(re);
+	if ((snum = pcre_get_stringnumber(re->re, sname)) < 1)
+		return NULL;
+
+	if (pcre_get_substring(subject, re->ovector, re->oveccount, snum, &buf) < 1)
+		return NULL;
+
+	ms = my_strdup(buf);
+	pcre_free_substring(buf);
+	return ms;
+#endif /* HAVE_LIB_PCRE2 */
+}
+
+
 t_bool
 regex_use_utf8(
 	void)
