@@ -3,10 +3,10 @@
  *  Module    : rfc1524.c
  *  Author    : Urs Janssen <urs@tin.org>, Jason Faultless <jason@altarstone.com>
  *  Created   : 2000-05-15
- *  Updated   : 2024-05-10
+ *  Updated   : 2025-01-30
  *  Notes     : mailcap parsing as defined in RFC 1524
  *
- * Copyright (c) 2000-2024 Urs Janssen <urs@tin.org>, Jason Faultless <jason@altarstone.com>
+ * Copyright (c) 2000-2025 Urs Janssen <urs@tin.org>, Jason Faultless <jason@altarstone.com>
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -51,9 +51,9 @@
 #define MAILCAPFIELDS 13
 
 /* local prototypes */
-static char *expand_mailcap_meta(const char *mailcap, t_part *part, t_bool escape_shell_meta_chars, const char *path);
+static char *expand_mailcap_meta(const char *mailcap, const t_part *part, t_bool escape_shell_meta_chars, const char *path);
 static char *get_mailcap_field(char *mailcap);
-static t_mailcap *parse_mailcap_line(const char *mailcap, t_part *part, const char *path);
+static t_mailcap *parse_mailcap_line(const char *mailcap, const t_part *part, const char *path);
 
 
 /*
@@ -65,20 +65,20 @@ static t_mailcap *parse_mailcap_line(const char *mailcap, t_part *part, const ch
  */
 t_mailcap *
 get_mailcap_entry(
-	t_part *part,
+	const t_part *part,
 	const char *path)
 {
 	FILE *fp;
 	char *ptr, *ptr2, *nptr;
+	char *mailcaps;		/* possible mailcap files */
 	char buf[LEN];
 	char filename[LEN];	/* name of current mailcap file */
 	char mailcap[LEN];	/* full match */
-	char *mailcaps;		/* possible mailcap files */
 	char wildcap[LEN];	/* basetype match */
 	t_mailcap *foo = (t_mailcap *) 0;
 
 	/* build list of mailcap files */
-	if ((ptr = getenv("MAILCAPS")) != NULL && strlen(ptr)) {
+	if ((ptr = getenv("MAILCAPS")) != NULL && *ptr) {
 		mailcaps = my_malloc(strlen(ptr) + strlen(DEFAULT_MAILCAPS) + 2);
 		sprintf(mailcaps, "%s:%s", ptr, DEFAULT_MAILCAPS);
 	} else
@@ -93,7 +93,7 @@ get_mailcap_entry(
 	nptr = strtok(mailcaps, ":");
 	while (nptr != NULL) {
 		/* expand ~ and/or $HOME etc. */
-		if (strfpath(nptr, filename, sizeof(filename) - 1, &CURR_GROUP, FALSE)) {
+		if (strfpath(nptr, filename, sizeof(filename) - 1, NULL, FALSE)) {
 			if ((fp = tin_fopen(filename, "r")) != NULL) {
 				while ((fgets(ptr, (int) (sizeof(buf) - strlen(buf)), fp)) != NULL) {
 					if (*ptr == '#' || *ptr == '\n')		/* skip comments & blank lines */
@@ -123,7 +123,7 @@ get_mailcap_entry(
 								}
 							} else {
 								if ((*(ptr2 + 1) == '*') || (*(ptr2 + 1) == ';')) { /* wildmat match */
-									if (!strlen(wildcap)) { /* we don't already have a wildmat match */
+									if (!*wildcap) { /* we don't already have a wildmat match */
 										STRCPY(wildcap, buf);
 										FreeIfNeeded(foo);
 										foo = parse_mailcap_line(wildcap, part, path);
@@ -157,7 +157,7 @@ get_mailcap_entry(
 static t_mailcap*
 parse_mailcap_line(
 	const char *mailcap,
-	t_part *part,
+	const t_part *part,
 	const char *path)
 {
 	char *ptr, *optr, *buf;
@@ -213,6 +213,7 @@ parse_mailcap_line(
 			ptr += strlen(ptr) + 1;
 		}
 		if (!strncasecmp(ptr, "nametemplate=", 13)) {
+			FreeIfNeeded(tmailcap->nametemplate);
 			tmailcap->nametemplate = expand_mailcap_meta(ptr + 13, part, FALSE, path);
 			ptr += strlen(ptr) + 1;
 		}
@@ -324,7 +325,7 @@ get_mailcap_field(
 				backquote = FALSE;
 				break;
 		}
-		ptr++;
+		++ptr;
 	}
 	return mailcap;
 }
@@ -349,13 +350,13 @@ get_mailcap_field(
 static char *
 expand_mailcap_meta(
 	const char *mailcap,
-	t_part *part,
+	const t_part *part,
 	t_bool escape_shell_meta_chars,
 	const char *path)
 {
 	const char *ptr;
 	char *line, *lptr;
-	int quote = no_quote;
+	enum quote_enum quote = no_quote;
 	size_t linelen, space, olen;
 
 	if (!(strchr(mailcap, '%'))) /* nothing to expand */
@@ -382,15 +383,15 @@ expand_mailcap_meta(
 		}
 
 		if (*ptr == '\\') {
-			ptr++;
+			++ptr;
 			if ((*ptr == '\\') || (*ptr == '%')) {
 				*lptr++ = *ptr++;
-				space--;
+				--space;
 			}
 			continue;
 		}
 		if (*ptr == '%') {
-			ptr++;
+			++ptr;
 			if (*ptr == '{') {	/* Content-Type parameter */
 				char *end;
 
@@ -412,7 +413,7 @@ expand_mailcap_meta(
 						free(parameter);
 					}
 					ptr = end;	/* skip past closing } */
-					ptr++;
+					++ptr;
 				} else {
 					/* sequence broken, output literally */
 					*lptr++ = '%';
@@ -432,7 +433,7 @@ expand_mailcap_meta(
 				strcat(line, nptr);
 				lptr = line + strlen(line);
 				space -= strlen(line);
-				ptr++;
+				++ptr;
 				continue;
 			} else if (*ptr == 't') {	/* Content-Type */
 				const char *nptr = escape_shell_meta_chars ? escape_shell_meta(part->subtype, quote) : part->subtype;
@@ -443,11 +444,11 @@ expand_mailcap_meta(
 				strcat(line, nptr);
 				lptr = line + strlen(line);
 				space -= strlen(line);
-				ptr++;
+				++ptr;
 				continue;
 			} else {	/* unknown % sequence */
 				*lptr++ = '%';
-				space--;
+				--space;
 				continue;
 			}
 		}
@@ -461,7 +462,7 @@ expand_mailcap_meta(
 
 		/* any other char */
 		*lptr++ = *ptr++;
-		space--;
+		--space;
 	}
 	return line;
 }
