@@ -3,10 +3,10 @@
  *  Module    : screen.c
  *  Author    : I. Lea & R. Skrenta
  *  Created   : 1991-04-01
- *  Updated   : 2024-10-19
+ *  Updated   : 2025-02-13
  *  Notes     :
  *
- * Copyright (c) 1991-2024 Iain Lea <iain@bricbrac.de>, Rich Skrenta <skrenta@pbm.com>
+ * Copyright (c) 1991-2025 Iain Lea <iain@bricbrac.de>, Rich Skrenta <skrenta@pbm.com>
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -56,7 +56,7 @@ static FILE* msglog = NULL;
 	struct t_screen *screen;
 #endif /* !USE_CURSES */
 
-static void log_formatted_msg(const char* tag, const char *msg);
+static void log_formatted_msg(const char *tag, const char *msg);
 
 /*
  * Move the cursor to the lower-left of the screen, where it won't be annoying
@@ -203,7 +203,12 @@ wait_message(
 #	endif /* HAVE_SELECT_INTP */
 			if (nfds == -1) {
 
-				if (errno != EINTR) {
+#	ifdef EINTR
+				if (errno != EINTR)
+#	else
+				if (errno != 0)
+#	endif /* EINTR */
+				{
 					perror_message("wait_message(select()) failed");
 					free(tin_progname);
 					giveup();
@@ -341,7 +346,7 @@ center_line(
 		}
 	}
 
-    if (len >= cCOLS) {
+	if (len >= cCOLS) {
 		char *buffer = strunc(ln, cCOLS - 2);
 
 		if (buffer != ln)
@@ -452,8 +457,6 @@ erase_arrow(
 #	endif /* MULTIBYTE_ABLE && !NO_LOCALE */
 		s = screen_contents(line, 0, buffer);
 #else
-		s = screen[line - INDEX_TOP].col;
-
 		if (line - INDEX_TOP < 0) /* avoid underruns */
 			line = INDEX_TOP;
 
@@ -561,13 +564,12 @@ ring_bell(
 #ifdef USE_CURSES
 	if (!cmd_line)
 		beep();
-	else {
+	else
 #endif /* USE_CURSES */
-	my_fputc('\007', stdout);
-	my_flush();
-#ifdef USE_CURSES
+	{
+		my_fputc('\007', stdout);
+		my_flush();
 	}
-#endif /* USE_CURSES */
 }
 
 
@@ -669,7 +671,7 @@ show_progress(
 		count_diff = (count - last_count);
 
 		if (!count_diff) /* avoid div by zero */
-			count_diff++;
+			++count_diff;
 
 		/*
 		 * Calculate a running average based on the last 20 samples. For the
