@@ -3,10 +3,10 @@
  *  Module    : search.c
  *  Author    : I. Lea & R. Skrenta
  *  Created   : 1991-04-01
- *  Updated   : 2023-11-27
+ *  Updated   : 2025-05-27
  *  Notes     :
  *
- * Copyright (c) 1991-2024 Iain Lea <iain@bricbrac.de>, Rich Skrenta <skrenta@pbm.com>
+ * Copyright (c) 1991-2025 Iain Lea <iain@bricbrac.de>, Rich Skrenta <skrenta@pbm.com>
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -46,7 +46,7 @@
 /*
  * local prototypes
  */
-static char *get_search_pattern(t_bool *forward, t_bool repeat, const char *fwd_msg, const char *bwd_msg, char *def, int which_hist);
+static char *get_search_pattern(t_bool *forward, t_bool repeat, const char *fwd_msg, const char *bwd_msg, char **def, int which_hist);
 static int author_search(int i, char *searchbuf);
 static int body_search(int i, char *searchbuf);
 static int subject_search(int i, char *searchbuf);
@@ -99,7 +99,7 @@ get_search_pattern(
 	t_bool repeat,
 	const char *fwd_msg,
 	const char *bwd_msg,
-	char *def,
+	char **def,
 	int which_hist)
 {
 	static char tmpbuf[LEN];	/* Hold the last pattern used */
@@ -108,15 +108,16 @@ get_search_pattern(
 
 	if (repeat) {
 		*forward = last_forward;
-		my_strncpy(def, last_pattern, LEN);
+		FreeIfNeeded(*def);
+		*def = my_strdup(last_pattern);
 	} else {
-		snprintf(tmpbuf, sizeof(tmpbuf), (*forward ? fwd_msg : bwd_msg), def);
+		snprintf(tmpbuf, sizeof(tmpbuf), (*forward ? fwd_msg : bwd_msg), BlankIfNull(*def));
 
-		if (!prompt_string_default(tmpbuf, def, _(txt_no_search_string), which_hist))
+		if (!prompt_string_ptr_default(tmpbuf, def, _(txt_no_search_string), which_hist))
 			return NULL;
 
 		last_forward = *forward;
-		my_strncpy(last_pattern, def, LEN);
+		my_strncpy(last_pattern, *def, LEN);
 
 		/* HIST_BODY_SEARCH doesn't exist, hence last_search is set directly in search_body() */
 		if (which_hist == HIST_AUTHOR_SEARCH)
@@ -134,22 +135,27 @@ get_search_pattern(
 	if (IS_LOCAL_CHARSET("UTF-8")) {
 		char *tmp;
 
-		tmp = normalize(def);
-		my_strncpy(def, tmp, LEN);
+		tmp = normalize(*def);
+		FreeIfNeeded(*def);
+		*def = my_strdup(tmp);
 		free(tmp);
 	}
 #endif /* HAVE_UNICODE_NORMALIZATION */
 
 	if (tinrc.wildcard) {			/* ie, not wildmat() */
-		strcpy(def, quote_wild_whitespace(def));
-		return def;
+		char tmp[LEN];
+
+		STRCPY(tmp, *def);
+		FreeIfNeeded(*def);
+		*def = my_strdup(quote_wild_whitespace(tmp));
+		return *def;
 	}
 
 	/*
 	 * A gross hack to simulate substrings with wildmat()
 	 */
 /* TODO: somehow use REGEX_FMT here? */
-	snprintf(tmpbuf, sizeof(tmpbuf), "*%s*", def);
+	snprintf(tmpbuf, sizeof(tmpbuf), "*%s*", *def);
 	return tmpbuf;
 }
 
@@ -168,21 +174,25 @@ search_config(
 	enum option_enum n = current;
 	enum option_enum result = current;
 
-	if (!(pattern = get_search_pattern(&forward, repeat, _(txt_search_forwards), _(txt_search_backwards), tinrc.default_search_config, HIST_CONFIG_SEARCH)))
+	if (!(pattern = get_search_pattern(&forward, repeat, _(txt_search_forwards), _(txt_search_backwards), &tinrc.default_search_config, HIST_CONFIG_SEARCH)))
 		return result;
 
 	if (tinrc.wildcard && !(compile_regex(pattern, &search_regex, REGEX_CASELESS)))
 		return result;
 
 	do {
-		if (n == 0 && !forward)
-			n = last;
-		else {
-			if (n == last && forward)
-				n = 0;
+		if (forward) {
+			if (n == last)
+				n = (enum option_enum) 0;
 			else
-				n = (n + (enum option_enum) (forward ? 1 : -1));
+				++n;
+		} else {
+			if (n > 0)
+				--n;
+			else
+				n = last;
 		}
+
 		/* search only visible options */
 		if (option_is_visible(n)) {
 #ifdef HAVE_UNICODE_NORMALIZATION
@@ -230,21 +240,25 @@ generic_search(
 	t_url *urlptr;
 	t_posted *phptr;
 
-	if (!(pattern = get_search_pattern(&forward, repeat, _(txt_search_forwards), _(txt_search_backwards), tinrc.default_search_config, HIST_CONFIG_SEARCH)))
+	if (!(pattern = get_search_pattern(&forward, repeat, _(txt_search_forwards), _(txt_search_backwards), &tinrc.default_search_config, HIST_CONFIG_SEARCH)))
 		return result;
 
 	if (tinrc.wildcard && !(compile_regex(pattern, &search_regex, REGEX_CASELESS)))
 		return result;
 
 	do {
-		if (n == 0 && !forward)
-			n = last;
-		else {
-			if (n == last && forward)
+		if (forward) {
+			if (n == last)
 				n = 0;
 			else
-				n += (forward ? 1 : -1);
+				++n;
+		} else {
+			if (n > 0)
+				--n;
+			else
+				n = last;
 		}
+
 		switch (level) {
 			case ATTACHMENT_LEVEL:
 				part = get_part(n);
@@ -258,7 +272,7 @@ generic_search(
 
 			case POSTED_LEVEL:
 				phptr = find_post_hist(n);
-				snprintf(buf, sizeof(buf), "%s %s %s", phptr->date, phptr->group, phptr->subj);
+				snprintf(buf, sizeof(buf), "%s %s %s", phptr->date, BlankIfNull(phptr->group), BlankIfNull(phptr->subj));
 				break;
 
 			case URL_LEVEL:
@@ -308,7 +322,7 @@ search_active(
 		return -1;
 	}
 
-	if (!(buf = get_search_pattern(&forward, repeat, _(txt_search_forwards), _(txt_search_backwards), tinrc.default_search_group, HIST_GROUP_SEARCH)))
+	if (!(buf = get_search_pattern(&forward, repeat, _(txt_search_forwards), _(txt_search_backwards), &tinrc.default_search_group, HIST_GROUP_SEARCH)))
 		return -1;
 
 	if (tinrc.wildcard && !(compile_regex(buf, &search_regex, REGEX_CASELESS)))
@@ -375,10 +389,19 @@ body_search(
 			art_close(&artinfo);
 			info_message(_(txt_no_match));
 			return 0;
+
+		default:
+			break;
 	}
 
 	/*
 	 * Skip the header - is this right ?
+	 * then we currently don't have a way to search in the headers
+	 * (except author/subject search). if we add some kind of full
+	 * article search we need to decide if it should act on the raw
+	 * headers or even article (so one could search for
+	 * "\b\?iso-8859-\d\d?\?" or the like or on the decoded headers
+	 * (you don't search for base64 raw-data, do you?)
 	 */
 	for (i = 0; artinfo.cookl[i].flags & C_HEADER; ++i)
 		;
@@ -401,7 +424,7 @@ body_search(
 			line = my_strdup(tmp);
 
 		if (tinrc.wildcard) {
-			if (match_regex_ex(line, (int) strlen(line), 0, 0, &search_regex) >= 0) {
+			if (MATCH_REGEX(search_regex, line, strlen(line))) {
 				copy_offsets(srch_offsets, srch_offsets_size, &search_regex);
 				srch_lineno = i;
 				art_close(&pgart);		/* Switch the pager over to matched art */
@@ -423,7 +446,7 @@ body_search(
 				return 1;
 			}
 		}
-		i++;
+		++i;
 		free(line);
 	}
 
@@ -448,15 +471,14 @@ author_search(
 	char *searchbuf)
 {
 	char *buf, *tmp;
+	int len;
 
-	if (arts[i].name == NULL)
-		tmp = my_strdup(arts[i].from);
+	if (arts[i].mailbox.name == NULL)
+		tmp = my_strdup(arts[i].mailbox.from);
 	else {
-		int len;
-
-		len = snprintf(NULL, 0, "%s <%s>", arts[i].name, arts[i].from);
+		len = snprintf(NULL, 0, "%s <%s>", arts[i].mailbox.name, arts[i].mailbox.from);
 		tmp = my_malloc(++len);
-		snprintf(tmp, len, "%s <%s>", arts[i].name, arts[i].from);
+		snprintf(tmp, len, "%s <%s>", arts[i].mailbox.name, arts[i].mailbox.from);
 	}
 
 #ifdef HAVE_UNICODE_NORMALIZATION
@@ -545,10 +567,9 @@ search_group(
 			continue;
 
 		ret = search_func(i, searchbuff);
-		if (tinrc.wildcard && (ret == 1 || ret == -1)) {
-			/* we will exit soon, clean up */
+		if (tinrc.wildcard && (ret == 1 || ret == -1)) /* we will exit soon, clean up */
 			regex_cache_destroy(&search_regex);
-		}
+
 		switch (ret) {
 			case 1:								/* Found */
 				clear_message();
@@ -557,6 +578,7 @@ search_group(
 			case -1:							/* User abort */
 				return -1;
 		}
+
 #ifdef HAVE_SELECT
 		if (wait_for_input())	/* allow abort */
 			return -1;
@@ -584,7 +606,7 @@ search(
 	t_bool repeat)
 {
 	char *buf;
-	int (*search_func) (int i, char *searchbuff) = author_search;
+	int (*search_func) (int i, char *searchbuff);
 	t_bool forward;
 
 	if (func == GLOBAL_SEARCH_SUBJECT_FORWARD || func == GLOBAL_SEARCH_AUTHOR_FORWARD)
@@ -595,7 +617,7 @@ search(
 	switch (func) {
 		case GLOBAL_SEARCH_SUBJECT_FORWARD:
 		case GLOBAL_SEARCH_SUBJECT_BACKWARD:
-			if (!(buf = get_search_pattern(&forward, repeat, _(txt_search_forwards), _(txt_search_backwards), tinrc.default_search_subject, HIST_SUBJECT_SEARCH)))
+			if (!(buf = get_search_pattern(&forward, repeat, _(txt_search_forwards), _(txt_search_backwards), &tinrc.default_search_subject, HIST_SUBJECT_SEARCH)))
 				return -1;
 			search_func = subject_search;
 			break;
@@ -603,7 +625,7 @@ search(
 		case GLOBAL_SEARCH_AUTHOR_FORWARD:
 		case GLOBAL_SEARCH_AUTHOR_BACKWARD:
 		default:
-			if (!(buf = get_search_pattern(&forward, repeat, _(txt_author_search_forwards), _(txt_author_search_backwards), tinrc.default_search_author, HIST_AUTHOR_SEARCH)))
+			if (!(buf = get_search_pattern(&forward, repeat, _(txt_author_search_forwards), _(txt_author_search_backwards), &tinrc.default_search_author, HIST_AUTHOR_SEARCH)))
 				return -1;
 			search_func = author_search;
 			break;
@@ -623,7 +645,7 @@ search_article(
 	t_bool repeat,
 	int start_line,
 	int lines,
-	t_lineinfo *line,
+	const t_lineinfo *line,
 	int reveal_ctrl_l_lines,
 	FILE *fp)
 {
@@ -633,7 +655,7 @@ search_article(
 	t_bool wrap = FALSE;
 	t_bool match = FALSE;
 
-	if (!(pattern = get_search_pattern(&forward, repeat, _(txt_search_forwards), _(txt_search_backwards), tinrc.default_search_art, HIST_ART_SEARCH)))
+	if (!(pattern = get_search_pattern(&forward, repeat, _(txt_search_forwards), _(txt_search_backwards), &tinrc.default_search_art, HIST_ART_SEARCH)))
 		return 0;
 
 	if (tinrc.wildcard && !(compile_regex(pattern, &search_regex, REGEX_CASELESS)))
@@ -673,7 +695,7 @@ search_article(
 			ptr = my_strdup(tmp);
 
 		if (tinrc.wildcard) {
-			while (match_regex_ex(ptr, (int) strlen(ptr), srch_offsets[1], REGEX_NOTEMPTY, &search_regex) >= 0) {
+			while (match_regex_ex(ptr, (REGEX_SIZE) strlen(ptr), srch_offsets[1], REGEX_NOTEMPTY, &search_regex) >= 0) {
 				copy_offsets(srch_offsets, srch_offsets_size, &search_regex);
 				match = TRUE;
 				if (forward)
@@ -709,13 +731,13 @@ search_article(
 				i = 0;
 				wrap = TRUE;
 			} else
-				i++;
+				++i;
 		} else {
 			if (i <= 0) {
 				i = lines - 1;
 				wrap = TRUE;
 			} else
-				i--;
+				--i;
 		}
 
 		/* search at the beginning of the line */
@@ -740,7 +762,7 @@ search_article(
  */
 int
 search_body(
-	struct t_group *group,
+	const struct t_group *group,
 	int current_art,
 	t_bool repeat)
 {
@@ -753,7 +775,7 @@ search_body(
 			repeat,
 			_(txt_search_body),
 			_(txt_search_body),
-			tinrc.default_search_art,
+			&tinrc.default_search_art,
 			HIST_ART_SEARCH
 	))) return -1;
 
@@ -769,7 +791,7 @@ search_body(
 	} else {
 		for_each_art(i) {
 			if (!IGNORE_ART(i))
-				total_cnt++;
+				++total_cnt;
 		}
 	}
 
@@ -784,14 +806,15 @@ search_body(
  */
 int
 get_search_vectors(
-	int *start,
-	int *end)
+	REGEX_SIZE *start,
+	REGEX_SIZE *end)
 {
 	int i = srch_lineno;
 
 	*start = srch_offsets[0];
 	*end = srch_offsets[1];
 	srch_lineno = -1;			/* We can only retrieve this info once */
+
 	return i;
 }
 
