@@ -3,10 +3,10 @@
  *  Module    : signal.c
  *  Author    : I.Lea
  *  Created   : 1991-04-01
- *  Updated   : 2023-11-12
+ *  Updated   : 2025-12-16
  *  Notes     : signal handlers for different modes and window resizing
  *
- * Copyright (c) 1991-2024 Iain Lea <iain@bricbrac.de>
+ * Copyright (c) 1991-2026 Iain Lea <iain@bricbrac.de>
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -104,8 +104,9 @@ static void _CDECL signal_handler(SIG_ARGS);
 	static t_bool redraw_after_suspend;
 #endif /* SIGWINCH || SIGTSTP */
 
-int signal_context = cMain;
+enum context signal_context = cMain;
 int input_context = cNone;
+int need_parse_fmt = 0;
 int need_resize = cNo;
 /*
  * # lines of non-static data available for display
@@ -230,7 +231,7 @@ signal_name(
 	int code)
 {
 	size_t n;
-	const char *name = "unknown";
+	const char *name = txt_unknown;
 
 	for (n = 0; n < ARRAY_SIZE(signal_list); n++) {
 		if (signal_list[n].code == code) {
@@ -269,12 +270,12 @@ handle_resize(
 #	ifdef USE_CURSES
 #		ifdef HAVE_RESIZETERM
 	resizeterm(cLINES + 1, cCOLS);
-	my_retouch();					/* seems necessary if win size unchanged */
-#		else
-	my_retouch();
 #		endif /* HAVE_RESIZETERM */
+	my_retouch();
 #	endif /* USE_CURSES */
 
+	need_parse_fmt |= (SELECT_LEVEL|GROUP_LEVEL|THREAD_LEVEL);
+
 	switch (signal_context) {
 		case cArt:
 			ClearScreen();
@@ -306,7 +307,7 @@ handle_resize(
 			break;
 
 		case cPage:
-			resize_article(TRUE, &pgart);
+			resize_article(TRUE, &pgart); /* loses current state raw/cooked */
 			draw_page(0);
 			break;
 
@@ -334,9 +335,14 @@ handle_resize(
 		case cMain:
 			break;
 	}
+
 	switch (input_context) {
 		case cGetline:
-			gl_redraw();
+			/* leave input field to avoid confusion after redrawing the screen */
+			if (signal_context == cAttrib || signal_context == cConfig)
+				pending_sactions = 1;
+			else
+				gl_redraw();
 			break;
 
 		case cPromptCONT:
@@ -344,6 +350,11 @@ handle_resize(
 				info_message(_(txt_return_key));
 			break;
 
+		case cPromptList:
+			/* leave input field to avoid confusion after redrawing the screen */
+			pending_sactions = 1;
+			break;
+
 		case cPromptSLK:
 			prompt_slk_redraw();
 			break;
@@ -355,6 +366,7 @@ handle_resize(
 		default:
 			break;
 	}
+
 	my_fflush(stdout);
 	redraw_after_suspend = FALSE;
 #endif /* SIGWINCH || SIGTSTP */
@@ -380,7 +392,7 @@ handle_suspend(
 		Raw(FALSE);
 	}
 
-	wait_message(0, _(txt_suspended_message), tin_progname);
+	my_fprintf(stdout, _(txt_suspended_message), tin_progname); /* stderr? */
 
 	kill(0, SIGSTOP);				/* Put ourselves to sleep */
 
@@ -406,6 +418,7 @@ static void _CDECL
 signal_handler(
 	int sig)
 {
+	int serrno = errno;
 #ifdef SIGCHLD
 #	ifdef HAVE_TYPE_UNIONWAIT
 	union wait wait_status;
@@ -419,6 +432,7 @@ signal_handler(
 #ifdef SIGINT
 		case SIGINT:
 			RESTORE_HANDLER(sig, signal_handler);
+			errno = serrno;
 			return;
 #endif /* SIGINT */
 
@@ -429,37 +443,23 @@ signal_handler(
 #if defined(HAVE_ALARM) && defined(SIGALRM)
 		case SIGALRM:
 #	ifdef NNTP_ABLE
-#	ifdef DEBUG
-			if ((debug & DEBUG_NNTP) && verbose > 1)
-				debug_print_file("NNTP", "get_server() %d sec elapsed without response", TIN_NNTP_TIMEOUT);
-#	endif /* DEBUG */
-
-#	ifdef USE_ZLIB
+#		ifdef USE_ZLIB
 			/*
 			 * response compression from the server may take a while
 			 * when running interactively and not being in connection
 			 * phase give the user a chance to go on instead of exiting
 			 */
-			{
-				char *prompt;
-				size_t len;
-
-				len = strlen(_(txt_read_timeout_quit)) + snprintf(NULL, 0, "%d", tinrc.nntp_read_timeout_secs) - 1;
-				prompt = my_malloc(len);
-				snprintf(prompt, len, _(txt_read_timeout_quit), tinrc.nntp_read_timeout_secs);
-				if (signal_context == cReconnect || batch_mode || !use_compress || !nntp_caps.compress || prompt_yn(prompt, FALSE) == 1) {
-					free(prompt);
-#	endif /* USE_ZLIB */
-					tin_done(NNTP_ERROR_EXIT, _(txt_connection_error));
-#	ifdef USE_ZLIB
-				} else {
-					free(prompt);
-					RESTORE_HANDLER(sig, signal_handler);
-					wait_message(0, _(txt_continuing));
-				}
+			if (signal_context == cReconnect || batch_mode || !use_compress || !nntp_caps.compress || prompt_yn(_(txt_read_timeout_quit), FALSE) == 1)
+				tin_done(NNTP_ERROR_EXIT, _(txt_connection_error_timeout), TIN_NNTP_TIMEOUT); /* NOTE: a lot of signal-unsafe calls in tin_done() */
+			else {
+				RESTORE_HANDLER(sig, signal_handler);
+				my_fprintf(stdout, "%s", _(txt_continuing));
 			}
-#	endif /* USE_ZLIB */
+#		else
+			tin_done(NNTP_ERROR_EXIT, _(txt_connection_error_timeout), TIN_NNTP_TIMEOUT); /* NOTE: a lot of signal-unsafe calls in tin_done() */
+#		endif /* USE_ZLIB */
 #	endif /* NNTP_ABLE */
+			errno = serrno;
 			return;
 #endif /* HAVE_ALARM && SIGALRM */
 
@@ -468,12 +468,14 @@ signal_handler(
 			wait(&wait_status);
 			RESTORE_HANDLER(sig, signal_handler);	/* death of a child */
 			system_status = WIFEXITED(wait_status) ? WEXITSTATUS(wait_status) : 0;
+			errno = serrno;
 			return;
 #endif /* SIGCHLD */
 
 #ifdef SIGTSTP
 		case SIGTSTP:
 			handle_suspend();
+			errno = serrno;
 			return;
 #endif /* SIGTSTP */
 
@@ -481,14 +483,16 @@ signal_handler(
 		case SIGWINCH:
 			need_resize = cYes;
 			RESTORE_HANDLER(sig, signal_handler);
+			errno = serrno;
 			return;
 #endif /* SIGWINCH */
 
 #ifdef SIGUSR2
 		case SIGUSR2:
 			if (!no_write) /* TODO: add more config-files to be saved */
-				write_newsrc();
+				write_newsrc(); /* TODO: uses signal-unsafe calls */
 			RESTORE_HANDLER(sig, signal_handler);
+			errno = serrno;
 			return;
 #endif /* SIGUSR2 */
 
@@ -496,7 +500,7 @@ signal_handler(
 			break;
 	}
 
-	fprintf(stderr, "\n%s: signal handler caught %s signal (%d).\n", tin_progname, signal_name(sig), sig);
+	my_fprintf(stderr, "\n%s: signal handler caught %s signal (%d).\n", tin_progname, signal_name(sig), sig);
 
 	switch (sig) {
 #ifdef SIGHUP
@@ -510,7 +514,7 @@ signal_handler(
 #endif /* SIGTERM */
 #if defined(SIGHUP) || defined(SIGUSR1) || defined(SIGTERM)
 			dangerous_signal_exit = TRUE;
-			tin_done(-sig, NULL);
+			tin_done(-sig, NULL); /* NOTE: a lot of signal-unsafe calls in tin_done() */
 			/* NOTREACHED */
 			break;
 #endif /* SIGHUP || SIGUSR1 || SIGTERM */
@@ -525,7 +529,6 @@ signal_handler(
 		case SIGBUS:
 #	endif /* SIGBUS */
 #endif /* SIGSEGV */
-
 #if defined(SIGBUS) || defined(SIGSEGV)
 			my_fprintf(stderr, _(txt_send_bugreport), tin_progname, VERSION, RELEASEDATE, RELEASENAME, bug_addr);
 			my_fflush(stderr);
