@@ -3,10 +3,10 @@
  *  Module    : string.c
  *  Author    : Urs Janssen <urs@tin.org>
  *  Created   : 1997-01-20
- *  Updated   : 2024-10-28
+ *  Updated   : 2025-02-27
  *  Notes     :
  *
- * Copyright (c) 1997-2024 Urs Janssen <urs@tin.org>
+ * Copyright (c) 1997-2025 Urs Janssen <urs@tin.org>
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -83,7 +83,7 @@ tin_ltoa(
 		digits = sizeof(buffer) - 1;
 
 	if (!digits)
-		digits++;
+		++digits;
 
 	snprintf(buffer, sizeof(buffer), "%"T_ARTNUM_PFMT, value);
 	len = strlen(buffer);
@@ -95,7 +95,7 @@ tin_ltoa(
 	if (len > digits && digits >= 3) {
 		while (len >= digits) {
 			len -= 3;
-			i++;
+			++i;
 		}
 	}
 
@@ -136,21 +136,40 @@ my_strdup(
 	size_t len = 1;
 	void *ptr;
 
-	if (str)
+	if (str && *str)
 		len += strlen(str);
 
 	ptr = my_malloc(len);
 
-#	if 0 /* as my_malloc exits on error, ptr can't be NULL */
-	if (ptr == NULL)
-		return NULL;
-#	endif /* 0 */
-
 	memcpy(ptr, BlankIfNull(str), len);
 	return (char *) ptr;
 }
+
+
+char *
+my_strndup(
+	const char *str,
+	size_t n)
+{
+	char *ptr;
+	size_t slen;
+
+	if (str && *str) {
+		if ((slen = strlen(str)) < n)
+			n = slen;
+	} else
+		n = 0;
+
+	ptr = (char *) my_malloc(n + 1);
+	if (n)
+		memcpy(ptr, str, n);
+	*(ptr + n) = '\0';
+
+	return ptr;
+}
 #endif /* !USE_DMALLOC || (USE_DMALLOC && !HAVE_STRDUP) */
 
+
 /*
  * strtok that understands empty tokens
  * ie 2 adjacent delims count as two delims around a \0
@@ -197,6 +216,14 @@ my_strncpy(
 	const char *q,
 	size_t n)
 {
+#ifdef DEBUG
+	assert(((void) "p (dst) must not be NULL", p != NULL));
+	assert(((void) "n (len) must not be < 1", n > 0));
+#else
+	if (!p || n < 1)
+		return;
+#endif /* DEBUG */
+
 	while (n--) {
 		if (!*q || *q == '\n')
 			break;
@@ -222,8 +249,8 @@ strcasestr(
 	n = needle;
 	while (*haystack) {
 		if (my_tolower((unsigned char) *h) == my_tolower((unsigned char) *n)) {
-			h++;
-			n++;
+			++h;
+			++n;
 			if (!*n)
 				return (char *) haystack;
 		} else {
@@ -246,7 +273,7 @@ mystrcat(
 
 	while (*s) {
 		*((*t)++) = *s++;
-		len++;
+		++len;
 	}
 	**t = '\0';
 	return len;
@@ -329,6 +356,7 @@ strpbrk(
 }
 #endif /* !HAVE_STRPBRK */
 
+
 #ifndef HAVE_STRSTR
 /*
  * ANSI C strstr() - Uses Boyer-Moore algorithm.
@@ -370,7 +398,7 @@ strstr(
 	 * pattern, and i counts the amount of text remaining, not
 	 * including the part to be tested.
 	 */
-	p1--;
+	--p1;
 	p = (unsigned char *) pattern + p1;
 	t = (unsigned char *) text + p1;
 	i = textlen - patlen;
@@ -387,7 +415,9 @@ strstr(
 }
 #endif /* !HAVE_STRSTR */
 
-#ifndef HAVE_ATOL
+
+#if !defined(USE_LONG_ARTICLE_NUMBERS) && !defined(HAVE_ATOL)
+/* unused except in atoartnum-marco */
 /*
  * handrolled atol
  */
@@ -399,22 +429,24 @@ atol(
 
 	/* skip leading whitespace(s) */
 	while (*s && isspace((unsigned char) *s))
-		s++;
+		++s;
 
 	while (*s) {
 		if (isdigit((unsigned char) *s))
 			ret = ret * 10 + (*s - '0');
 		else
 			return -1;
-		s++;
+		++s;
 	}
 	return ret;
 }
-#endif /* !HAVE_ATOL */
+#endif /* !USE_LONG_ARTICLE_NUMBERS && !HAVE_ATOL */
+
 
 #ifndef HAVE_STRTOL
 #	define DIGIT(x) (isdigit((unsigned char) x) ? ((x) - '0') : (10 + my_tolower((unsigned char) x) - 'a'))
 #	define MBASE 36
+/* doesn't do ERANGE */
 long
 strtol(
 	const char *str,
@@ -471,6 +503,7 @@ OUT:
 #	undef MBASE
 #endif /* !HAVE_STRTOL */
 
+
 #if !defined(HAVE_STRCASECMP) || !defined(HAVE_STRNCASECMP)
 #	define FOLD_TO_UPPER(a)	(my_toupper((unsigned char) (a)))
 #endif /* !HAVE_STRCASECMP || !HAVE_STRNCASECMP */
@@ -494,6 +527,7 @@ strcasecmp(
 }
 #endif /* !HAVE_STRCASECMP */
 
+
 #ifndef HAVE_STRNCASECMP
 int
 strncasecmp(
@@ -510,6 +544,7 @@ strncasecmp(
 }
 #endif /* !HAVE_STRNCASECMP */
 
+
 #ifndef HAVE_STRSEP
 /*
  * strsep() is not mandatory in ANSI-C
@@ -556,10 +591,10 @@ str_trim(
 	if (!(s = strlen(string)))
 		return string;
 
-	/* remove training spaces */
+	/* remove trailing spaces */
 	ep = string + s - 1;
 	while (ep >= string && isspace((unsigned char) *ep))
-		ep--;
+		--ep;
 	*(ep + 1) = '\0';
 
 	/* skip leading space */
@@ -590,12 +625,12 @@ eat_tab(
 
 	while (*p1) {
 		if (*p1 == '\t' || *p1 == '\r' || *p1 == '\n') {
-			p1++;
+			++p1;
 		} else if (p1 != p2) {
 			*p2++ = *p1++;
 		} else {
-			p1++;
-			p2++;
+			++p1;
+			++p2;
 		}
 	}
 	if (p1 != p2)
@@ -608,12 +643,12 @@ eat_tab(
 #if defined(MULTIBYTE_ABLE) && !defined(NO_LOCALE)
 wchar_t *
 wexpand_tab(
-	wchar_t *wstr,
+	const wchar_t *wstr,
 	size_t tab_width)
 {
 	size_t cur_len = LEN, i = 0, tw;
 	wchar_t *wbuf = my_malloc(cur_len * sizeof(wchar_t));
-	wchar_t *wc = wstr;
+	const wchar_t *wc = wstr;
 
 	while (*wc) {
 		if (i > cur_len - (tab_width + 1)) {
@@ -626,7 +661,7 @@ wexpand_tab(
 				wbuf[i++] = (wchar_t) ' ';
 		} else
 			wbuf[i++] = *wc;
-		wc++;
+		++wc;
 	}
 	wbuf[i] = '\0';
 
@@ -671,7 +706,7 @@ expand_tab(
 				buf[i++] = ' ';
 		} else
 			buf[i++] = *c;
-		c++;
+		++c;
 	}
 	buf[i] = '\0';
 #endif /* MULTIBYTE_ABLE && !NO_LOCALE */
@@ -813,12 +848,10 @@ strrstr(
 	const char *str,
 	const char *pat)
 {
-	const char *ptr;
-	size_t slen, plen;
-
 	if ((str != NULL) && (pat != NULL)) {
-		slen = strlen(str);
-		plen = strlen(pat);
+		const char *ptr;
+		size_t slen = strlen(str);
+		size_t plen = strlen(pat);
 
 		if ((plen != 0) && (plen <= slen)) {
 			for (ptr = str + (slen - plen); ptr > str; --ptr) {
@@ -850,13 +883,11 @@ char2wchar_t(
 		if ((len = mbstowcs(NULL, test, 0)) == (size_t) (-1))
 			test[--pos] = '?';
 	}
-	/* shouldn't happen anymore */
-	if (len == (size_t) (-1)) {
+
+	if ((len = mbstowcs(NULL, test, 0)) == (size_t) (-1)) {
 		free(test);
 		return NULL;
 	}
-
-	/* if ((len = mbstowcs(NULL, test, 0) == (size_t) (-1)) { free(test); return NULL; } */
 	wstr = my_calloc(len + 1, sizeof(wchar_t));
 	/* pos = */ mbstowcs(wstr, test, len);
 	/* wstr[pos == (size_t) (-1) ? 0 : pos] = '\0'; */
@@ -1022,7 +1053,7 @@ abbr_groupname(
 
 	if (strlen(grpname) > len) {
 		if ((src = strchr(grpname, '.')) != NULL) {
-			dest++;
+			++dest;
 			tmplen = 1;
 
 			do {
@@ -1141,7 +1172,7 @@ wstrunc(
 		wchar_t *wtmp2, *tail;
 
 		if (tinrc.utf8_graphics)
-			tail = my_wcsdup(WTRUNC_TAIL);
+			tail = my_wcsdup((const wchar_t *) WTRUNC_TAIL);
 		else
 			tail = char2wchar_t(TRUNC_TAIL);
 
@@ -1532,7 +1563,7 @@ parse_format_string(
 	for (; *in; in++) {
 		if (*in != '%') {
 			*out++ = *in;
-			cnt++;
+			++cnt;
 			continue;
 		}
 		*out++ = *in++;
@@ -1592,9 +1623,9 @@ parse_format_string(
 					*d_fmt++ = *in;
 
 				*d_fmt = '\0';
-				in++;
+				++in;
 			} else {
-				out--;
+				--out;
 				*out++ = *in;
 				continue;
 			}
@@ -1605,7 +1636,7 @@ parse_format_string(
 				break;
 
 			case '%':
-				cnt++;
+				++cnt;
 				break;
 
 			case 'd':
@@ -1711,7 +1742,11 @@ parse_format_string(
 				/* Article marks */
 				if ((cCOLS > min_cols && cCOLS < max_cols) && !(flags & ART_MARKS) && (signal_context == cGroup || signal_context == cThread)) {
 					flags |= ART_MARKS;
+#if defined(MULTIBYTE_ABLE) && !defined(NO_LOCALE)
+					cnt += (art_mark_width + 2);
+#else
 					cnt += 3;
+#endif /* MULTIBYTE_ABLE && !NO_LOCALE */
 				} else
 					out -= 2;
 				break;
@@ -1793,7 +1828,7 @@ parse_format_string(
 			default:
 				out -= 2;
 				*out++ = *in;
-				cnt++;
+				++cnt;
 				break;
 		}
 	}
@@ -1868,7 +1903,7 @@ parse_format_string(
 		size_t *len_first, *len_last;
 		t_bool flags_first, flags_last;
 
-		len_cnt = (cCOLS - (int) cnt - 1) < 0 ? 0 : cCOLS - cnt - 1;
+		len_cnt = (int) (cCOLS - (int) cnt - 1 < 0 ? 0 : cCOLS - cnt - 1);
 
 		if (flags & (GRP_NAME | GRP_DESC))
 			fmt->len_grpname_max = (size_t) len_cnt;
@@ -1910,31 +1945,31 @@ parse_format_string(
 			len_last = &fmt->len_from;
 		}
 
-		len_tmp = len_cnt - (int) fmt->len_date_max < 0 ? 0 : len_cnt - fmt->len_date_max;
+		len_tmp = len_cnt - (int) fmt->len_date_max < 0 ? 0 : len_cnt - (int) fmt->len_date_max;
 		if (flags_first) {
 			if (!*len_last || *len_last > (size_t) len_tmp) {
 				if (flags_last) {
 					if (*len_first) {
-						len_tmp = len_cnt - fmt->len_date_max - *len_first;
-						*len_last = len_tmp < 0 ? 0 : (size_t) len_tmp;
+						len_tmp = len_cnt - (int) fmt->len_date_max - (int) *len_first;
+						*len_last = (size_t) (len_tmp < 0 ? 0 : len_tmp);
 					} else {
-						len_tmp = len_cnt - fmt->len_date_max;
-						*len_last = len_tmp < 0 ? 0 : (size_t) len_tmp / 3;
+						len_tmp = len_cnt - (int) fmt->len_date_max;
+						*len_last = (size_t) (len_tmp < 0 ? 0 : len_tmp / 3);
 					}
-					len_tmp = len_cnt - fmt->len_date_max - *len_last;
-					*len_first = len_tmp < 0 ? 0 : (size_t) len_tmp;
+					len_tmp = len_cnt - (int) fmt->len_date_max - (int) *len_last;
+					*len_first = (size_t) (len_tmp < 0 ? 0 : len_tmp);
 				} else {
-					len_tmp = len_cnt - fmt->len_date_max;
-					*len_last = len_tmp < 0 ? 0 : (size_t) len_tmp;
+					len_tmp = len_cnt - (int) fmt->len_date_max;
+					*len_last = (size_t) (len_tmp < 0 ? 0 : len_tmp);
 				}
 			} else {
 				if (flags_last) {
 					if (*len_first) {
-						len_tmp = len_cnt - fmt->len_date_max - *len_first;
-						*len_last = len_tmp < 0 ? 0 : MIN(*len_last, (size_t) len_tmp);
+						len_tmp = len_cnt - (int) fmt->len_date_max - (int) *len_first;
+						*len_last = (size_t) (len_tmp < 0 ? 0 : MIN(*len_last, (size_t) len_tmp));
 					}
-					len_tmp = len_cnt - fmt->len_date_max - *len_last;
-					*len_first = len_tmp < 0 ? 0 : MIN(*len_first, (size_t) len_tmp);
+					len_tmp = len_cnt - (int) fmt->len_date_max - (int) *len_last;
+					*len_first = (size_t) (len_tmp < 0 ? 0 : MIN(*len_first, (size_t) len_tmp));
 				}
 			}
 		} else if (flags_last && (!*len_first || *len_first > (size_t) len_tmp))
@@ -1945,6 +1980,8 @@ parse_format_string(
 
 #if defined(HAVE_LIBICUUC) && defined(MULTIBYTE_ABLE) && defined(HAVE_UNICODE_UBIDI_H) && !defined(NO_LOCALE)
 /*
+ * TODO: this is way to simple and needs to be reworked
+ *
  * prepare a string with bi-directional text for display
  * (converts from logical order to visual order)
  *
@@ -1987,7 +2024,7 @@ render_bidi(
 
 	ustr_len = u_strlen(ustr) + 1;
 	ustr_reordered = my_malloc(sizeof(UChar) * ustr_len);
-	ubidi_writeReordered(bidi_data, ustr_reordered, ustr_len, UBIDI_REMOVE_BIDI_CONTROLS|UBIDI_DO_MIRRORING, &status);
+	ubidi_writeReordered(bidi_data, ustr_reordered, ustr_len, UBIDI_REMOVE_BIDI_CONTROLS|UBIDI_DO_MIRRORING, &status); /* UBIDI_KEEP_BASE_COMBINING ? */
 	if (U_FAILURE(status)) {
 		ubidi_close(bidi_data);
 		free(ustr);
@@ -2027,11 +2064,17 @@ s2i(
 	signed int min,
 	signed int max)
 {
-	char *end;
+	const char *hex;
+	char *end = NULL;
 	signed long res;
 
 	errno = 0;
-	res = strtol(s, &end, 10);
+
+	/* allow 0[xX]n for hex, leading 0 is required */
+	if ((hex = strpbrk(s, "xX")) != NULL && hex > s && *(hex - 1) == '0')
+		res = strtol(s, &end, 16);
+	else
+		res = strtol(s, &end, 10);
 
 	if (res < min) {
 		res = min;
@@ -2043,8 +2086,10 @@ s2i(
 		errno = ERANGE;
 	}
 
-	if (s == end)
+	if (s == end || min > max) {
+		res = 0;
 		errno = EINVAL;
+	}
 
 	return ((signed int) res);
 }
