@@ -3,10 +3,10 @@
  *  Module    : version.c
  *  Author    : U. Janssen
  *  Created   : 2003-05-11
- *  Updated   : 2024-01-17
+ *  Updated   : 2024-11-12
  *  Notes     :
  *
- * Copyright (c) 2003-2024 Urs Janssen <urs@tin.org>
+ * Copyright (c) 2003-2025 Urs Janssen <urs@tin.org>
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -57,6 +57,10 @@
  *              RC_ERROR      3rd arg is not a dotted triple (usage error)
  *         ->file_version     version number in file as int
  *              (rc_majorv * 10000 + rc_minorv * 100 + rc_subv) or -1
+ *              TODO: switch to rc_majorv << 16 + rc_minorv << 8 + rc_subv
+ *                    (requires int_least32_t aka INT_WIDTH >= 32, but
+ *                     I guess a lot of the code silently assumes that
+ *                     anyway)
  *
  * usage:
  * if (upgrade == NULL && match_string(line, skip, NULL, 0)) {
@@ -66,11 +70,11 @@
  */
 struct t_version *
 check_upgrade(
-	char *line,
+	const char *line,
 	const char *skip,
 	const char *version)
 {
-	const char *p;
+	const char *p = line + strlen(skip);
 	int rc_majorv, rc_minorv, rc_subv; /* version numbers in the file */
 	int current_version, c_majorv, c_minorv, c_subv;	/* version numbers we require */
 	struct t_version *fversion = my_malloc(sizeof(struct t_version));
@@ -78,41 +82,53 @@ check_upgrade(
 	fversion->state = RC_ERROR;
 	fversion->file_version = -1;
 
-	/* we pre-checked that the beginning of line matches skip via
-	   match_string() before calling check_upgrade(), no need to
-	   strncmp() again.
+	/*
+	 * we pre-checked that the beginning of line matches skip via
+	 * match_string() before calling check_upgrade(), no need to
+	 * strncmp() again.
 	 */
-	p = line + strlen(skip);
-	if (!isdigit(*p))
+	if (!isdigit((unsigned char) *p))
 		return fversion;
-	rc_majorv = atoi(p);
-	while (isdigit(*p))
-		p++;
-	if (*p != '.' || !isdigit(*++p))
+	rc_majorv = s2i(p, 0, 99);
+	if (errno)
 		return fversion;
-	rc_minorv = atoi(p);
-	while (isdigit(*p))
-		p++;
-	if (*p != '.' || !isdigit(*++p))
+	while (isdigit((unsigned char) *p))
+		++p;
+	if (*p != '.' || !isdigit((unsigned char) *++p))
+		return fversion;
+	rc_minorv = s2i(p, 0, 99);
+	if (errno)
+		return fversion;
+	while (isdigit((unsigned char) *p))
+		++p;
+	if (*p != '.' || !isdigit((unsigned char) *++p))
+		return fversion;
+	rc_subv = s2i(p, 0, 99);
+	if (errno)
 		return fversion;
-	rc_subv = atoi(p);
 
 	fversion->file_version = rc_majorv * 10000 + rc_minorv * 100 + rc_subv;
 
 	p = version;
-	c_majorv = atoi(p);
-	while (isdigit(*p))
-		p++;
+	c_majorv = s2i(p, 0, 99);
+	if (errno)
+		return fversion;
+	while (isdigit((unsigned char) *p))
+		++p;
 	if (*p != '.')
 		return fversion;
 
-	c_minorv = atoi(++p);
-	while (isdigit(*p))
-		p++;
+	c_minorv = s2i(++p, 0, 99);
+	if (errno)
+		return fversion;
+	while (isdigit((unsigned char) *p))
+		++p;
 	if (*p != '.')
 		return fversion;
 
-	c_subv = atoi(++p);
+	c_subv = s2i(++p, 0, 99);
+	if (errno)
+		return fversion;
 	/* we don't care about trailing text */
 
 	current_version = c_majorv * 10000 + c_minorv * 100 + c_subv;
@@ -166,7 +182,8 @@ upgrade_prompt_quit(
 			free(upgrade);
 			if (fp)
 				fclose(fp);
-			tin_done(EXIT_FAILURE, NULL); /* TODO: leaks cmdargs from $TINRC */
+			no_write = TRUE;
+			tin_done(EXIT_FAILURE, _(txt_exiting));
 			/* NOTREACHED */
 			break;
 
