@@ -3,10 +3,10 @@
  *  Module    : xface.c
  *  Author    : Joshua Crawford & Drazen Kacar
  *  Created   : 2003-04-27
- *  Updated   : 2024-05-20
+ *  Updated   : 2025-02-05
  *  Notes     :
  *
- * Copyright (c) 2003-2024 Joshua Crawford <mortarn@softhome.net> & Drazen Kacar <dave@willfork.com>
+ * Copyright (c) 2003-2025 Joshua Crawford <mortarn@softhome.net> & Drazen Kacar <dave@willfork.com>
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -59,10 +59,10 @@ slrnface_start(
 {
 	char *fifo;
 	const char *ptr;
+	const char *m;
 	int n;
 	pid_t pid, pidst;
 	size_t pathlen;
-	struct utsname u;
 
 	if (tinrc.use_slrnface == FALSE)
 		return;
@@ -71,7 +71,7 @@ slrnface_start(
 	if (!is_xterm()) {
 #		ifdef DEBUG
 		if (debug & DEBUG_MISC)
-			error_message(2, "%s", _(txt_xface_error_no_xterm));
+			error_message(2, _(txt_xface_error_no_xterm));
 #		endif /* DEBUG */
 		return;
 	}
@@ -99,14 +99,13 @@ slrnface_start(
 		return;
 	}
 
-	uname(&u);
-	ptr = get_val("XDG_RUNTIME_DIR", get_val("HOME", ""));
+	ptr = get_val("XDG_RUNTIME_DIR", get_val("HOME", NULL));
 	/*
 	 * TODO:
 	 * - check if $XDG_RUNTIME_DIR is on a local filesystem and has secure permissions
 	 * <http://standards.freedesktop.org/basedir-spec/basedir-spec-latest.html>
 	 */
-	if (!strlen(ptr)) { /* TODO: mention XDG_RUNTIME_DIR in error message? */
+	if (!ptr) { /* TODO: mention XDG_RUNTIME_DIR in error message? */
 #	ifdef DEBUG
 		if (debug & DEBUG_MISC)
 			error_message(2, _(txt_xface_error_missing_env_var), "HOME");
@@ -146,12 +145,13 @@ slrnface_start(
 	}
 	free(fifo);
 	pid = getpid();
-	if ((n = snprintf(NULL, 0, "%s/.slrnfaces/%s.%ld", ptr, u.nodename, (long) pid)) < 0)
+	m = get_host_name();
+	if ((n = snprintf(NULL, 0, "%s/.slrnfaces/%s.%ld", ptr, m, (long) pid)) < 0)
 		return;
 	pathlen = (size_t) n + 1;
 	fifo = my_malloc(pathlen);
-	if (snprintf(fifo, pathlen, "%s/.slrnfaces/%s.%ld", ptr, u.nodename, (long) pid) != n) {
-		error_message(2, "%s", _(txt_xface_error_construct_fifo_name));
+	if (snprintf(fifo, pathlen, "%s/.slrnfaces/%s.%ld", ptr, m, (long) pid) != n) {
+		error_message(2, _(txt_xface_error_construct_fifo_name));
 		unlink(fifo);
 		free(fifo);
 		return;
@@ -186,56 +186,58 @@ slrnface_start(
 			break;
 
 		default:
+#ifdef EINTR
 			do {
 				pidst = waitpid(pid, &n, 0);
 			} while (pidst == -1 && errno == EINTR);
+#else
+			pidst = waitpid(pid, &n, 0);
+#endif /* EINTR */
 			if (!WIFEXITED(n))
 				error_message(2, _(txt_xface_error_exited_abnormal), n);
 			else {
-				const char *message;
-
 				switch (WEXITSTATUS(n)) {
 					case 0:	/* All fine, open the pipe */
 						if ((slrnface_fd = open(fifo, O_WRONLY, (S_IRUSR|S_IWUSR))) != -1) {
 							WRITE_FACE_FD("start\n");
-							message = NULL;
+							m = NULL;
 						} else
-							message = _(txt_xface_msg_cannot_open_fifo);
+							m = _(txt_xface_msg_cannot_open_fifo);
 						break;
 
 					case 1:
-						message = _(txt_xface_msg_cannot_connect_display);
+						m = _(txt_xface_msg_cannot_connect_display);
 						break;
 
 					case 2:
-						message = _(txt_xface_msg_windowid_not_found);
+						m = _(txt_xface_msg_windowid_not_found);
 						break;
 
 					case 3:
-						message = _(txt_xface_msg_no_controlling_terminal);
+						m = _(txt_xface_msg_no_controlling_terminal);
 						break;
 
 					case 4:
-						message = _(txt_xface_msg_no_width_and_height_avail);
+						m = _(txt_xface_msg_no_width_and_height_avail);
 						break;
 
 					case 5:
-						message = _(txt_xface_msg_cannot_open_fifo);
+						m = _(txt_xface_msg_cannot_open_fifo);
 						break;
 
 					case 6:
-						message = _(txt_xface_msg_fork_failed);
+						m = _(txt_xface_msg_fork_failed);
 						break;
 
 					case 10:
-						message = _(txt_xface_msg_executable_not_found);
+						m = _(txt_xface_msg_executable_not_found);
 						break;
 
 					default:
-						message = _(txt_unknown_error);
+						m = _(txt_unknown_error);
 				}
-				if (message)
-					error_message(2, _(txt_xface_error_finally_failed), message);
+				if (m)
+					error_message(2, _(txt_xface_error_finally_failed), m);
 			}
 	}
 	unlink(fifo);
@@ -257,7 +259,7 @@ slrnface_stop(
 
 void
 slrnface_display_xface(
-	char *face)
+	const char *face)
 {
 	if (slrnface_fd < 0)
 		return;
@@ -306,6 +308,7 @@ slrnface_show_xface(
 	WRITE_FACE_FD("show\n");
 }
 
+
 #else
 static void no_xface(void);	/* proto-type */
 static void
