@@ -3,10 +3,10 @@
  *  Module    : xref.c
  *  Author    : I. Lea & H. Brugge
  *  Created   : 1993-07-01
- *  Updated   : 2024-01-31
+ *  Updated   : 2025-06-14
  *  Notes     :
  *
- * Copyright (c) 1993-2024 Iain Lea <iain@bricbrac.de>
+ * Copyright (c) 1993-2025 Iain Lea <iain@bricbrac.de>
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -65,13 +65,18 @@ open_overview_fmt_fp(
 		if (!*nntp_caps.over_cmd)
 			return (FILE *) 0;
 
-		if ((nntp_caps.type == CAPABILITIES && nntp_caps.list_overview_fmt) || nntp_caps.type != CAPABILITIES)
+		if (nntp_caps.list_overview_fmt)
 			return (nntp_command("LIST OVERVIEW.FMT", OK_GROUPS, NULL, 0));
 
 		return (FILE *) 0;
 	}
 #endif /* NNTP_ABLE */
+
+#ifndef NNTP_ONLY
 	return (fopen(overviewfmt_file, "r"));
+#else
+	return (FILE *) 0;
+#endif /* !NNTP_ONLY */
 }
 
 
@@ -102,7 +107,7 @@ overview_xref_support(
 				debug_print_file("NNTP", "<<<%s%s", logtime(), ptr);
 #endif /* DEBUG && NNTP_ABLE */
 
-			fields++;
+			++fields;
 
 			/* expand overview fmt array */
 			if (fields >= res_fields) {
@@ -263,7 +268,7 @@ overview_xref_support(
 		TIN_FCLOSE(fp);
 	}
 
-	fields++;
+	++fields;
 	/* resize */
 	ofmt = my_realloc(ofmt, sizeof(struct t_overview_fmt) * (fields + 1));
 
@@ -317,9 +322,10 @@ overview_xref_support(
 	 *         (use nntp_caps.hdr_cmd in txt_warn_xref_not_supported ?)
 	 *       - if server doesn't mention XREF in LIST HEADERS
 	 */
-	if (read_news_via_nntp && !supported)
-		wait_message(2, _(txt_warn_xref_not_supported));
-
+	if (read_news_via_nntp && !supported) {
+		if (*serverrc.disabled_nntp_cmds && !strstr(serverrc.disabled_nntp_cmds, "LIST OVERVIEW.FMT") && !strstr(serverrc.disabled_nntp_cmds, "HDR"))
+			wait_message(2, _(txt_warn_xref_not_supported));
+	}
 	return supported;
 }
 
@@ -348,18 +354,18 @@ art_mark_xref_read(
 
 	/* skip sitename */
 	while (*xref_ptr != ' ' && *xref_ptr != '\t' && *xref_ptr)
-		xref_ptr++;
+		++xref_ptr;
 
 	/*
 	 * tokenize each pair and update that newsgroup if it is in my_group[].
 	 */
 	forever {
 		while (*xref_ptr == ' ' || *xref_ptr == '\t')
-			xref_ptr++;
+			++xref_ptr;
 
 		groupname = xref_ptr;
 		while (*xref_ptr != ':' && *xref_ptr != ' ' && *xref_ptr != '\t' && *xref_ptr)
-			xref_ptr++;
+			++xref_ptr;
 
 		if (*xref_ptr != ':')
 			break;
@@ -368,7 +374,7 @@ art_mark_xref_read(
 		if ((artnum = atoartnum(xref_ptr)) <= 0)
 			break;
 		while (isdigit((unsigned char) *xref_ptr))
-			xref_ptr++;
+			++xref_ptr;
 
 		if (&ptr[1] == xref_ptr)
 			break;
@@ -425,12 +431,11 @@ NSETRNG1(
 	t_artnum low,
 	t_artnum high)
 {
-	t_artnum i;
-
 	assert(((void) "NSETRNG1() failed. bitmap == NULL", bitmap != NULL));
 
 	if (high >= low) {
 		if (NOFFSET(high) == NOFFSET(low)) {
+			t_artnum i;
 			for (i = low; i <= high; i++) {
 				NSET1(bitmap, i);
 			}
@@ -439,7 +444,7 @@ NSETRNG1(
 			if (NOFFSET(high) > NOFFSET(low) + 1)
 				memset(&bitmap[NOFFSET(low) + 1], NBITSON, (size_t) (NOFFSET(high) - NOFFSET(low) - 1));
 
-			BIT_OR(bitmap, high, ~ (NBITNEG1 << NBITIDX(high)));
+			BIT_OR(bitmap, high, ~(NBITNEG1 << NBITIDX(high)));
 		}
 	}
 }
@@ -454,12 +459,11 @@ NSETRNG0(
 	t_artnum low,
 	t_artnum high)
 {
-	t_artnum i;
-
 	assert(((void) "NSETRNG0() failed. bitmap == NULL", bitmap != NULL));
 
 	if (high >= low) {
 		if (NOFFSET(high) == NOFFSET(low)) {
+			t_artnum i;
 			for (i = low; i <= high; i++) {
 				NSET0(bitmap, i);
 			}
