@@ -1,10 +1,10 @@
-#! /usr/bin/perl -w
+#! /usr/bin/perl
 #
 # reads an article on STDIN, mails any copies if required,
 # signs the article and posts it.
 #
 #
-# Copyright (c) 2002-2024 Urs Janssen <urs@tin.org>,
+# Copyright (c) 2002-2026 Urs Janssen <urs@tin.org>,
 #                         Marc Brockschmidt <marc@marcbrockschmidt.de>
 #
 # Redistribution and use in source and binary forms, with or without
@@ -39,6 +39,7 @@
 #       - add pid to pgptmpf to allow multiple simultaneous instances
 #       - check for /etc/nntpserver (and /etc/news/server)
 #       - add $PGPOPTS, $PGPPATH and $GNUPGHOME support
+#       - add $AUTHORCOPY (pnews(1)) support?
 #       - cleanup and remove duplicated code
 #       - quote inpupt properly before passing to shell
 #       - $ENV{'NEWSHOST'} / $ENV{'NNTPSERVER'} and $ENV{'NNTPPORT'}
@@ -50,6 +51,7 @@
 #         for gpg?
 #       - option to break long header lines?
 #       - option to trim References
+#       - option to foce connection via AF_INET6 (-6)
 #       ...
 #
 # cmd-line options used in other inews:
@@ -57,12 +59,20 @@
 #    -C accepted for historic reasons and errors out
 # inews (inn)
 #    -P don't add Sender
+# inews (Eric S. Raymond; 1989)
+#    -C create grp
+#    -d Distribution:
+#    -p [file] ; run rnews mode
+#    -M moderator option
+#    -v print returned ID
+
+require 5.004;
 
 use strict;
 use warnings;
 
 # version Number
-my $version = "1.1.64";
+my $version = "1.1.71";
 
 my %config;
 
@@ -107,21 +117,22 @@ $config{'canlock-algorithm'}	= 'sha1'; 	# Digest algorithm used for cancel-lock
 
 # $config{'ignore-headers'} = '';		# headers to be ignored during signing
 
-$config{'pgp-sign-headers'} = ['From', 'Newsgroups', 'Subject', 'Control',
-	'Supersedes', 'Followup-To', 'Date', 'Injection-Date', 'Sender', 'Approved',
-	'Message-ID', 'Reply-To', 'Cancel-Key', 'Also-Control',
-	'Distribution'];
-$config{'pgp-order-headers'} = ['from', 'newsgroups', 'subject', 'control',
-	'supersedes', 'followup-To', 'date', 'injection-date', 'organization',
-	'lines', 'sender', 'approved', 'distribution', 'message-id',
-	'references', 'reply-to', 'mime-version', 'content-type',
-	'content-transfer-encoding', 'summary', 'keywords', 'cancel-lock',
-	'cancel-key', 'also-control', 'x-pgp', 'user-agent'];
+$config{'pgp-sign-headers'} = [
+	'From', 'Newsgroups', 'Subject', 'Control', 'Supersedes', 'Followup-To',
+	'Date', 'Injection-Date', 'Sender', 'Approved', 'Message-ID', 'Reply-To',
+	'Cancel-Key', 'Also-Control', 'Distribution' ];
+$config{'pgp-order-headers'} = [
+	'from', 'newsgroups', 'subject', 'control', 'supersedes', 'followup-To',
+	'date', 'injection-date', 'organization', 'lines', 'sender', 'approved',
+	'distribution', 'message-id', 'references', 'reply-to', 'mime-version',
+	'content-type', 'content-transfer-encoding', 'summary', 'keywords',
+	'cancel-lock', 'cancel-key', 'also-control', 'x-pgp', 'user-agent' ];
 
 ################################################################################
 
 use Getopt::Long qw(GetOptions);
 use Net::NNTP;
+use IO::Socket qw(AF_INET PF_INET);
 use Time::Local;
 use Term::ReadLine;
 
@@ -180,10 +191,10 @@ $config{'nntp-server'} = $ENV{'NEWSHOST'} if ($ENV{'NEWSHOST'});
 $config{'nntp-server'} = $ENV{'NNTPSERVER'} if ($ENV{'NNTPSERVER'});
 $config{'nntp-port'} = $ENV{'NNTPPORT'} if ($ENV{'NNTPPORT'});
 
-# Get options:
-$Getopt::Long::ignorecase=0;
-$Getopt::Long::bundling=1;
-GetOptions('A|V|W|h|headers' => [], # do nothing
+# Get options
+Getopt::Long::Configure ("bundling", "no_ignore_case");
+my $oret = GetOptions(
+	'A|V|W|h|headers'	=> [], # do nothing
 	'debug|D|N'	=> \$config{'debug'},
 	'port|p=i'	=> \$config{'nntp-port'},
 	'no-sign|X'	=> \$config{'no-sign'},
@@ -211,6 +222,8 @@ GetOptions('A|V|W|h|headers' => [], # do nothing
 	'references|F=s'	=> \$config{'references'},
 	'organization|o=s'	=> \$config{'organization'},
 	'path|x=s'	=> \$config{'path'},
+	'timeout|T=i'	=> \$config{'timeout'},
+	'ipv4|4'	=> \$config{'ipv4'},
 	'help|H'	=> \$config{'help'},
 	'transform'	=> \$config{'transform'},
 	'verbose|v'	=> \$config{'verbose'},
@@ -218,10 +231,7 @@ GetOptions('A|V|W|h|headers' => [], # do nothing
 	'man'	=>	\$config{'man'}
 );
 
-foreach (@ARGV) {
-	print STDERR "Unknown argument $_.\n";
-	usage();
-}
+usage() unless $oret;
 
 if ($config{'version'}) {
 	version();
@@ -232,13 +242,13 @@ usage() if ($config{'help'});
 
 # not listed in usage() or man-page as it may not work
 if ($config{'man'}) {
-	eval "use Pod::Usage";
-	if ($@) {
+	if (eval { require Pod::Usage;1; } != 1) {
 		$config{'man'} = 0;
 		print STDERR "Unknown option: man.\n";
 		usage();
 	} else {
-		pod2usage(-verbose => 2, -exit => 0);
+		use Pod::Usage;
+		pod2usage(-verbose => 3, -exit => 0);
 	}
 }
 
@@ -250,6 +260,7 @@ if ($config{'ssl'}) {
 		$config{'ssl'} = 0;
 	}
 }
+
 # and now adjust default port depending on SSL requested and
 # available or not
 if ($config{'ssl'}) {
@@ -258,8 +269,8 @@ if ($config{'ssl'}) {
 	$config{'nntp-port'} = 119 if ($config{'nntp-port'} == 563);
 }
 
-my $sha_mod = undef;
 # Cancel-Locks require some more modules
+my $sha_mod = undef;
 if ($config{'canlock-secret'} && !$config{'no-canlock'}) {
 	$config{'canlock-algorithm'} = lc($config{'canlock-algorithm'});
 	# we support sha1, sha256 and sha512, fallback to sha1 if something else is given
@@ -279,7 +290,7 @@ if ($config{'canlock-secret'} && !$config{'no-canlock'}) {
 			eval "use $_";
 			if ($@ || !defined($sha_mod)) {
 				$config{'no-canlock'} = 1;
-				warn "Cancel-Locks disabled: Can't locate ".$_."\n" if ($config{'debug'} || $config{'verbose'});
+				warn "Cancel-Locks disabled: Can't locate ".$_." (".__FILE__.":".__LINE__.")\n" if ($config{'debug'} || $config{'verbose'});
 				last;
 			}
 		}
@@ -288,7 +299,7 @@ if ($config{'canlock-secret'} && !$config{'no-canlock'}) {
 			eval "use $_";
 			if ($@) {
 	 			$config{'no-canlock'} = 1;
-				warn "Cancel-Locks disabled: Can't locate ".$_."\n" if ($config{'debug'} || $config{'verbose'});
+				warn "Cancel-Locks disabled: Can't locate ".$_." (".__FILE__.":".__LINE__.")\n" if ($config{'debug'} || $config{'verbose'});
 				last;
 			}
 		}
@@ -297,7 +308,7 @@ if ($config{'canlock-secret'} && !$config{'no-canlock'}) {
 			eval "use $_";
 			if ($@) {
 	 			$config{'no-canlock'} = 1;
-				warn "Cancel-Locks disabled: Can't locate ".$_."\n" if ($config{'debug'} || $config{'verbose'});
+				warn "Cancel-Locks disabled: Can't locate ".$_." (".__FILE__.":".__LINE__.")\n" if ($config{'debug'} || $config{'verbose'});
 				last;
 			}
 		}
@@ -330,6 +341,7 @@ if (${config{'ignore-headers'}}) {
 		@{$config{'pgp-sign-headers'}} = map {lc($_) eq lc($hdr) ? () : $_} @{$config{'pgp-sign-headers'}};
 	}
 }
+
 # Read the message and split the header
 readarticle(\%Header, \@Body);
 
@@ -373,7 +385,7 @@ foreach ('DISTRIBUTION', 'ORGANIZATION') {
 
 # overwrite headers if specified via cmd-line
 foreach ('Approved', 'Control', 'Distribution', 'Expires',
-	'From', 'Followup-To', 'Message-ID', 'Newsgroups',' Reply-To',
+	'From', 'Followup-To', 'Message-ID', 'Newsgroups', 'Reply-To',
 	'Subject', 'References', 'Organization') {
 	next if (!defined($config{lc($_)}));
 	chomp($Header{lc($_)} = $_ . ": " . $config{lc($_)});
@@ -431,15 +443,38 @@ if (!$config{'nntp-pass'}) {
 	if (-r (glob("~/.newsauth"))[0]) {
 		open (my $NEWSAUTH, '<', (glob("~/.newsauth"))[0]) or die("Can't open ~/.newsauth: $!");
 		while ($l = <$NEWSAUTH>) {
+			next if ($l =~ m/^([#\s]|$)/);
 			chomp $l;
-			next if ($l =~ m/(^[#\s]|)/);
-			($server, $pass, $user) = split(/\s+\b/, $l);
+			$user = $pass = $server = undef;
+			if ($l =~ m/^
+				(\S+)\s+				# server
+				("(?:[^"]+)"|(?:\S+))	# password
+				\s+("(?:[^"]+)"|(?:\S+))	# user
+			/x) {
+				$server = $1;
+				$pass = $2;
+				$user = $3;
+				if ($pass =~ m/^"([^"]+)"/) { # strip enclising "
+					$pass = $1;
+				}
+				if ($user =~ m/^"([^"]+)"/) { # likewise
+					$user = $1;
+				}
+			} else { # server passwrd
+				if ($l =~ m/^(\S+)\s+("(?:[^"]+)"|(?:\S+))/) {
+					$server = $1;
+					$pass = $2;
+					if ($pass =~ m/^"([^"]+)"/) { # likewise
+						$pass = $1;
+					}
+				}
+			}
 			last if ($server =~ m/\Q$config{'nntp-server'}\E/);
 		}
 		close($NEWSAUTH);
 		if ($pass && $server =~ m/\Q$config{'nntp-server'}\E/) {
 			$config{'nntp-pass'} = $pass;
-			$config{'nntp-user'} = $user || getlogin || getpwuid($<) || $ENV{USER};
+			$config{'nntp-user'} = $user || getlogin || getpwuid($<) || $ENV{USER} || $ENV{LOGNAME};
 		} else {
 			$pass = $user = "";
 		}
@@ -457,7 +492,7 @@ if (!$config{'nntp-pass'}) {
 			close($NNTPAUTH);
 			if ($pass && $server =~ m/\Q$config{'nntp-server'}\E/) {
 				$config{'nntp-pass'} = $pass;
-				$config{'nntp-user'} = $user || getlogin || getpwuid($<) || $ENV{USER};
+				$config{'nntp-user'} = $user || getlogin || getpwuid($<) || $ENV{USER} || $ENV{LOGNAME};
 			}
 		}
 	}
@@ -477,10 +512,10 @@ if (!($config{'no-sign'} && $config{'no-canlock'})) {
 
 	if (!defined($Header{'message-id'})) {
 		my $hname;
-		eval "use Sys::Hostname";
-		if ($@) {
+		if (eval { require Sys::Hostname;1; } != 1) {
 			chomp($hname = `hostname`);
 		} else {
+			use Sys::Hostname;
 			$hname = hostname();
 		}
 		my ($hostname,) = gethostbyname($hname);
@@ -553,7 +588,7 @@ if (! $config{'no-sign'}) {
 
 # exit with error if neither $Newsgroups nor any of $To, $Cc or $Bcc are set
 my $required = 0;
-foreach ('Newsgroups', 'To,', 'Cc', 'Bcc') {
+foreach ('Newsgroups', 'To', 'Cc', 'Bcc') {
 	$required++ if (defined($Header{lc($_)}));
 	last if $required;
 }
@@ -616,7 +651,7 @@ sub readarticle {
 				$currentheader = lc($1);
 				$$HeaderR{$currentheader} = "$1: $2";
 				$l = length($_);
-				print $1 . ":-header exceeds line length limit " . $l . " > " . $config{'max-header-length'} . " octets.\n" if (($config{'verbose'} || $config{'debug'}) && length($_) > $config{'max-header-length'});
+				print "" . $1 . ":-header exceeds line length limit " . $l . " > " . $config{'max-header-length'} . " octets.\n" if (($config{'verbose'} || $config{'debug'}) && length($_) > $config{'max-header-length'});
 			} elsif (m/^[ \t]/o) {
 				$$HeaderR{$currentheader} .= $_;
 				$l = length($_);
@@ -644,7 +679,7 @@ sub getdate {
 	my $mm = ($time[1]<10) ? "0".$time[1] : $time[1];
 	my $hh = ($time[2]<10) ? "0".$time[2] : $time[2];
 	my $day = $time[3];
-	my $month = ($time[4]+1 < 10) ? "0".($time[4]+1) : $time[4]+1;
+#	my $month = ($time[4]+1 < 10) ? "0".($time[4]+1) : $time[4]+1; # 01...12; unused
 	my $monthN = ("Jan","Feb","Mar","Apr","May","Jun","Jul","Aug","Sep","Oct","Nov","Dec")[$time[4]];
 	my $wday = ("Sun","Mon","Tue","Wed","Thu","Fri","Sat")[$time[6]];
 	my $year = $time[5] + 1900;
@@ -656,7 +691,7 @@ sub getdate {
 	}
 	my $offseth = int($offset/3600);
 	my $offsetm = int(($offset - $offseth*3600)/60);
-	my $tz = sprintf ("%s%0.2d%0.2d", $sign, $offseth, $offsetm);
+	my $tz = sprintf("%s%0.2d%0.2d", $sign, $offseth, $offsetm);
 	return "$wday, $day $monthN $year $hh:$mm:$ss $tz";
 }
 
@@ -674,11 +709,16 @@ sub AuthonNNTP {
 		Reader 	=> 1,
 		Debug 	=> $config{'debug'},
 		Port 	=> $config{'nntp-port'},
+		Timeout => $config{'timeout'},
+		Domain  => ($config{'ipv4'} ? AF_INET : undef),
 		SSL 	=> $config{'ssl'},
 		SSL_verify_mode => 0
 	) or die("$0: Can't connect to ".$config{'nntp-server'}.":".$config{'nntp-port'}."!\n");
-	if ($config{'ssl'} && $config{'debug'}) {
-		printf("SSL_fingerprint: %s %s\n", split(/\$/, $Server->get_fingerprint));
+	if ($config{'debug'}) {
+		printf("Connected to   : ".$Server->peerhost.":".$Server->peerport." [%s]\n", ($Server->sockdomain == PF_INET) ? "IPv4" : "IPv6");
+		if ($config{'ssl'}) {
+		    printf("SSL_fingerprint: %s %s\n", split(/\$/, $Server->get_fingerprint));
+		}
 	}
 	my $ServerMsg = $Server->message();
 	my $ServerCod = $Server->code();
@@ -860,11 +900,11 @@ sub savearticle {
 # 	- $MessageRef: A reference to an array containing the whole message.
 sub signarticle {
 	my ($HeaderR, $BodyR) = @_;
-	my (@pgphead, @pgpbody, $pgphead, $pgpbody, $signheaders, @signheaders);
+	my (@pgp_head, @pgp_body, @sign_headers, $pgphead, $pgpbody, $signheaders);
 
 	foreach (@{$config{'pgp-sign-headers'}}) {
 		if (defined($$HeaderR{lc($_)}) && $$HeaderR{lc($_)} =~ m/^[^\s:]+: .+/o) {
-			push @signheaders, $_;
+			push @sign_headers, $_;
 		}
 	}
 
@@ -873,16 +913,16 @@ sub signarticle {
 	# Delete and create the temporary pgp-Files
 	unlink $config{'pgptmpf'}.".txt";
 	unlink $config{'pgptmpf'}.".txt.asc";
-	$signheaders = join(",", @signheaders);
+	$signheaders = join(",", @sign_headers);
 
 	$pgphead = "X-Signed-Headers: $signheaders\n";
-	foreach my $header (@signheaders) {
+	foreach my $header (@sign_headers) {
 		if ($$HeaderR{lc($header)} =~ m/^[^\s:]+: (.+?)\n?$/so) {
 			$pgphead .= $header.": ".$1."\n";
 		}
 	}
 
-	unless (substr($pgpbody,-1,1)=~ /\n/ ) {$pgpbody.="\n"};
+	unless (substr($pgpbody, -1, 1) =~ /\n/) {$pgpbody .= "\n"};
 	open(my $FH, '>', $config{'pgptmpf'} . ".txt") or die("$0: can't open ".$config{'pgptmpf'}.": $!\n");
 	print $FH $pgphead, "\n", $pgpbody;
 	print $FH "\n" if ($config{'pgp-version'} =~ m/GPG/io); # workaround a pgp/gpg incompatibility - should IMHO be fixed in pgpverify
@@ -937,27 +977,27 @@ sub signarticle {
 	my $tmppgpheader = $config{'pgpheader'} . ": " . $$HeaderR{$config{'pgpheader'}};
 	delete $$HeaderR{$config{'pgpheader'}};
 
-	@pgphead = ();
+	@pgp_head = ();
 	foreach my $header (@{$config{'pgp-order-headers'}}) {
 		if ($$HeaderR{$header} && $$HeaderR{$header} ne "\n") {
-			push(@pgphead, "$$HeaderR{$header}");
+			push(@pgp_head, "$$HeaderR{$header}");
 			delete $$HeaderR{$header};
 		}
 	}
 
 	foreach my $header (keys %$HeaderR) {
 		if ($$HeaderR{$header} && $$HeaderR{$header} ne "\n") {
-			push(@pgphead, "$$HeaderR{$header}");
+			push(@pgp_head, "$$HeaderR{$header}");
 			delete $$HeaderR{$header};
 		}
 	}
 
-	push @pgphead, ("X-PGP-Hash: " . $config{'digest-algo'} . "\n") if (defined($config{'digest-algo'}));
-	push @pgphead, ("X-PGP-Key: " . $config{'pgp-signer'} . "\n"), $tmppgpheader;
+	push @pgp_head, ("X-PGP-Hash: " . $config{'digest-algo'} . "\n") if (defined($config{'digest-algo'}));
+	push @pgp_head, ("X-PGP-Key: " . $config{'pgp-signer'} . "\n"), $tmppgpheader;
 	undef $tmppgpheader;
 
-	@pgpbody = split(/$/m, $pgpbody);
-	my @pgpmessage = (@pgphead, "\n", @pgpbody);
+	@pgp_body = split(/$/m, $pgpbody);
+	my @pgpmessage = (@pgp_head, "\n", @pgp_body);
 	return \@pgpmessage;
 }
 
@@ -994,10 +1034,10 @@ sub buildcancelkey {
 # Returns:
 # 	- $cancel_lock: The calculated cancel-lock.
 sub buildcancellock {
-	my ($cancel_key, $sha_mod) = @_;
+	my ($cancel_key, $sha_module) = @_;
 	my $cancel_lock;
 	if ($config{'canlock-algorithm'} eq 'sha1') {
-		if ($sha_mod =~ m/SHA1/) {
+		if ($sha_module =~ m/SHA1/) {
 			$cancel_lock = MIME::Base64::encode(Digest::SHA1::sha1($cancel_key, ''), '');
 		} else {
 			$cancel_lock = MIME::Base64::encode(Digest::SHA::sha1($cancel_key, ''), '');
@@ -1011,13 +1051,14 @@ sub buildcancellock {
 }
 
 sub version {
-	print $pname." ".$version."\n";
+	print "".$pname." ".$version."\n";
 	return;
 }
 
 sub usage {
 	version();
 	print "Usage: ".$pname." [OPTS] < article\n";
+	print "  -4         force connecting via IPv4\n";
 	print "  -a string  set Approved:-header to string\n";
 	print "  -c string  set Control:-header to string\n";
 	print "  -d string  set Distribution:-header to string\n";
@@ -1043,6 +1084,7 @@ sub usage {
 	print "  -O         do not add Organization:-header\n";
 	print "  -R         disallow control messages\n";
 	print "  -S         do not append " . $config{'sig-path'} . "\n";
+	print "  -T seconds set connection timeout to seconds\n";
 	print "  -X         do not sign article\n";
 	print "  -Y         force authentication on connect\n";
 	print " --canlock-algorithm string\n";
@@ -1050,7 +1092,7 @@ sub usage {
 	print " --ssl       use NNTPS (via port 563) if available\n";
 	print " --transform convert <CR><LF> to <LF>\n";
 	print " --version   show version\n";
-	printf ("\nAvailable tinewsrc-vars: %s\n", join(", ",sort keys %config)) if ($config{'verbose'} || $config{'debug'});
+	printf("\nAvailable tinewsrc-vars: %s\n", join(", ", sort keys %config)) if ($config{'verbose'} || $config{'debug'});
 	exit 0;
 }
 
@@ -1088,6 +1130,11 @@ X<tinews, command-line options>
 
 =over 4
 
+=item -B<4> | --B<ipv4>
+X<-4> X<--ipv4>
+
+Force connecting via IPv4 to the remote NNTP server.
+
 =item -B<a> C<Approved> | --B<approved> C<Approved>
 X<-a> X<--approved>
 
@@ -1124,7 +1171,7 @@ Date, Injection-Date, Sender, Approved, Message-ID, Reply-To,
 Cancel-Key, Also-Control and Distribution.
 
 Some of them may be altered on the Server (i.e. Cancel-Key) which would
-invalid the signature, this option can be used the exclude such headers
+invalid the signature, this option can be used to exclude such headers
 if required.
 
 =item -B<m> C<Message-ID> | --B<message-id> C<Message-ID>
@@ -1225,6 +1272,11 @@ X<-s> X<--no-signature>
 
 Do not append F<$HOME/.signature>.
 
+=item -B<T> C<seconds> | --B<timeout> C<seconds>
+X<-T> X<--timeout>
+
+Override the connection timeout setting. Default is 120 seconds.
+
 =item -B<X> | --B<no-sign>
 X<-X> X<--no-sign>
 
