@@ -3,15 +3,12 @@
 # reads a tin filter file with wildmat filters on STDIN, converts it to
 # regexp filters and returns it on STDOUT
 #
-# 2020-11-10 <urs@tin.org>
+# 2025-05-22 <urs@tin.org>
 #
 # NOTE: don't use w2r.pl on regexp filters
 #
 # for case optimization of your regexp filters use opt-case.pl, i.e.:
 # w2r.pl < wildmat-filter-file | opt-case.pl > regexp-filter-file
-#
-# for joining regexp filters with the same group= and score= use
-# joinf.pl (not written yet)
 
 # perl 5 is needed for lookahead assertions and perl < 5.004 is know
 # to be buggy
@@ -21,8 +18,9 @@ use strict;
 use warnings;
 
 # version Number
-# $VERSION = "0.2.8";
+# $VERSION = "0.2.9";
 
+my $line;
 while (defined($line = <>)) {
 	chomp $line;
 
@@ -48,12 +46,12 @@ while (defined($line = <>)) {
 
 # turns a wildmat into a regexp
 sub w2p {
-	local ($wild)  = @_;	# input line
+	my ($wild)  = @_;	# input line
 	my $cchar = "";		# current char
 	my $lchar = "";		# last char
 	my $reg = "";		# translated char
-	$bmode = 0;		# inside [] ?
-	$rval = "";		# output line
+	my $bmode = 0;		# inside [] ?
+	my $rval = "";		# output line
 
 	# break line into chars
 	while ($wild =~ s/(.)//) {
@@ -84,10 +82,15 @@ sub w2p {
 			$reg =~ s/\$/\\\$/o;	# quote $
 		}
 
-		# if last char was a qute, current char can't be a meta
-		if ($lchar =~ m/\\/o || $bmode != 0) {
+		if ($bmode == 1 && $lchar =~ m/\[/o && $cchar =~ m/!/o) {
 			$reg = $cchar;
-			$cchar =~ s/\\//o;	# skip 2nd \\ inside []
+			$reg =~ s/!/^/o;
+		} else {
+			# if last char was a quote, current char can't be a meta
+			if ($lchar =~ m/\\/o || $bmode != 0) {
+				$reg = $cchar;
+				$cchar =~ s/\\//o;	# skip 2nd \\ inside []
+			}
 		}
 
 		$lchar = $cchar;	# store last char
@@ -97,16 +100,41 @@ sub w2p {
 	# common abbreviations
 	# TODO: make this global
 	# replace [0-9] with [\d] in the first []
-	# replace [a-zA-Z0-9_] with [\w] in the first []
-	# replace [a-zA-Z0-9] with [^\W_] in the first []
+	# replace [a-zA-Z0-9_] and permutations with [\w] in the first []
+	# replace [a-zA-Z0-9] and permutations with [^\W_] in the first []
 	# replace [a-zA-Z] with [^\W\d_] in the first []
 	$rval =~ s/^([^\[]*)\[0-9\]/$1\[\\d\]/o;
+	$rval =~ s/([^\[]*)\[0-9a-za-z_\]/$1\[\\w\]/io;
+	$rval =~ s/([^\[]*)\[_a-z0-9a-z\]/$1\[\\w\]/io;
+	$rval =~ s/([^\[]*)\[_a-za-z0-9\]/$1\[\\w\]/io;
+	$rval =~ s/([^\[]*)\[a-z0-9_a-z\]/$1\[\\w\]/io;
+	$rval =~ s/([^\[]*)\[a-z0-9a-z_\]/$1\[\\w\]/io;
+	$rval =~ s/([^\[]*)\[a-z0-9a-z_\]/$1\[\\w\]/io;
+	$rval =~ s/([^\[]*)\[a-z_0-9a-z\]/$1\[\\w\]/io;
+	$rval =~ s/([^\[]*)\[a-z_a-z0-9\]/$1\[\\w\]/io;
 	$rval =~ s/([^\[]*)\[a-za-z0-9_\]/$1\[\\w\]/io;
+	$rval =~ s/([^\[]*)\[a-za-z_0-9\]/$1\[\\w\]/io;
+	$rval =~ s/([^\[]*)\[0-9a-za-z\]/$1\[^\\W_\]/io;
+	$rval =~ s/([^\[]*)\[a-z0-9a-z\]/$1\[^\\W_\]/io;
 	$rval =~ s/([^\[]*)\[a-za-z0-9\]/$1\[^\\W_\]/io;
 	$rval =~ s/([^\[]*)\[a-za-z\]/$1\[^\\W\\d_\]/io;
+	# negated classes
+	$rval =~ s/^([^\[]*)\[\^0-9\]/$1\[\\D\]/o;
+	$rval =~ s/([^\[]*)\[\^0-9a-za-z_\]/$1\[\\W\]/io;
+	$rval =~ s/([^\[]*)\[\^_a-z0-9a-z\]/$1\[\\W\]/io;
+	$rval =~ s/([^\[]*)\[\^_a-za-z0-9\]/$1\[\\W\]/io;
+	$rval =~ s/([^\[]*)\[\^a-z0-9_a-z\]/$1\[\\W\]/io;
+	$rval =~ s/([^\[]*)\[\^a-z0-9a-z_\]/$1\[\\W\]/io;
+	$rval =~ s/([^\[]*)\[\^a-z0-9a-z_\]/$1\[\\W\]/io;
+	$rval =~ s/([^\[]*)\[\^a-z_0-9a-z\]/$1\[\\W\]/io;
+	$rval =~ s/([^\[]*)\[\^a-z_a-z0-9\]/$1\[\\W\]/io;
+	$rval =~ s/([^\[]*)\[\^a-za-z0-9_\]/$1\[\\W\]/io;
+	$rval =~ s/([^\[]*)\[\^a-za-z_0-9\]/$1\[\\W\]/io;
+	$rval =~ s/([^\[]*)\[\^0-9a-za-z\]/$1\[^\\w_\]/io;
+	$rval =~ s/([^\[]*)\[\^a-z0-9a-z\]/$1\[^\\w_\]/io;
+	$rval =~ s/([^\[]*)\[\^a-za-z0-9\]/$1\[^\\w_\]/io;
 
 	# optimizations
-	#
 	# add ^-anchor if needed
 	$rval =~ s/^(?!\.\*)(.*)/\^$1/o;
 	# add $-anchor if needed
@@ -136,7 +164,7 @@ converts it to regexp filters and returns it on STDOUT.
 
 =head1 NOTES
 
-Don't use B<w2r.pl> on regexp filter files
+Don't use B<w2r.pl> on regexp filter files.
 
 =head1 AUTHOR
 
